@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2012-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -61,18 +61,17 @@ static const struct starpu_data_copy_methods custom_copy_data_methods_s =
 	.ram_to_opencl       = copy_ram_to_opencl,
 	.opencl_to_ram       = copy_opencl_to_ram,
 	.opencl_to_opencl    = copy_opencl_to_opencl,
-        .ram_to_opencl_async = copy_ram_to_opencl_async,
+	.ram_to_opencl_async = copy_ram_to_opencl_async,
 	.opencl_to_ram_async = copy_opencl_to_ram_async,
 #endif
 };
 
 static void     register_custom_handle(starpu_data_handle_t handle,
-				       unsigned home_node,
+				       int home_node,
 				       void *data_interface);
 static starpu_ssize_t  allocate_custom_buffer_on_node(void *data_interface_,
 					       unsigned dst_node);
 static void*    custom_to_pointer(void *data_interface, unsigned node);
-static int      custom_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void     free_custom_buffer_on_node(void *data_interface, unsigned node);
 static size_t   custom_interface_get_size(starpu_data_handle_t handle);
 static uint32_t footprint_custom_interface_crc32(starpu_data_handle_t handle);
@@ -93,7 +92,6 @@ static struct starpu_data_interface_ops interface_custom_ops =
 	.register_data_handle  = register_custom_handle,
 	.allocate_data_on_node = allocate_custom_buffer_on_node,
 	.to_pointer            = custom_to_pointer,
-	.pointer_is_inside     = custom_pointer_is_inside,
 	.free_data_on_node     = free_custom_buffer_on_node,
 	.copy_methods          = &custom_copy_data_methods_s,
 	.get_size              = custom_interface_get_size,
@@ -107,13 +105,13 @@ static struct starpu_data_interface_ops interface_custom_ops =
 };
 
 static void
-register_custom_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+register_custom_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct custom_data_interface *custom_interface;
 	custom_interface = (struct custom_data_interface *) data_interface;
 
-	unsigned node;
-	unsigned nnodes = starpu_memory_nodes_get_count();
+	int node;
+	int nnodes = starpu_memory_nodes_get_count();
 	for (node = 0; node < nnodes; node++)
 	{
 		struct custom_data_interface *local_interface =
@@ -193,11 +191,14 @@ static void free_custom_buffer_on_node(void *data_interface, unsigned node)
 	size_t size = custom_interface->nx * custom_interface->ops->cpu_elemsize;
 
 	starpu_free_on_node(node, (uintptr_t) custom_interface->cpu_ptr, size);
+	custom_interface->cpu_ptr = NULL;
 #ifdef STARPU_USE_CUDA
 	starpu_free_on_node(node, (uintptr_t) custom_interface->cuda_ptr, size);
+	custom_interface->cuda_ptr = NULL;
 #endif
 #ifdef STARPU_USE_OPENCL
 	starpu_free_on_node(node, (uintptr_t) custom_interface->opencl_ptr, size);
+	custom_interface->opencl_ptr = NULL;
 #endif
 }
 
@@ -224,31 +225,6 @@ custom_to_pointer(void *data, unsigned node)
 	}
 }
 
-static int
-custom_pointer_is_inside(void *data, unsigned node, void *ptr)
-{
-	struct custom_data_interface *data_interface = data;
-
-	switch(starpu_node_get_kind(node))
-	{
-		case STARPU_CPU_RAM:
-			return (char*) ptr >= (char*) data_interface->cpu_ptr &&
-				(char*) ptr < (char*) data_interface->cpu_ptr + data_interface->nx * data_interface->ops->cpu_elemsize;
-#ifdef STARPU_USE_CUDA
-		case STARPU_CUDA_RAM:
-			return (char*) ptr >= (char*) data_interface->cuda_ptr &&
-				(char*) ptr < (char*) data_interface->cuda_ptr + data_interface->nx * data_interface->ops->cuda_elemsize;
-#endif
-#ifdef STARPU_USE_OPENCL
-		case STARPU_OPENCL_RAM:
-			return (char*) ptr >= (char*) data_interface->opencl_ptr &&
-				(char*) ptr < (char*) data_interface->opencl_ptr + data_interface->nx * data_interface->ops->opencl_elemsize;
-#endif
-		default:
-			assert(0);
-	}
-}
-
 static size_t custom_interface_get_size(starpu_data_handle_t handle)
 {
 	size_t size;
@@ -283,7 +259,7 @@ custom_get_nx(starpu_data_handle_t handle)
 
 
 void custom_data_register(starpu_data_handle_t *handle,
-				 unsigned home_node,
+				 int home_node,
 				 void *ptr,
 				 uint32_t nx,
 				 struct starpu_multiformat_data_interface_ops *format_ops)
@@ -301,10 +277,6 @@ void custom_data_register(starpu_data_handle_t *handle,
 		.ops = format_ops
 	};
 
-	if (interface_custom_ops.interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
-	{
-		interface_custom_ops.interfaceid = starpu_data_interface_get_next_id();
-	}
 	starpu_data_register(handle, home_node, &custom, &interface_custom_ops);
 }
 
