@@ -1,7 +1,6 @@
 ! StarPU --- Runtime system for heterogeneous multicore architectures.
 !
-! Copyright (C) 2017                                     CNRS
-! Copyright (C) 2016                                     Inria
+! Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 !
 ! StarPU is free software; you can redistribute it and/or modify
 ! it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +28,8 @@ program nf_matrix
         type(c_ptr) :: dh_mb    ! a pointer for the 'mb' vector data handle
         integer(c_int) :: err   ! return status for fstarpu_init
         integer(c_int) :: ncpu  ! number of cpus workers
+        real(c_double) :: start_time ! start clock in usec
+        real(c_double) :: end_time   ! end clock in usec
 
         allocate(ma(5,6))
         do i=1,5
@@ -57,6 +58,9 @@ program nf_matrix
                 stop 77
         end if
 
+        ! collect the start clock time
+        start_time = fstarpu_timing_now()
+
         ! allocate an empty codelet structure
         cl_mat = fstarpu_codelet_allocate()
 
@@ -88,7 +92,7 @@ program nf_matrix
         !     . . .
         !     C_NULL_PTR
         !   )/
-        call fstarpu_insert_task((/ cl_mat, FSTARPU_R, dh_ma, FSTARPU_RW, dh_mb, C_NULL_PTR /))
+        call fstarpu_task_insert((/ cl_mat, FSTARPU_R, dh_ma, FSTARPU_RW, dh_mb, C_NULL_PTR /))
 
         ! wait for task completion
         call fstarpu_task_wait_for_all()
@@ -102,10 +106,15 @@ program nf_matrix
         ! free codelet structure
         call fstarpu_codelet_free(cl_mat)
 
+        ! collect the start clock time
+        end_time = fstarpu_timing_now()
+
         ! shut StarPU down
         call fstarpu_shutdown()
 
         deallocate(mb)
         deallocate(ma)
 
+        print "(es 10.3)", end_time - start_time
+
 end program nf_matrix
