@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +20,7 @@
 #include <stdio.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -33,35 +32,49 @@ extern "C"
 /**
    Start recording tasks (resets stats). \p deps tells whether
    dependencies should be recorded too (this is quite expensive)
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_start(int deps, int prio);
 
 /**
    Stop recording tasks
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_stop(void);
 
 /**
    Emit the DAG that was recorded on \p output.
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_print_dot(FILE *output);
 
 /**
    Get theoretical upper bound (in ms) (needs glpk support detected by
    configure script). It returns 0 if some performance models are not
-   calibrated.
+   calibrated. \p integer permits to choose between integer solving
+   (which takes a long time but is correct), and relaxed solving
+   (which provides an approximate solution).
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_compute(double *res, double *integer_res, int integer);
 
 /**
    Emit the Linear Programming system on \p output for the recorded
    tasks, in the lp format
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_print_lp(FILE *output);
 
 /**
    Emit the Linear Programming system on \p output for the recorded
    tasks, in the mps format
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_print_mps(FILE *output);
 
@@ -70,6 +83,8 @@ void starpu_bound_print_mps(FILE *output);
    upper bound. \p integer permits to choose between integer solving
    (which takes a long time but is correct), and relaxed solving
    (which provides an approximate solution).
+
+   See \ref TheoreticalLowerBoundOnExecutionTime for more details.
 */
 void starpu_bound_print(FILE *output, int integer);
 
