@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Joris Pablo
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
@@ -32,6 +32,9 @@ extern "C" {
    @{
 */
 
+/**
+   todo
+*/
 struct starpu_fxt_codelet_event
 {
 	char symbol[2048];
@@ -42,7 +45,9 @@ struct starpu_fxt_codelet_event
 	float time;
 };
 
-/** Store informations related to clock synchronizations: mainly the offset to apply to each time. */
+/**
+   Store information related to clock synchronizations: mainly the offset to apply to each time.
+*/
 struct starpu_fxt_mpi_offset
 {
 	uint64_t local_time_start; /**< node time for the barrier at the beginning of the program */
@@ -53,6 +58,9 @@ struct starpu_fxt_mpi_offset
 			   (can be 0, 1 or 2) */
 };
 
+/**
+   todo
+*/
 struct starpu_fxt_options
 {
 	unsigned per_task_colour;
