@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,10 @@
 #ifdef STARPU_SIMGRID
 #ifndef main
 #define main starpu_main
+#ifdef __cplusplus
+extern "C" int starpu_main(int argc, char *argv[]);
+extern "C" int starpu_main(int argc, char **argv);
+#endif
 #endif
 #endif
 
