@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +23,10 @@
 #include <starpu_thread.h>
 #include <errno.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #if !(defined(_MSC_VER) && !defined(BUILDING_STARPU))
 /*
  * Encapsulation of the starpu_pthread_create_* functions.
@@ -518,4 +522,8 @@ static STARPU_INLINE int _starpu_pthread_cond_timedwait(starpu_pthread_cond_t *c
 	while (0)
 #endif /* _MSC_VER */
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* __STARPU_THREAD_UTIL_H__ */
