@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2015  CNRS
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,9 +23,21 @@ extern unsigned size;
 extern unsigned nblocks;
 extern unsigned nbigblocks;
 extern unsigned noprio;
+extern unsigned check;
 extern unsigned display;
 extern int dblockx;
 extern int dblocky;
+extern unsigned long checkpoint_period;
+extern int checkpoint_enabled;
+
+enum submission
+{
+	TRIANGLES,
+	COLUMNS,
+	ANTIDIAGONALS,
+	PRIOS,
+};
+extern enum submission submission;
 
 void parse_args(int argc, char **argv, int nodes);
 
