@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,13 @@ extern int _starpu_mpi_tag;
 #define _STARPU_MPI_TAG_DATA      _starpu_mpi_tag+1
 #define _STARPU_MPI_TAG_SYNC_DATA _starpu_mpi_tag+2
 
+#ifdef STARPU_USE_MPI_FT
+#define _STARPU_MPI_TAG_CP_ACK    _starpu_mpi_tag+3
+#define _STARPU_MPI_TAG_CP_RCVRY  _starpu_mpi_tag+4
+#define _STARPU_MPI_TAG_EXT_DATA  _starpu_mpi_tag+5
+#define _STARPU_MPI_TAG_CP_INFO    _starpu_mpi_tag+6
+#endif // STARPU_USE_MPI_FT
+
 enum _starpu_envelope_mode
 {
 	_STARPU_MPI_ENVELOPE_DATA=0,
@@ -67,7 +74,7 @@ struct _starpu_mpi_req_backend
 	unsigned to_destroy:1;
 	struct _starpu_mpi_req *internal_req;
 	struct _starpu_mpi_early_data_handle *early_data_handle;
-     	UT_hash_handle hh;
+	UT_hash_handle hh;
 };
 
 #endif // STARPU_USE_MPI_MPI
