@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,6 +17,7 @@
 #include <stdlib.h>
 #include "starpu_mpi_nmad_backend.h"
 #include <starpu_mpi_private.h>
+#include "starpu_mpi_nmad.h"
 
 #ifdef STARPU_USE_MPI_NMAD
 
@@ -51,16 +52,24 @@ void _starpu_mpi_nmad_backend_request_init(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_CALLOC(req->backend, 1, sizeof(struct _starpu_mpi_req_backend));
 	piom_cond_init(&req->backend->req_cond, 0);
+	req->backend->data_request = NM_SR_REQUEST_NULL;
+	req->backend->posted = 0;
+	req->backend->has_received_data = 0;
+	req->backend->finalized = 0;
+	req->backend->to_destroy = 0;
+	_starpu_spin_init(&req->backend->finalized_to_destroy_lock);
 }
 
-void _starpu_mpi_nmad_backend_request_fill(struct _starpu_mpi_req *req, MPI_Comm comm, int is_internal_req)
+void _starpu_mpi_nmad_backend_request_fill(struct _starpu_mpi_req *req, int is_internal_req STARPU_ATTRIBUTE_UNUSED)
 {
-	nm_mpi_nmad_dest(&req->backend->session, &req->backend->gate, comm, req->node_tag.node.rank);
+	/* this function gives session and gate: */
+	nm_mpi_nmad_dest(&req->backend->session, &req->backend->gate, req->node_tag.node.comm, req->node_tag.node.rank);
 }
 
 void _starpu_mpi_nmad_backend_request_destroy(struct _starpu_mpi_req *req)
 {
 	piom_cond_destroy(&(req->backend->req_cond));
+	_starpu_spin_destroy(&req->backend->finalized_to_destroy_lock);
 	free(req->backend);
 }
 
@@ -82,15 +91,29 @@ void _starpu_mpi_nmad_backend_comm_register(MPI_Comm comm)
 
 struct _starpu_mpi_backend _mpi_backend =
 {
- 	._starpu_mpi_backend_init = _starpu_mpi_nmad_backend_init,
- 	._starpu_mpi_backend_shutdown = _starpu_mpi_nmad_backend_shutdown,
+	._starpu_mpi_backend_init = _starpu_mpi_nmad_backend_init,
+	._starpu_mpi_backend_shutdown = _starpu_mpi_nmad_backend_shutdown,
 	._starpu_mpi_backend_reserve_core = _starpu_mpi_nmad_backend_reserve_core,
 	._starpu_mpi_backend_request_init = _starpu_mpi_nmad_backend_request_init,
 	._starpu_mpi_backend_request_fill = _starpu_mpi_nmad_backend_request_fill,
 	._starpu_mpi_backend_request_destroy = _starpu_mpi_nmad_backend_request_destroy,
 	._starpu_mpi_backend_data_clear = _starpu_mpi_nmad_backend_data_clear,
 	._starpu_mpi_backend_data_register = _starpu_mpi_nmad_backend_data_register,
-	._starpu_mpi_backend_comm_register = _starpu_mpi_nmad_backend_comm_register
+	._starpu_mpi_backend_comm_register = _starpu_mpi_nmad_backend_comm_register,
+
+	._starpu_mpi_backend_progress_init = _starpu_mpi_progress_init,
+	._starpu_mpi_backend_progress_shutdown = _starpu_mpi_progress_shutdown,
+//#ifdef STARPU_SIMGRID
+//	._starpu_mpi_backend_wait_for_initialization = _starpu_mpi_wait_for_initialization,
+//#endif
+
+	._starpu_mpi_backend_barrier = _starpu_mpi_barrier,
+	._starpu_mpi_backend_wait_for_all = _starpu_mpi_wait_for_all,
+	._starpu_mpi_backend_wait = _starpu_mpi_wait,
+	._starpu_mpi_backend_test = _starpu_mpi_test,
+
+	._starpu_mpi_backend_isend_size_func = _starpu_mpi_isend_func,
+	._starpu_mpi_backend_irecv_size_func = _starpu_mpi_irecv_func,
 };
 
 #endif /* STARPU_USE_MPI_NMAD*/
