@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,22 +19,29 @@
 #include "helper.h"
 
 #ifdef STARPU_QUICK_CHECK
-#  define NITER	16
+#  define NITER	4
 #else
 #  define NITER	2048
 #endif
 
-#define BIGSIZE	128
-#define SIZE	64
+#define BIGSIZE	32
+#define SIZE	8
 
 int main(int argc, char **argv)
 {
 	int ret, rank, size;
 	int mpi_init;
+	struct starpu_conf conf;
 
 	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
 
-	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
@@ -74,8 +81,8 @@ int main(int argc, char **argv)
 		}
 
 		starpu_block_data_register(&block_handle, STARPU_MAIN_RAM,
-			(uintptr_t)block, BIGSIZE, BIGSIZE*BIGSIZE,
-			SIZE, SIZE, SIZE, sizeof(float));
+					   (uintptr_t)block, BIGSIZE, BIGSIZE*BIGSIZE,
+					   SIZE, SIZE, SIZE, sizeof(float));
 	}
 	else if (rank == 1)
 	{
@@ -83,16 +90,17 @@ int main(int argc, char **argv)
 		assert(block);
 
 		starpu_block_data_register(&block_handle, STARPU_MAIN_RAM,
-			(uintptr_t)block, SIZE, SIZE*SIZE,
-			SIZE, SIZE, SIZE, sizeof(float));
+					   (uintptr_t)block, SIZE, SIZE*SIZE,
+					   SIZE, SIZE, SIZE, sizeof(float));
 	}
 
 	if (rank == 0)
 	{
+		MPI_Status status;
+
 		ret = starpu_mpi_send(block_handle, 1, 0x42, MPI_COMM_WORLD);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
 
-		MPI_Status status;
 		ret = starpu_mpi_recv(block_handle, 1, 0x1337, MPI_COMM_WORLD, &status);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
 
@@ -113,6 +121,7 @@ int main(int argc, char **argv)
 	else if (rank == 1)
 	{
 		MPI_Status status;
+
 		ret = starpu_mpi_recv(block_handle, 0, 0x42, MPI_COMM_WORLD, &status);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
 
@@ -134,14 +143,15 @@ int main(int argc, char **argv)
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
 	}
 
-	FPRINTF(stdout, "Rank %d is done\n", rank);
-	fflush(stdout);
-
 	if (rank == 0 || rank == 1)
 	{
 		starpu_data_unregister(block_handle);
 		free(block);
 	}
+
+	FPRINTF(stdout, "Rank %d is done\n", rank);
+	fflush(stdout);
+
 	starpu_mpi_shutdown();
 
 	if (!mpi_init)
