@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2015-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -46,11 +46,17 @@ int main(int argc, char **argv)
 	struct starpu_task *task;
 	starpu_data_handle_t handles[3];
 	int mpi_init;
+	struct starpu_conf conf;
 
 	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
 	(void)mpi_init;
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
 
-	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, NULL);
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
@@ -106,13 +112,13 @@ int main(int argc, char **argv)
 	FPRINTF_MPI(stderr, "Task %p\n", task);
 	if (rank == 1)
 	{
-		STARPU_ASSERT_MSG(task, "Task should be executed by rank 1\n");
+		STARPU_ASSERT_MSG(task, "Task should be executed by rank 1");
 		task->destroy = 0;
 		starpu_task_destroy(task);
 	}
 	else
 	{
-		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 1\n");
+		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 1");
 	}
 
 	// Force the execution on node 1
@@ -123,13 +129,13 @@ int main(int argc, char **argv)
 	FPRINTF_MPI(stderr, "Task %p\n", task);
 	if (rank == 1)
 	{
-		STARPU_ASSERT_MSG(task, "Task should be executed by rank 1\n");
+		STARPU_ASSERT_MSG(task, "Task should be executed by rank 1");
 		task->destroy = 0;
 		starpu_task_destroy(task);
 	}
 	else
 	{
-		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 1\n");
+		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 1");
 	}
 
 	// Let StarPU choose the node
@@ -140,13 +146,13 @@ int main(int argc, char **argv)
 	FPRINTF_MPI(stderr, "Task %p\n", task);
 	if (rank == 0)
 	{
-		STARPU_ASSERT_MSG(task, "Task should be executed by rank 0\n");
+		STARPU_ASSERT_MSG(task, "Task should be executed by rank 0");
 		task->destroy = 0;
 		starpu_task_destroy(task);
 	}
 	else
 	{
-		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 2\n");
+		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 2");
 	}
 
 	// Let StarPU choose the node
@@ -156,13 +162,13 @@ int main(int argc, char **argv)
 	FPRINTF_MPI(stderr, "Task %p\n", task);
 	if (rank == 0)
 	{
-		STARPU_ASSERT_MSG(task, "Task should be executed by rank 0\n");
+		STARPU_ASSERT_MSG(task, "Task should be executed by rank 0");
 		task->destroy = 0;
 		starpu_task_destroy(task);
 	}
 	else
 	{
-		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 0\n");
+		STARPU_ASSERT_MSG(task == NULL, "Task should be executed by rank 0");
 	}
 
 	starpu_data_unregister(handles[0]);
