@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -50,7 +51,7 @@ int _starpu_silent;
 
 void _starpu_util_init(void)
 {
-	_starpu_silent = starpu_get_env_number_default("STARPU_SILENT", 0);
+	_starpu_silent = starpu_getenv_number_default("STARPU_SILENT", 0);
 	STARPU_HG_DISABLE_CHECKING(_starpu_silent);
 }
 
@@ -480,7 +481,7 @@ int _starpu_check_mutex_deadlock(starpu_pthread_mutex_t *mutex)
 	if (ret == EBUSY)
 		return 0;
 
-	STARPU_ASSERT (ret != EDEADLK);
+	STARPU_ASSERT(ret != EDEADLK);
 
 	return 1;
 }
@@ -517,10 +518,50 @@ char *_starpu_get_home_path(void)
 	return path;
 }
 
+#pragma weak starpu_mpi_world_rank
+extern int starpu_mpi_world_rank(void);
+
 void _starpu_gethostname(char *hostname, size_t size)
 {
+	char *force_mpi_hostnames = starpu_getenv("STARPU_MPI_HOSTNAMES");
 	char *forced_hostname = starpu_getenv("STARPU_HOSTNAME");
-	if (forced_hostname && forced_hostname[0])
+
+	if (force_mpi_hostnames && force_mpi_hostnames[0])
+	{
+		char *host=NULL, *srv_hosts;
+		srv_hosts = strdup(force_mpi_hostnames);
+		int rank;
+
+#ifdef STARPU_HAVE_DARWIN
+		_STARPU_DISP("Detection of MPI rank disabled on darwin, assuming rank is 0\n");
+		rank = 0;
+#else
+		if (starpu_mpi_world_rank)
+		{
+			rank = starpu_mpi_world_rank();
+		}
+		else
+		{
+			_STARPU_DISP("StarPU-MPI unavailable, the rank of this process is 0\n");
+			rank = 0;
+		}
+#endif
+
+		if (force_mpi_hostnames != NULL)
+		{
+			host = strtok(srv_hosts, " ");
+			while (rank-->0 && (host = strtok(NULL, " ")));
+			if(rank>=0)
+			{
+				_STARPU_MSG("Missing hostnames in STARPU_MPI_HOSTNAMES\n");
+				STARPU_ABORT();
+			}
+		}
+		snprintf(hostname, size-1, "%s", host);
+		free(srv_hosts);
+		hostname[size-1] = 0;
+	}
+	else if (forced_hostname && forced_hostname[0])
 	{
 		snprintf(hostname, size-1, "%s", forced_hostname);
 		hostname[size-1] = 0;
@@ -582,7 +623,7 @@ char *starpu_getenv(const char *str)
 	worker = _starpu_get_local_worker_key();
 
 	if (worker && worker->worker_is_initialized)
-		_STARPU_DISP( "getenv should not be called from running workers, only for main() or worker initialization, since it is not reentrant\n");
+		_STARPU_DISP("getenv should not be called from running workers, only for main() or worker initialization, since it is not reentrant\n");
 #endif
 #endif
 	return getenv(str);
@@ -708,7 +749,7 @@ int starpu_get_env_size_default(const char *str, int defval)
 
 void starpu_display_bindings(void)
 {
-#ifdef STARPU_HAVE_HWLOC
+#if defined(STARPU_HAVE_HWLOC) && !defined(STARPU_SIMGRID)
 	int hwloc_ret = system("hwloc-ps -a -t -c");
 	if (hwloc_ret)
 	{
