@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -50,7 +50,9 @@
 
 #ifdef STARPU_HAVE_HWLOC
 #include <hwloc.h>
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
+#ifdef STARPU_HAVE_NVML_H
+#define nvmlDeviceGetPciInfo _starpu_nvmlDeviceGetPciInfo
+#define nvmlDeviceGetUUID _starpu_nvmlDeviceGetUUID
 #include <hwloc/nvml.h>
 #endif
 #ifndef HWLOC_API_VERSION
@@ -87,8 +89,10 @@ struct dev_timing
 	double latency_dtoh;
 };
 
-static double bandwidth_matrix[STARPU_MAXNODES][STARPU_MAXNODES]; /* MB/s */
-static double latency_matrix[STARPU_MAXNODES][STARPU_MAXNODES]; /* µs */
+static double raw_bandwidth_matrix[STARPU_MAXNODES][STARPU_MAXNODES];	/* MB/s, indexed by device ids */
+static double bandwidth_matrix[STARPU_MAXNODES][STARPU_MAXNODES];	/* MB/s, indexed by memory nodes */
+static double raw_latency_matrix[STARPU_MAXNODES][STARPU_MAXNODES];	/* µs, indexed by devices ids */
+static double latency_matrix[STARPU_MAXNODES][STARPU_MAXNODES];		/* µs, indexed by memory nodes */
 static unsigned was_benchmarked = 0;
 #ifndef STARPU_SIMGRID
 static unsigned ncpus = 0;
@@ -290,9 +294,10 @@ static void measure_bandwidth_between_host_and_dev_on_numa_with_cuda(int dev, un
 	{
 		/* we use STARPU_MAIN_RAM */
 		_STARPU_MALLOC(h_buffer, size);
-		cudaHostRegister((void *)h_buffer, size, 0);
 	}
 
+	cudaHostRegister((void *)h_buffer, size, 0);
+
 	if (STARPU_UNLIKELY(cures)) STARPU_CUDA_REPORT_ERROR(cures);
 
 	/* hack to avoid third party libs to rebind threads */
@@ -1278,7 +1283,7 @@ static int load_bus_latency_file_content(void)
 				return 0;
 			}
 
-			latency_matrix[src][dst] = latency;
+			raw_latency_matrix[src][dst] = latency;
 
 			/* Look out for \t\n */
 			n = getc(f);
@@ -1290,7 +1295,7 @@ static int load_bus_latency_file_content(void)
 
 		/* No more values, take NAN */
 		for (; dst < STARPU_MAXNODES; dst++)
-			latency_matrix[src][dst] = NAN;
+			raw_latency_matrix[src][dst] = NAN;
 
 		while (n == '\t')
 		{
@@ -1329,7 +1334,7 @@ static int load_bus_latency_file_content(void)
 	/* No more values, take NAN */
 	for (; src < STARPU_MAXNODES; src++)
 		for (dst = 0; dst < STARPU_MAXNODES; dst++)
-			latency_matrix[src][dst] = NAN;
+			raw_latency_matrix[src][dst] = NAN;
 
 	return 1;
 }
@@ -1359,6 +1364,7 @@ static double search_bus_best_latency(int src, enum starpu_node_kind type, int h
 
 static void write_bus_latency_file_content(void)
 {
+	enum starpu_node_kind type;
 	unsigned src, dst, maxnode;
 	/* Boundaries to check if src or dst are inside the interval */
 	unsigned b_low, b_up;
@@ -1385,11 +1391,15 @@ static void write_bus_latency_file_content(void)
 	_starpu_fftruncate(f, 0);
 
 	fprintf(f, "# ");
-	for (dst = 0; dst < STARPU_MAXNODES; dst++)
-		fprintf(f, "to %u\t\t", dst);
+	for (type = STARPU_CPU_RAM; type < STARPU_NRAM; type++)
+	{
+		for (dst = 0; dst < nmem[type]; dst++)
+		{
+			fprintf(f, "to %s %d\t", _starpu_node_get_prefix(type), dst);
+		}
+	}
 	fprintf(f, "\n");
 
-	enum starpu_node_kind type;
 	maxnode = 0;
 	for (type = STARPU_CPU_RAM; type < STARPU_NRAM; type++)
 		maxnode += nmem[type];
@@ -1561,7 +1571,7 @@ static int load_bus_bandwidth_file_content(void)
 #endif
 			}
 
-			bandwidth_matrix[src][dst] = bandwidth;
+			raw_bandwidth_matrix[src][dst] = bandwidth;
 
 			/* Look out for \t\n */
 			n = getc(f);
@@ -1573,7 +1583,7 @@ static int load_bus_bandwidth_file_content(void)
 
 		/* No more values, take NAN */
 		for (; dst < STARPU_MAXNODES; dst++)
-			bandwidth_matrix[src][dst] = NAN;
+			raw_bandwidth_matrix[src][dst] = NAN;
 
 		while (n == '\t')
 		{
@@ -1612,7 +1622,7 @@ static int load_bus_bandwidth_file_content(void)
 	/* No more values, take NAN */
 	for (; src < STARPU_MAXNODES; src++)
 		for (dst = 0; dst < STARPU_MAXNODES; dst++)
-			latency_matrix[src][dst] = NAN;
+			raw_bandwidth_matrix[src][dst] = NAN;
 
 	return 1;
 }
@@ -1642,6 +1652,7 @@ static double search_bus_best_timing(int src, enum starpu_node_kind type, int ht
 
 static void write_bus_bandwidth_file_content(void)
 {
+	enum starpu_node_kind type;
 	unsigned src, dst, maxnode;
 	unsigned b_low, b_up;
 	FILE *f;
@@ -1662,11 +1673,15 @@ static void write_bus_bandwidth_file_content(void)
 	_starpu_fftruncate(f, 0);
 
 	fprintf(f, "# ");
-	for (dst = 0; dst < STARPU_MAXNODES; dst++)
-		fprintf(f, "to %u\t\t", dst);
+	for (type = STARPU_CPU_RAM; type < STARPU_NRAM; type++)
+	{
+		for (dst = 0; dst < nmem[type]; dst++)
+		{
+			fprintf(f, "to %s %d\t", _starpu_node_get_prefix(type), dst);
+		}
+	}
 	fprintf(f, "\n");
 
-	enum starpu_node_kind type;
 	maxnode = 0;
 	for (type = STARPU_CPU_RAM; type < STARPU_NRAM; type++)
 		maxnode += nmem[type];
@@ -2288,13 +2303,13 @@ static hwloc_obj_t get_hwloc_cuda_obj(hwloc_topology_t topology, unsigned devid)
 		if (res)
 			return res;
 
-#if defined(STARPU_HAVE_LIBNVIDIA_ML) && !defined(STARPU_USE_CUDA0) && !defined(STARPU_USE_CUDA1)
+#if defined(STARPU_HAVE_NVML_H) && !defined(STARPU_USE_CUDA0) && !defined(STARPU_USE_CUDA1)
 		nvmlDevice_t nvmldev = _starpu_cuda_get_nvmldev(&props);
 
-		if (nvmldev)
+		if (nvmldev && _starpu_nvmlDeviceGetIndex && _starpu_nvmlDeviceGetPciInfo && _starpu_nvmlDeviceGetUUID)
 		{
 			unsigned int index;
-			if (nvmlDeviceGetIndex(nvmldev, &index) == NVML_SUCCESS)
+			if (_starpu_nvmlDeviceGetIndex(nvmldev, &index) == NVML_SUCCESS)
 			{
 				res = hwloc_nvml_get_device_osdev_by_index(topology, index);
 				if (res)
@@ -2806,7 +2821,7 @@ static void write_bus_platform_file_content(int version)
 		memset(nvswitch, 0, sizeof(nvswitch));
 
 		/* TODO: move to drivers */
-#if defined(STARPU_HAVE_LIBNVIDIA_ML) && !defined(STARPU_USE_CUDA0) && !defined(STARPU_USE_CUDA1)
+#if defined(STARPU_HAVE_NVML_H) && !defined(STARPU_USE_CUDA0) && !defined(STARPU_USE_CUDA1)
 		/* First find NVLinks */
 		struct cudaDeviceProp props[ncuda];
 
@@ -2817,6 +2832,7 @@ static void write_bus_platform_file_content(int version)
 				props[i].name[0] = 0;
 		}
 
+		if (_starpu_nvmlDeviceGetNvLinkState && _starpu_nvmlDeviceGetNvLinkRemotePciInfo)
 		for (i = 0; i < ncuda; i++)
 		{
 			unsigned j;
@@ -2836,12 +2852,12 @@ static void write_bus_platform_file_content(int version)
 				nvmlPciInfo_t pci;
 				unsigned k;
 
-				nvmlret = nvmlDeviceGetNvLinkState(nvmldev, j, &active);
+				nvmlret = _starpu_nvmlDeviceGetNvLinkState(nvmldev, j, &active);
 				if (nvmlret != NVML_SUCCESS)
 					continue;
 				if (active != NVML_FEATURE_ENABLED)
 					continue;
-				nvmlret = nvmlDeviceGetNvLinkRemotePciInfo(nvmldev, j, &pci);
+				nvmlret = _starpu_nvmlDeviceGetNvLinkRemotePciInfo(nvmldev, j, &pci);
 				if (nvmlret != NVML_SUCCESS)
 					continue;
 
@@ -3156,6 +3172,35 @@ void _starpu_load_bus_performance_files(void)
 #endif
 }
 
+static unsigned _get_raw_memory_node_index(unsigned node)
+{
+	enum starpu_node_kind type = starpu_node_get_kind(node), cur;
+	int devid = starpu_memory_node_get_devid(node);
+	unsigned base;
+
+	base = 0;
+	for (cur = STARPU_CPU_RAM; cur < type; cur++)
+		base += nmem[cur];
+
+	return base + devid;
+}
+
+void _starpu_init_bus_performance(void)
+{
+	unsigned src, dst, raw_src, raw_dst;
+
+	for (src = 0; src < STARPU_MAXNODES; src++)
+	{
+		for (dst = 0; dst < STARPU_MAXNODES; dst++)
+		{
+			raw_src = _get_raw_memory_node_index(src);
+			raw_dst = _get_raw_memory_node_index(dst);
+			bandwidth_matrix[src][dst] = raw_bandwidth_matrix[raw_src][raw_dst];
+			latency_matrix[src][dst] = raw_latency_matrix[raw_src][raw_dst];
+		}
+	}
+}
+
 /* (in MB/s) */
 double starpu_transfer_bandwidth(unsigned src_node, unsigned dst_node)
 {
