@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Simon Archipoff
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
@@ -22,10 +22,15 @@
 #include <common/utils.h>
 #include <core/sched_policy.h>
 #include <profiling/profiling.h>
+#include <datawizard/memory_nodes.h>
 #include <common/barrier.h>
 #include <core/debug.h>
 #include <core/task.h>
 
+#ifdef HAVE_DLOPEN
+#include <dlfcn.h>
+#endif
+
 static int use_prefetch = 0;
 static double idle[STARPU_NMAXWORKERS];
 static double idle_start[STARPU_NMAXWORKERS];
@@ -35,13 +40,15 @@ long _starpu_task_break_on_sched = -1;
 long _starpu_task_break_on_pop = -1;
 long _starpu_task_break_on_exec = -1;
 static const char *starpu_idle_file;
+static void *dl_sched_handle = NULL;
+static const char *sched_lib = NULL;
 
 void _starpu_sched_init(void)
 {
-	_starpu_task_break_on_push = starpu_get_env_number_default("STARPU_TASK_BREAK_ON_PUSH", -1);
-	_starpu_task_break_on_sched = starpu_get_env_number_default("STARPU_TASK_BREAK_ON_SCHED", -1);
-	_starpu_task_break_on_pop = starpu_get_env_number_default("STARPU_TASK_BREAK_ON_POP", -1);
-	_starpu_task_break_on_exec = starpu_get_env_number_default("STARPU_TASK_BREAK_ON_EXEC", -1);
+	_starpu_task_break_on_push = starpu_getenv_number_default("STARPU_TASK_BREAK_ON_PUSH", -1);
+	_starpu_task_break_on_sched = starpu_getenv_number_default("STARPU_TASK_BREAK_ON_SCHED", -1);
+	_starpu_task_break_on_pop = starpu_getenv_number_default("STARPU_TASK_BREAK_ON_POP", -1);
+	_starpu_task_break_on_exec = starpu_getenv_number_default("STARPU_TASK_BREAK_ON_EXEC", -1);
 	starpu_idle_file = starpu_getenv("STARPU_IDLE_FILE");
 }
 
@@ -102,6 +109,20 @@ struct starpu_sched_policy *_starpu_get_sched_policy(struct _starpu_sched_ctx *s
 	return sched_ctx->sched_policy;
 }
 
+struct starpu_sched_policy *starpu_sched_get_sched_policy_in_ctx(unsigned sched_ctx_id)
+{
+	struct _starpu_machine_config *config = _starpu_get_machine_config();
+	struct _starpu_sched_ctx *sched_ctx = &config->sched_ctxs[sched_ctx_id];
+	return sched_ctx->sched_policy;
+}
+
+struct starpu_sched_policy *starpu_sched_get_sched_policy(void)
+{
+	unsigned nsched_ctxs = _starpu_get_nsched_ctxs();
+	unsigned sched_ctx_id = nsched_ctxs == 1 ? 0 : starpu_sched_ctx_get_context();
+	return starpu_sched_get_sched_policy_in_ctx(sched_ctx_id);
+}
+
 /*
  *	Methods to initialize the scheduling policy
  */
@@ -114,15 +135,35 @@ static void load_sched_policy(struct starpu_sched_policy *sched_policy, struct _
 	if (sched_policy->policy_name)
 	{
 		if (sched_policy->policy_description)
-                        _STARPU_DEBUG("Use %s scheduler (%s)\n", sched_policy->policy_name, sched_policy->policy_description);
-                else
-                        _STARPU_DEBUG("Use %s scheduler \n", sched_policy->policy_name);
+			_STARPU_DEBUG("Use %s scheduler (%s)\n", sched_policy->policy_name, sched_policy->policy_description);
+		else
+			_STARPU_DEBUG("Use %s scheduler \n", sched_policy->policy_name);
 
 	}
 #endif
 
-	struct starpu_sched_policy *policy = sched_ctx->sched_policy;
-	memcpy(policy, sched_policy, sizeof(*policy));
+	*(sched_ctx->sched_policy) = *sched_policy;
+}
+
+static void load_sched_lib()
+{
+	/* check if the requested policy can be loaded dynamically */
+	sched_lib = starpu_getenv("STARPU_SCHED_LIB");
+	if (sched_lib)
+	{
+#ifdef HAVE_DLOPEN
+		if (dl_sched_handle)
+		{
+			dlclose(dl_sched_handle);
+			dl_sched_handle = NULL;
+		}
+		dl_sched_handle = dlopen(sched_lib, RTLD_NOW);
+		if (!dl_sched_handle)
+			_STARPU_MSG("Warning: scheduling dynamic library '%s' can not be loaded\n", sched_lib);
+#else
+		_STARPU_MSG("Environment variable 'STARPU_SCHED_LIB' defined but the dlopen functionality is unavailable on the system\n");
+#endif
+	}
 }
 
 static struct starpu_sched_policy *find_sched_policy_from_name(const char *policy_name)
@@ -133,6 +174,34 @@ static struct starpu_sched_policy *find_sched_policy_from_name(const char *polic
 	if (strcmp(policy_name, "") == 0)
 		return NULL;
 
+	/* check if the requested policy can be loaded dynamically */
+	load_sched_lib();
+#ifdef HAVE_DLOPEN
+	if (dl_sched_handle)
+	{
+		struct starpu_sched_policy *(*func_sched)(const char *);
+		*(void**)(&func_sched) = dlsym(dl_sched_handle, "starpu_get_sched_lib_policy");
+		if (!func_sched)
+		{
+			/* no such symbol */
+			_STARPU_MSG("Warning: the library '%s' does not define the function 'starpu_get_sched_lib_policy' (error '%s')\n", sched_lib, dlerror());
+			dlclose(dl_sched_handle);
+			dl_sched_handle = NULL;
+		}
+		else
+		{
+			struct starpu_sched_policy *dl_sched_policy = func_sched(policy_name);
+			if (dl_sched_policy)
+				return dl_sched_policy;
+			else
+			{
+				dlclose(dl_sched_handle);
+				dl_sched_handle = NULL;
+			}
+		}
+	}
+#endif
+
 	if (strncmp(policy_name, "heft", 4) == 0)
 	{
 		_STARPU_MSG("Warning: heft is now called \"dmda\".\n");
@@ -152,8 +221,11 @@ static struct starpu_sched_policy *find_sched_policy_from_name(const char *polic
 			}
 		}
 	}
-	if (strcmp(policy_name, "help") != 0)
-		_STARPU_MSG("Warning: scheduling policy '%s' was not found, try 'help' to get a list\n", policy_name);
+
+	if (strcmp(policy_name, "help") == 0)
+		return NULL;
+
+	_STARPU_MSG("Warning: scheduling policy '%s' was not found, try 'help' to get a list\n", policy_name);
 
 	/* nothing was found */
 	return NULL;
@@ -174,6 +246,26 @@ static void display_sched_help_message(FILE *stream)
 			fprintf(stream, "%-30s\t-> %s\n", p->policy_name, p->policy_description);
 		}
 		fprintf(stream, "\n");
+
+		load_sched_lib();
+#ifdef HAVE_DLOPEN
+		if (dl_sched_handle)
+		{
+			struct starpu_sched_policy **(*func_scheds)(void);
+			*(void**)(&func_scheds) = dlsym(dl_sched_handle, "starpu_get_sched_lib_policies");
+			if (func_scheds)
+			{
+				fprintf(stream, "(dynamically available policies)\n");
+				struct starpu_sched_policy **dl_sched_policies = func_scheds();
+				for(policy=dl_sched_policies ; *policy!=NULL ; policy++)
+				{
+					struct starpu_sched_policy *p = *policy;
+					fprintf(stream, "%-30s\t-> %s\n", p->policy_name, p->policy_description);
+				}
+				fprintf(stream, "\n");
+			}
+		}
+#endif
 	 }
 }
 
@@ -189,7 +281,7 @@ struct starpu_sched_policy *_starpu_select_sched_policy(struct _starpu_machine_c
 	if (selected_policy)
 		return selected_policy;
 
-	/* First, we check whether the application explicitely gave a scheduling policy or not */
+	/* First, we check whether the application explicitly gave a scheduling policy or not */
 	if (user_conf && (user_conf->sched_policy))
 		return user_conf->sched_policy;
 
@@ -215,7 +307,7 @@ void _starpu_init_sched_policy(struct _starpu_machine_config *config, struct _st
 	display_sched_help_message(stderr);
 
 	/* Prefetch is activated by default */
-	use_prefetch = starpu_get_env_number("STARPU_PREFETCH");
+	use_prefetch = starpu_getenv_number("STARPU_PREFETCH");
 	if (use_prefetch == -1)
 		use_prefetch = 1;
 
@@ -224,12 +316,12 @@ void _starpu_init_sched_policy(struct _starpu_machine_config *config, struct _st
 
 	load_sched_policy(selected_policy, sched_ctx);
 
-	if (starpu_get_env_number_default("STARPU_WORKER_TREE", 0))
+	if (starpu_getenv_number_default("STARPU_WORKER_TREE", 0))
 	{
 #ifdef STARPU_HAVE_HWLOC
 		sched_ctx->sched_policy->worker_type = STARPU_WORKER_TREE;
 #else
-		_STARPU_DISP("STARPU_WORKER_TREE ignored, please rebuild StarPU with hwloc support to enable it.");
+		_STARPU_DISP("STARPU_WORKER_TREE ignored, please rebuild StarPU with hwloc support to enable it.\n");
 #endif
 	}
 	starpu_sched_ctx_create_worker_collection(sched_ctx->id,
@@ -250,6 +342,13 @@ void _starpu_deinit_sched_policy(struct _starpu_sched_ctx *sched_ctx)
 		_STARPU_SCHED_END;
 	}
 	starpu_sched_ctx_delete_worker_collection(sched_ctx->id);
+#ifdef HAVE_DLOPEN
+	if (dl_sched_handle)
+	{
+		dlclose(dl_sched_handle);
+		dl_sched_handle = NULL;
+	}
+#endif
 }
 
 void _starpu_sched_task_submit(struct starpu_task *task)
@@ -295,7 +394,7 @@ static void _starpu_push_task_on_specific_worker_notify_sched(struct starpu_task
 	}
 }
 
-/* Enqueue a task into the list of tasks explicitely attached to a worker. In
+/* Enqueue a task into the list of tasks explicitly attached to a worker. In
  * case workerid identifies a combined worker, a task will be enqueued into
  * each worker of the combination. */
 static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int workerid)
@@ -371,10 +470,7 @@ static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int wo
 		}
 //		if(task->sched_ctx != _starpu_get_initial_sched_ctx()->id)
 
-		if(task->priority > 0)
-			return _starpu_push_local_task(worker, task, 1);
-		else
-			return _starpu_push_local_task(worker, task, 0);
+		return _starpu_push_local_task(worker, task);
 	}
 	else
 	{
@@ -403,9 +499,9 @@ static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int wo
 			struct starpu_task *alias = starpu_task_dup(task);
 			alias->destroy = 1;
 
-			_STARPU_TRACE_JOB_PUSH(alias, alias->priority > 0);
+			_STARPU_TRACE_JOB_PUSH(alias, alias->priority);
 			worker = _starpu_get_worker_struct(combined_workerid[j]);
-			ret |= _starpu_push_local_task(worker, alias, 0);
+			ret |= _starpu_push_local_task(worker, alias);
 		}
 
 		return ret;
@@ -416,6 +512,10 @@ static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int wo
 
 int _starpu_push_task(struct _starpu_job *j)
 {
+#ifdef STARPU_SIMGRID
+	if (_starpu_simgrid_task_push_cost())
+		starpu_sleep(0.000001);
+#endif
 	if(j->task->prologue_callback_func)
 	{
 		_starpu_set_current_task(j->task);
@@ -423,6 +523,24 @@ int _starpu_push_task(struct _starpu_job *j)
 		_starpu_set_current_task(NULL);
 	}
 
+	if (j->task->transaction)
+	{
+		/* If task is part of a transaction and its epoch is cancelled, switch its
+		 * 'where' field to STARPU_NOWHERE to skip its execution */
+		struct starpu_transaction *p_trs = j->task->transaction;
+		STARPU_ASSERT(j->task->transaction->state == _starpu_trs_initialized);
+		_starpu_spin_lock(&p_trs->lock);
+		STARPU_ASSERT(!_starpu_trs_epoch_list_empty(&p_trs->epoch_list));
+		struct _starpu_trs_epoch *p_epoch = _starpu_trs_epoch_list_front(&p_trs->epoch_list);
+		STARPU_ASSERT(p_epoch == j->task->trs_epoch);
+		STARPU_ASSERT(p_epoch->state == _starpu_trs_epoch_confirmed || p_epoch->state == _starpu_trs_epoch_cancelled);
+		if (p_epoch->state == _starpu_trs_epoch_cancelled)
+		{
+			j->task->where = STARPU_NOWHERE;
+		}
+		_starpu_spin_unlock(&p_trs->lock);
+	}
+
 	return _starpu_repush_task(j);
 }
 
@@ -437,6 +555,27 @@ int _starpu_repush_task(struct _starpu_job *j)
 	unsigned can_push = _starpu_increment_nready_tasks_of_sched_ctx(task->sched_ctx, task->flops, task);
 	STARPU_ASSERT(task->status == STARPU_TASK_BLOCKED || task->status == STARPU_TASK_BLOCKED_ON_TAG || task->status == STARPU_TASK_BLOCKED_ON_TASK || task->status == STARPU_TASK_BLOCKED_ON_DATA);
 	task->status = STARPU_TASK_READY;
+	const unsigned continuation =
+#ifdef STARPU_OPENMP
+		j->continuation
+#else
+		0
+#endif
+		;
+	if (!_starpu_perf_counter_paused() && !j->internal && !continuation)
+	{
+		(void) STARPU_PERF_COUNTER_ADD64(& _starpu_task__g_current_submitted__value, -1);
+		int64_t value = STARPU_PERF_COUNTER_ADD64(& _starpu_task__g_current_ready__value, 1);
+		_starpu_perf_counter_update_max_int64(&_starpu_task__g_peak_ready__value, value);
+		if (task->cl && task->cl->perf_counter_values)
+		{
+			struct starpu_perf_counter_sample_cl_values * const pcv = task->cl->perf_counter_values;
+
+			(void)STARPU_PERF_COUNTER_ADD64(&pcv->task.current_submitted, -1);
+			value = STARPU_PERF_COUNTER_ADD64(&pcv->task.current_ready, 1);
+			_starpu_perf_counter_update_max_int64(&pcv->task.peak_ready, value);
+		}
+	}
 	STARPU_AYU_ADDTOTASKQUEUE(j->job_id, -1);
 	/* if the context does not have any workers save the tasks in a temp list */
 	if ((task->cl != NULL && task->where != STARPU_NOWHERE) && (!sched_ctx->is_initial_sched))
@@ -471,6 +610,16 @@ int _starpu_repush_task(struct _starpu_job *j)
 	 * corresponding dependencies */
 	if (task->cl == NULL || task->where == STARPU_NOWHERE)
 	{
+		_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
+		if (!_starpu_perf_counter_paused() && !j->internal)
+		{
+			(void)STARPU_PERF_COUNTER_ADD64(& _starpu_task__g_current_ready__value, -1);
+			if (task->cl && task->cl->perf_counter_values)
+			{
+				struct starpu_perf_counter_sample_cl_values * const pcv = task->cl->perf_counter_values;
+				(void)STARPU_PERF_COUNTER_ADD64(&pcv->task.current_ready, -1);
+			}
+		}
 		task->status = STARPU_TASK_RUNNING;
 		if (task->prologue_callback_pop_func)
 		{
@@ -486,7 +635,11 @@ int _starpu_repush_task(struct _starpu_job *j)
 		}
 		else
 		{
-			if (task->cl)
+			if (task->cl
+#ifdef STARPU_BUBBLE
+			    && !j->is_bubble
+#endif
+			    )
 				__starpu_push_task_output(j);
 			_starpu_handle_job_termination(j);
 			_STARPU_LOG_OUT_TAG("handle_job_termination");
@@ -505,7 +658,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 {
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(task->sched_ctx);
 
-	_STARPU_TRACE_JOB_PUSH(task, task->priority > 0);
+	_STARPU_TRACE_JOB_PUSH(task, task->priority);
 
 	/* if the contexts still does not have workers put the task back to its place in
 	   the empty ctx list */
@@ -539,30 +692,12 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 	int ret = 0;
 	if (STARPU_UNLIKELY(task->execute_on_a_specific_worker))
 	{
-		if (starpu_get_prefetch_flag())
-			starpu_prefetch_task_input_for(task, task->workerid);
-
 		ret = _starpu_push_task_on_specific_worker(task, task->workerid);
 	}
 	else
 	{
 		struct _starpu_machine_config *config = _starpu_get_machine_config();
 
-		/* When a task can only be executed on a given arch and we have
-		 * only one memory node for that arch, we can systematically
-		 * prefetch before the scheduling decision. */
-		if (starpu_get_prefetch_flag() && starpu_memory_nodes_get_count() > 1)
-		{
-			if (task->where == STARPU_CPU && config->cpus_nodeid >= 0)
-				starpu_prefetch_task_input_on_node(task, config->cpus_nodeid);
-			else if (task->where == STARPU_CUDA && config->cuda_nodeid >= 0)
-				starpu_prefetch_task_input_on_node(task, config->cuda_nodeid);
-			else if (task->where == STARPU_OPENCL && config->opencl_nodeid >= 0)
-				starpu_prefetch_task_input_on_node(task, config->opencl_nodeid);
-			else if (task->where == STARPU_MIC && config->mic_nodeid >= 0)
-				starpu_prefetch_task_input_on_node(task, config->mic_nodeid);
-		}
-
 		if(!sched_ctx->sched_policy)
 		{
 			/* Note: we have to call that early, or else the task may have
@@ -594,7 +729,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 					if (job->task_size > 1)
 					{
 						alias = starpu_task_dup(task);
-						_STARPU_TRACE_JOB_PUSH(alias, alias->priority > 0);
+						_STARPU_TRACE_JOB_PUSH(alias, alias->priority);
 						alias->destroy = 1;
 					}
 					else
@@ -605,6 +740,25 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 		}
 		else
 		{
+			/* When a task can only be executed on a given arch and we have
+			 * only one memory node for that arch, we can systematically
+			 * prefetch before the scheduling decision. */
+			if (!sched_ctx->sched_policy->prefetches
+				&& starpu_get_prefetch_flag()
+				&& starpu_memory_nodes_get_count() > 1)
+			{
+				enum starpu_worker_archtype type;
+				for (type = 0; type < STARPU_NARCH; type++)
+				{
+					if (task->where == (int32_t) STARPU_WORKER_TO_MASK(type))
+					{
+						if (config->arch_nodeid[type] >= 0)
+							starpu_prefetch_task_input_on_node(task, config->arch_nodeid[type]);
+						break;
+					}
+				}
+			}
+
 			STARPU_ASSERT(sched_ctx->sched_policy->push_task);
 			/* check out if there are any workers in the context */
 			unsigned nworkers = starpu_sched_ctx_get_nworkers(sched_ctx->id);
@@ -635,7 +789,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 		if(ret == -1)
 		{
 			_STARPU_MSG("repush task \n");
-			_STARPU_TRACE_JOB_POP(task, task->priority > 0);
+			_STARPU_TRACE_JOB_POP(task, task->priority);
 			ret = _starpu_push_task_to_workers(task);
 		}
 	}
@@ -662,7 +816,7 @@ int _starpu_pop_task_end(struct starpu_task *task)
 {
 	if (!task)
 		return 0;
-	_STARPU_TRACE_JOB_POP(task, task->priority > 0);
+	_STARPU_TRACE_JOB_POP(task, task->priority);
 	return 0;
 }
 
@@ -679,7 +833,9 @@ struct starpu_task *_starpu_create_conversion_task_for_arch(starpu_data_handle_t
 {
 	struct starpu_task *conversion_task;
 
-#if defined(STARPU_USE_OPENCL) || defined(STARPU_USE_CUDA) || defined(STARPU_USE_MIC) || defined(STARPU_SIMGRID)
+/* Driver porters: adding your driver here is optional, only needed for the support of multiple formats.  */
+
+#if defined(STARPU_USE_OPENCL) || defined(STARPU_USE_CUDA) || defined(STARPU_SIMGRID)
 	struct starpu_multiformat_interface *format_interface;
 #endif
 
@@ -688,7 +844,7 @@ struct starpu_task *_starpu_create_conversion_task_for_arch(starpu_data_handle_t
 	conversion_task->synchronous = 0;
 	STARPU_TASK_SET_HANDLE(conversion_task, handle, 0);
 
-#if defined(STARPU_USE_OPENCL) || defined(STARPU_USE_CUDA) || defined(STARPU_USE_MIC) || defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_OPENCL) || defined(STARPU_USE_CUDA) || defined(STARPU_SIMGRID)
 	/* The node does not really matter here */
 	format_interface = (struct starpu_multiformat_interface *) starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
 #endif
@@ -698,6 +854,8 @@ struct starpu_task *_starpu_create_conversion_task_for_arch(starpu_data_handle_t
 	handle->busy_count++;
 	_starpu_spin_unlock(&handle->header_lock);
 
+	/* Driver porters: adding your driver here is optional, only needed for the support of multiple formats.  */
+
 	switch(node_kind)
 	{
 	case STARPU_CPU_RAM:
@@ -722,15 +880,6 @@ struct starpu_task *_starpu_create_conversion_task_for_arch(starpu_data_handle_t
 			conversion_task->cl = mf_ops->opencl_to_cpu_cl;
 			break;
 		}
-#endif
-#ifdef STARPU_USE_MIC
-		case STARPU_MIC_RAM:
-		{
-			struct starpu_multiformat_data_interface_ops *mf_ops;
-			mf_ops = (struct starpu_multiformat_data_interface_ops *) handle->ops->get_mf_ops(format_interface);
-			conversion_task->cl = mf_ops->mic_to_cpu_cl;
-			break;
-		}
 #endif
 		default:
 			_STARPU_ERROR("Oops : %u\n", handle->mf_node);
@@ -753,15 +902,6 @@ struct starpu_task *_starpu_create_conversion_task_for_arch(starpu_data_handle_t
 		conversion_task->cl = mf_ops->cpu_to_opencl_cl;
 		break;
 	}
-#endif
-#ifdef STARPU_USE_MIC
-	case STARPU_MIC_RAM:
-	{
-		struct starpu_multiformat_data_interface_ops *mf_ops;
-		mf_ops = (struct starpu_multiformat_data_interface_ops *) handle->ops->get_mf_ops(format_interface);
-		conversion_task->cl = mf_ops->cpu_to_mic_cl;
-		break;
-	}
 #endif
 	default:
 		STARPU_ABORT();
@@ -961,7 +1101,7 @@ pick:
 
 	/*
 	 * This worker may not be able to execute this task. In this case, we
-	 * should return the task anyway. It will be pushed back almost immediatly.
+	 * should return the task anyway. It will be pushed back almost immediately.
 	 * This way, we avoid computing and executing the conversions tasks.
 	 * Here, we do not care about what implementation is used.
 	 */
@@ -997,7 +1137,7 @@ pick:
 	}
 
 	task->mf_skip = 1;
-	starpu_task_list_push_back(&worker->local_tasks, task);
+	starpu_task_prio_list_push_back(&worker->local_tasks, task);
 	goto pick;
 
 profiling:
@@ -1011,8 +1151,7 @@ profiling:
 		 * even though we already tested if profiling is enabled. */
 		if (profiling_info)
 		{
-			memcpy(&profiling_info->pop_start_time,
-				&pop_start_time, sizeof(struct timespec));
+			profiling_info->pop_start_time = pop_start_time;
 			_starpu_clock_gettime(&profiling_info->pop_end_time);
 		}
 	}
@@ -1027,24 +1166,6 @@ profiling:
 	return task;
 }
 
-struct starpu_task *_starpu_pop_every_task(struct _starpu_sched_ctx *sched_ctx)
-{
-	struct starpu_task *task = NULL;
-	if(sched_ctx->sched_policy)
-	{
-		STARPU_ASSERT(sched_ctx->sched_policy->pop_every_task);
-
-		/* TODO set profiling info */
-		if(sched_ctx->sched_policy->pop_every_task)
-		{
-			_STARPU_SCHED_BEGIN;
-			task = sched_ctx->sched_policy->pop_every_task(sched_ctx->id);
-			_STARPU_SCHED_END;
-		}
-	}
-	return task;
-}
-
 void _starpu_sched_pre_exec_hook(struct starpu_task *task)
 {
 	unsigned sched_ctx_id = starpu_sched_ctx_get_ctx_for_task(task);
@@ -1120,35 +1241,11 @@ void _starpu_sched_post_exec_hook(struct starpu_task *task)
 	}
 }
 
-void _starpu_wait_on_sched_event(void)
-{
-	struct _starpu_worker *worker = _starpu_get_local_worker_key();
-
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
-
-	_starpu_handle_all_pending_node_data_requests(worker->memory_node);
-
-	if (_starpu_machine_is_running())
-	{
-#ifndef STARPU_NON_BLOCKING_DRIVERS
-		STARPU_PTHREAD_COND_WAIT(&worker->sched_cond,
-					  &worker->sched_mutex);
-#endif
-	}
-
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
-}
-
-/* The scheduling policy may put tasks directly into a worker's local queue so
- * that it is not always necessary to create its own queue when the local queue
- * is sufficient. If "back" not null, the task is put at the back of the queue
- * where the worker will pop tasks first. Setting "back" to 0 therefore ensures
- * a FIFO ordering. */
-int starpu_push_local_task(int workerid, struct starpu_task *task, int prio)
+int starpu_push_local_task(int workerid, struct starpu_task *task, int back STARPU_ATTRIBUTE_UNUSED)
 {
 	struct _starpu_worker *worker = _starpu_get_worker_struct(workerid);
 
-	return  _starpu_push_local_task(worker, task, prio);
+	return  _starpu_push_local_task(worker, task);
 }
 
 void _starpu_print_idle_time()
@@ -1173,7 +1270,7 @@ void _starpu_print_idle_time()
 	}
 }
 
-void starpu_sched_task_break(struct starpu_task *task)
+void starpu_sched_task_break(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED)
 {
 	_STARPU_TASK_BREAK_ON(task, sched);
 }
