@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
  *
@@ -63,6 +63,8 @@
 #include <core/simgrid.h>
 #endif
 
+static int main_thread_cpuid = -1;
+
 static unsigned topology_is_initialized = 0;
 static int nobind;
 static int numa_enabled = -1;
@@ -596,7 +598,7 @@ static void _starpu_init_topology(struct _starpu_machine_config *config)
 
 	if (config->cpu_depth == HWLOC_TYPE_DEPTH_UNKNOWN)
 	{
-		/* unknown, using logical procesors as fallback */
+		/* unknown, using logical processors as fallback */
 		_STARPU_DISP("Warning: The OS did not report CPU cores. Assuming there is only one hardware thread per core.\n");
 		config->cpu_depth = hwloc_get_type_depth(topology->hwtopology,
 							 HWLOC_OBJ_PU);
@@ -739,7 +741,7 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 
 	if (topology->nhwpus % topology->nhwworker[STARPU_CPU_WORKER][0])
 	{
-		_STARPU_DISP("Warning: hwloc reported %d logical CPUs for %d cores, this is not homogeneous, will assume %d logical CPUs per core", topology->nhwpus, topology->nhwworker[STARPU_CPU_WORKER][0], nhyperthreads);
+		_STARPU_DISP("Warning: hwloc reported %d logical CPUs for %d cores, this is not homogeneous, will assume %d logical CPUs per core\n", topology->nhwpus, topology->nhwworker[STARPU_CPU_WORKER][0], nhyperthreads);
 	}
 
 	/* conf->workers_bindid indicates the successive logical PU identifier that
@@ -1079,6 +1081,12 @@ unsigned _starpu_topology_get_numa_core_binding(struct _starpu_machine_config *c
 	{
 		hwloc_obj_t obj = hwloc_get_obj_by_type(config->topology.hwtopology, HWLOC_OBJ_NUMANODE, numa_binding[n]);
 
+		if (!obj)
+		{
+			/* NUMA nodes not available, fall back to the whole machine */
+			return _starpu_topology_get_core_binding(binding, nbinding, hwloc_get_root_obj(config->topology.hwtopology));
+		}
+
 #if HWLOC_API_VERSION >= 0x00020000
 		/* Get the actual topology object */
 		obj = obj->parent;
@@ -1244,6 +1252,11 @@ static unsigned _starpu_topology_count_ngpus(hwloc_obj_t obj)
 
 	for (i = 0; i < obj->arity; i++)
 		n += _starpu_topology_count_ngpus(obj->children[i]);
+#if HWLOC_API_VERSION >= 0x00020000
+	hwloc_obj_t child;
+	for (child = obj->io_first_child; child; child = child->next_sibling)
+		n += _starpu_topology_count_ngpus(child);
+#endif
 
 	data->ngpus = n;
 //#ifdef STARPU_VERBOSE
@@ -1278,7 +1291,7 @@ static int _starpu_init_machine_config(struct _starpu_machine_config *config, in
 	_starpu_initialize_workers_bindid(config);
 
 	/* Reserve thread for main() */
-	int main_thread_cpuid = starpu_getenv_number_default("STARPU_MAIN_THREAD_CPUID", -1);
+	main_thread_cpuid = starpu_getenv_number_default("STARPU_MAIN_THREAD_CPUID", -1);
 	int main_thread_coreid = starpu_getenv_number_default("STARPU_MAIN_THREAD_COREID", -1);
 	if (main_thread_cpuid >= 0 && main_thread_coreid >= 0)
 	{
@@ -1422,19 +1435,89 @@ void _starpu_destroy_machine_config(struct _starpu_machine_config *config, int n
 		_starpu_may_bind_automatically[i] = 0;
 }
 
+int starpu_cpu_os_index(int cpuid)
+{
+#ifdef STARPU_HAVE_HWLOC
+	if (cpuid < 0)
+		return -1;
+
+	struct _starpu_machine_config *config = _starpu_get_machine_config();
+
+	hwloc_obj_t obj = hwloc_get_obj_by_depth(config->topology.hwtopology, config->pu_depth, cpuid);
+	hwloc_bitmap_t set = obj->cpuset;
+
+	return hwloc_bitmap_first(set);
+#else
+	return cpuid;
+#endif
+}
+
+void _starpu_do_bind_thread_on_cpu(int cpuid STARPU_ATTRIBUTE_UNUSED)
+{
+#ifndef STARPU_SIMGRID
+	if (nobind > 0)
+		return;
+	if (cpuid < 0)
+		return;
+
+#ifdef STARPU_HAVE_HWLOC
+	struct _starpu_machine_config *config = _starpu_get_machine_config();
+
+	const struct hwloc_topology_support *support = hwloc_topology_get_support(config->topology.hwtopology);
+	if (support->cpubind->set_thisthread_cpubind)
+	{
+		hwloc_obj_t obj = hwloc_get_obj_by_depth(config->topology.hwtopology, config->pu_depth, cpuid);
+		hwloc_bitmap_t set = obj->cpuset;
+		int res;
+
+		hwloc_bitmap_singlify(set);
+		res = hwloc_set_cpubind(config->topology.hwtopology, set, HWLOC_CPUBIND_THREAD);
+		if (res)
+		{
+			perror("hwloc_set_cpubind");
+			STARPU_ABORT();
+		}
+	}
+#elif defined(HAVE_PTHREAD_SETAFFINITY_NP) && defined(__linux__)
+	int res;
+	/* fix the thread on the correct cpu */
+	cpu_set_t aff_mask;
+	CPU_ZERO(&aff_mask);
+	CPU_SET(cpuid, &aff_mask);
+
+	starpu_pthread_t self = starpu_pthread_self();
+
+	res = pthread_setaffinity_np(self, sizeof(aff_mask), &aff_mask);
+	if (res)
+	{
+		const char *msg = strerror(res);
+		_STARPU_MSG("pthread_setaffinity_np: %s\n", msg);
+		STARPU_ABORT();
+	}
+
+#elif defined(_WIN32)
+	DWORD mask = 1 << cpuid;
+	if (!SetThreadAffinityMask(GetCurrentThread(), mask))
+	{
+		_STARPU_ERROR("SetThreadMaskAffinity(%lx) failed\n", mask);
+	}
+#else
+#warning no CPU binding support
+#endif
+#endif
+}
+
 int _starpu_bind_thread_on_cpu(int cpuid STARPU_ATTRIBUTE_UNUSED, int workerid STARPU_ATTRIBUTE_UNUSED, const char *name STARPU_ATTRIBUTE_UNUSED)
 {
 	int ret = 0;
-#ifdef STARPU_SIMGRID
-	return ret;
-#else
+
+#ifndef STARPU_SIMGRID
 	if (nobind > 0)
 		return ret;
 	if (cpuid < 0)
 		return ret;
 
 #ifdef STARPU_HAVE_HWLOC
-	const struct hwloc_topology_support *support;
 	struct _starpu_machine_config *config = _starpu_get_machine_config();
 
 	_starpu_init_topology(config);
@@ -1484,49 +1567,9 @@ int _starpu_bind_thread_on_cpu(int cpuid STARPU_ATTRIBUTE_UNUSED, int workerid S
 			}
 		}
 	}
-
-	support = hwloc_topology_get_support(config->topology.hwtopology);
-	if (support->cpubind->set_thisthread_cpubind)
-	{
-		hwloc_obj_t obj = hwloc_get_obj_by_depth(config->topology.hwtopology, config->pu_depth, cpuid);
-		hwloc_bitmap_t set = obj->cpuset;
-		int res;
-
-		hwloc_bitmap_singlify(set);
-		res = hwloc_set_cpubind(config->topology.hwtopology, set, HWLOC_CPUBIND_THREAD);
-		if (res)
-		{
-			perror("hwloc_set_cpubind");
-			STARPU_ABORT();
-		}
-	}
-
-#elif defined(HAVE_PTHREAD_SETAFFINITY_NP) && defined(__linux__)
-	int res;
-	/* fix the thread on the correct cpu */
-	cpu_set_t aff_mask;
-	CPU_ZERO(&aff_mask);
-	CPU_SET(cpuid, &aff_mask);
-
-	starpu_pthread_t self = starpu_pthread_self();
-
-	res = pthread_setaffinity_np(self, sizeof(aff_mask), &aff_mask);
-	if (res)
-	{
-		const char *msg = strerror(res);
-		_STARPU_MSG("pthread_setaffinity_np: %s\n", msg);
-		STARPU_ABORT();
-	}
-
-#elif defined(_WIN32)
-	DWORD mask = 1 << cpuid;
-	if (!SetThreadAffinityMask(GetCurrentThread(), mask))
-	{
-		_STARPU_ERROR("SetThreadMaskAffinity(%lx) failed\n", mask);
-	}
-#else
-#warning no CPU binding support
 #endif
+
+	_starpu_do_bind_thread_on_cpu(cpuid);
 #endif
 	return ret;
 }
@@ -1577,6 +1620,25 @@ void _starpu_bind_thread_on_cpus(struct _starpu_combined_worker *combined_worker
 #endif
 }
 
+void starpu_bind_thread_on_main(void)
+{
+	_starpu_do_bind_thread_on_cpu(main_thread_cpuid);
+}
+
+void starpu_bind_thread_on_cpu(int cpuid)
+{
+	_starpu_do_bind_thread_on_cpu(cpuid);
+}
+
+void starpu_bind_thread_on_worker(unsigned workerid)
+{
+	unsigned basic_worker_count = starpu_worker_get_count();
+	if (workerid < basic_worker_count)
+		_starpu_do_bind_thread_on_cpu(starpu_worker_get_bindid(workerid));
+	else
+		_starpu_bind_thread_on_cpus(_starpu_get_combined_worker_struct(workerid));
+}
+
 static size_t _starpu_cpu_get_global_mem_size(int nodeid, struct _starpu_machine_config *config)
 {
 	size_t global_mem;
@@ -1989,6 +2051,14 @@ static void _starpu_init_workers_binding_and_memory(struct _starpu_machine_confi
 	_starpu_init_numa_node(config);
 	_starpu_init_numa_bus();
 
+#ifdef STARPU_SIMGRID
+	_starpu_simgrid_count_ngpus();
+#else
+#ifdef STARPU_HAVE_HWLOC
+	_starpu_topology_count_ngpus(hwloc_get_root_obj(config->topology.hwtopology));
+#endif
+#endif
+
 	/* Eventually initialize accelerators memory nodes */
 	if (!no_mp_config)
 	for (worker = 0; worker < config->topology.nworkers; worker++)
@@ -2027,14 +2097,6 @@ static void _starpu_init_workers_binding_and_memory(struct _starpu_machine_confi
 		STARPU_ASSERT_MSG(ret, "oops, didn't find any worker to drive memory node %d!?", node);
 	}
 #endif
-
-#ifdef STARPU_SIMGRID
-	_starpu_simgrid_count_ngpus();
-#else
-#ifdef STARPU_HAVE_HWLOC
-	_starpu_topology_count_ngpus(hwloc_get_root_obj(config->topology.hwtopology));
-#endif
-#endif
 }
 
 int _starpu_build_topology(struct _starpu_machine_config *config, int no_mp_config)
@@ -2069,6 +2131,8 @@ int _starpu_build_topology(struct _starpu_machine_config *config, int no_mp_conf
 			config->arch_nodeid[type] = -2;
 	}
 
+	_starpu_init_bus_performance();
+
 	return 0;
 }
 
