@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2010-2016, 2018                                Université de Bordeaux
- * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,35 +15,30 @@
  */
 
 #include <starpu.h>
-#include <common/config.h>
-#include <datawizard/coherency.h>
-#include <datawizard/copy_driver.h>
-#include <datawizard/filters.h>
+#ifdef BUILDING_STARPU
 #include <datawizard/memory_nodes.h>
-#include <starpu_hash.h>
-#include <starpu_cuda.h>
-#include <starpu_opencl.h>
-#include <drivers/opencl/driver_opencl.h>
-#include <drivers/scc/driver_scc_source.h>
-#include <drivers/mic/driver_mic_source.h>
+#endif
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+static int map_variable(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int unmap_variable(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int update_map_variable(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
 
 static const struct starpu_data_copy_methods variable_copy_data_methods_s =
 {
 	.any_to_any = copy_any_to_any,
 };
 
-static void register_variable_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_variable_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static starpu_ssize_t allocate_variable_buffer_on_node(void *data_interface_, unsigned dst_node);
-static void *variable_handle_to_pointer(starpu_data_handle_t data_handle, unsigned node);
-static int variable_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
+static void *variable_to_pointer(void *data_interface, unsigned node);
 static void free_variable_buffer_on_node(void *data_interface, unsigned node);
 static size_t variable_interface_get_size(starpu_data_handle_t handle);
 static uint32_t footprint_variable_interface_crc32(starpu_data_handle_t handle);
 static int variable_compare(void *data_interface_a, void *data_interface_b);
 static void display_variable_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_variable_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_variable_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_variable_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
@@ -53,9 +46,11 @@ struct starpu_data_interface_ops starpu_interface_variable_ops =
 {
 	.register_data_handle = register_variable_handle,
 	.allocate_data_on_node = allocate_variable_buffer_on_node,
-	.handle_to_pointer = variable_handle_to_pointer,
-	.pointer_is_inside = variable_pointer_is_inside,
+	.to_pointer = variable_to_pointer,
 	.free_data_on_node = free_variable_buffer_on_node,
+	.map_data = map_variable,
+	.unmap_data = unmap_variable,
+	.update_map = update_map_variable,
 	.copy_methods = &variable_copy_data_methods_s,
 	.get_size = variable_interface_get_size,
 	.footprint = footprint_variable_interface_crc32,
@@ -64,30 +59,25 @@ struct starpu_data_interface_ops starpu_interface_variable_ops =
 	.interface_size = sizeof(struct starpu_variable_interface),
 	.display = display_variable_interface,
 	.pack_data = pack_variable_handle,
+	.peek_data = peek_variable_handle,
 	.unpack_data = unpack_variable_handle,
 	.describe = describe,
-	.name = "STARPU_VARIABLE_INTERFACE"
+	.name = "STARPU_VARIABLE_INTERFACE",
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
-static void *variable_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
-{
-	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
-
-	return (void*) STARPU_VARIABLE_GET_PTR(starpu_data_get_interface_on_node(handle, node));
-}
-
-static int variable_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
+static void *variable_to_pointer(void *data_interface, unsigned node)
 {
 	(void) node;
-	struct starpu_variable_interface *variable_interface = data_interface;
-	return (char*) ptr >= (char*) variable_interface->ptr &&
-		(char*) ptr < (char*) variable_interface->ptr + variable_interface->elemsize;
+	return (void*) STARPU_VARIABLE_GET_PTR(data_interface);
 }
 
-static void register_variable_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_variable_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_variable_interface *variable_interface = (struct starpu_variable_interface *)data_interface;
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_variable_interface *local_interface = (struct starpu_variable_interface *)
@@ -113,7 +103,7 @@ static void register_variable_handle(starpu_data_handle_t handle, unsigned home_
 
 /* declare a new data with the variable interface */
 void starpu_variable_data_register(starpu_data_handle_t *handleptr, int home_node,
-                        uintptr_t ptr, size_t elemsize)
+				   uintptr_t ptr, size_t elemsize)
 {
 	struct starpu_variable_interface variable =
 	{
@@ -124,23 +114,21 @@ void starpu_variable_data_register(starpu_data_handle_t *handleptr, int home_nod
 		.elemsize = elemsize
 	};
 #ifndef STARPU_SIMGRID
-	if (home_node == STARPU_MAIN_RAM)
+	if (home_node >= 0 && starpu_node_get_kind(home_node) == STARPU_CPU_RAM)
 	{
-		STARPU_ASSERT_ACCESSIBLE(ptr);
-		STARPU_ASSERT_ACCESSIBLE(ptr + elemsize - 1);
+		if (elemsize)
+		{
+			STARPU_ASSERT_ACCESSIBLE(ptr);
+			STARPU_ASSERT_ACCESSIBLE(ptr + elemsize - 1);
+		}
 	}
 #endif
 
-#ifdef STARPU_USE_SCC
-	_starpu_scc_set_offset_in_shared_memory((void*)variable.ptr, (void**)&(variable.dev_handle),
-			&(variable.offset));
-#endif
-
 	starpu_data_register(handleptr, home_node, &variable, &starpu_interface_variable_ops);
 }
 
 void starpu_variable_ptr_register(starpu_data_handle_t handle, unsigned node,
-			uintptr_t ptr, uintptr_t dev_handle, size_t offset)
+				  uintptr_t ptr, uintptr_t dev_handle, size_t offset)
 {
 	struct starpu_variable_interface *variable_interface = starpu_data_get_interface_on_node(handle, node);
 	starpu_data_ptr_register(handle, node);
@@ -183,14 +171,14 @@ static int pack_variable_handle(starpu_data_handle_t handle, unsigned node, void
 
 	if (ptr != NULL)
 	{
-		starpu_malloc_flags(ptr, *count, 0);
+		*ptr = (void *)starpu_malloc_on_node_flags(node, *count, 0);
 		memcpy(*ptr, (void*)variable_interface->ptr, variable_interface->elemsize);
 	}
 
 	return 0;
 }
 
-static int unpack_variable_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_variable_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -200,6 +188,15 @@ static int unpack_variable_handle(starpu_data_handle_t handle, unsigned node, vo
 	STARPU_ASSERT(count == variable_interface->elemsize);
 
 	memcpy((void*)variable_interface->ptr, ptr, variable_interface->elemsize);
+
+	return 0;
+}
+
+static int unpack_variable_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_variable_handle(handle, node, ptr, count);
+	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
+
 	return 0;
 }
 
@@ -218,7 +215,7 @@ static size_t variable_interface_get_size(starpu_data_handle_t handle)
 uintptr_t starpu_variable_get_local_ptr(starpu_data_handle_t handle)
 {
 	unsigned node;
-	node = _starpu_memory_node_get_local_key();
+	node = starpu_worker_get_local_memory_node();
 
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -244,6 +241,8 @@ static starpu_ssize_t allocate_variable_buffer_on_node(void *data_interface_, un
 
 	/* update the data properly in consequence */
 	variable_interface->ptr = addr;
+	variable_interface->dev_handle = addr;
+	variable_interface->offset = 0;
 
 	return elemsize;
 }
@@ -251,7 +250,50 @@ static starpu_ssize_t allocate_variable_buffer_on_node(void *data_interface_, un
 static void free_variable_buffer_on_node(void *data_interface, unsigned node)
 {
 	struct starpu_variable_interface *variable_interface = (struct starpu_variable_interface *) data_interface;
-	starpu_free_on_node(node, variable_interface->ptr, variable_interface->elemsize);
+	starpu_free_on_node(node, variable_interface->dev_handle, variable_interface->elemsize);
+	variable_interface->ptr = 0;
+	variable_interface->dev_handle = 0;
+}
+
+static int map_variable(void *src_interface, unsigned src_node,
+			void *dst_interface, unsigned dst_node)
+{
+	struct starpu_variable_interface *src_variable = src_interface;
+	struct starpu_variable_interface *dst_variable = dst_interface;
+	int ret;
+	uintptr_t mapped;
+
+	mapped = starpu_interface_map(src_variable->dev_handle, src_variable->offset, src_node, dst_node, src_variable->elemsize, &ret);
+	if (mapped)
+	{
+		dst_variable->dev_handle = mapped;
+		dst_variable->offset = 0;
+		if (starpu_node_get_kind(dst_node) != STARPU_OPENCL_RAM)
+			dst_variable->ptr = mapped;
+		return 0;
+	}
+	return ret;
+}
+
+static int unmap_variable(void *src_interface, unsigned src_node,
+			  void *dst_interface, unsigned dst_node)
+{
+	struct starpu_variable_interface *src_variable = src_interface;
+	struct starpu_variable_interface *dst_variable = dst_interface;
+
+	int ret = starpu_interface_unmap(src_variable->dev_handle, src_variable->offset, src_node, dst_variable->dev_handle, dst_node, src_variable->elemsize);
+	dst_variable->dev_handle = 0;
+
+	return ret;
+}
+
+static int update_map_variable(void *src_interface, unsigned src_node,
+			       void *dst_interface, unsigned dst_node)
+{
+	struct starpu_variable_interface *src_variable = src_interface;
+	struct starpu_variable_interface *dst_variable = dst_interface;
+
+	return starpu_interface_update_map(src_variable->dev_handle, src_variable->offset, src_node, dst_variable->dev_handle, dst_variable->offset, dst_node, src_variable->elemsize);
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
@@ -267,7 +309,7 @@ static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_int
 
 	ret = starpu_interface_copy(ptr_src, 0, src_node, ptr_dst, 0, dst_node, elemsize, async_data);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, elemsize);
+	starpu_interface_data_copy(src_node, dst_node, elemsize);
 
 	return ret;
 }
