@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,8 +15,14 @@
  */
 
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+static int map_vector(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int unmap_vector(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int update_map(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
 
 static const struct starpu_data_copy_methods vector_copy_data_methods_s =
 {
@@ -24,11 +30,12 @@ static const struct starpu_data_copy_methods vector_copy_data_methods_s =
 };
 
 static void vector_init(void *data_interface);
-static void register_vector_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_vector_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static starpu_ssize_t allocate_vector_buffer_on_node(void *data_interface_, unsigned dst_node);
 static void *vector_to_pointer(void *data_interface, unsigned node);
-static int vector_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_vector_buffer_on_node(void *data_interface, unsigned node);
+static void cache_vector_buffer_on_node(void *new_data_interface, void *data_interface, unsigned node);
+static void reuse_vector_buffer_on_node(void *data_interface, const void *new_data_interface, unsigned node);
 static size_t vector_interface_get_size(starpu_data_handle_t handle);
 static size_t vector_interface_get_alloc_size(starpu_data_handle_t handle);
 static uint32_t footprint_vector_interface_crc32(starpu_data_handle_t handle);
@@ -37,6 +44,7 @@ static int vector_compare(void *data_interface_a, void *data_interface_b);
 static int vector_alloc_compare(void *data_interface_a, void *data_interface_b);
 static void display_vector_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_vector_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
@@ -46,8 +54,12 @@ struct starpu_data_interface_ops starpu_interface_vector_ops =
 	.register_data_handle = register_vector_handle,
 	.allocate_data_on_node = allocate_vector_buffer_on_node,
 	.to_pointer = vector_to_pointer,
-	.pointer_is_inside = vector_pointer_is_inside,
 	.free_data_on_node = free_vector_buffer_on_node,
+	.cache_data_on_node = cache_vector_buffer_on_node,
+	.reuse_data_on_node = reuse_vector_buffer_on_node,
+	.map_data = map_vector,
+	.unmap_data = unmap_vector,
+	.update_map = update_map,
 	.copy_methods = &vector_copy_data_methods_s,
 	.get_size = vector_interface_get_size,
 	.get_alloc_size = vector_interface_get_alloc_size,
@@ -59,9 +71,13 @@ struct starpu_data_interface_ops starpu_interface_vector_ops =
 	.interface_size = sizeof(struct starpu_vector_interface),
 	.display = display_vector_interface,
 	.pack_data = pack_vector_handle,
+	.peek_data = peek_vector_handle,
 	.unpack_data = unpack_vector_handle,
 	.describe = describe,
-	.name = "STARPU_VECTOR_INTERFACE"
+	.name = "STARPU_VECTOR_INTERFACE",
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
 static void vector_init(void *data_interface)
@@ -78,20 +94,11 @@ static void *vector_to_pointer(void *data_interface, unsigned node)
 	return (void*) vector_interface->ptr;
 }
 
-static int vector_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
-{
-	(void) node;
-	struct starpu_vector_interface *vector_interface = data_interface;
-
-	return (char*) ptr >= (char*) vector_interface->ptr &&
-		(char*) ptr < (char*) vector_interface->ptr + vector_interface->nx*vector_interface->elemsize;
-}
-
-static void register_vector_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_vector_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_vector_interface *local_interface = (struct starpu_vector_interface *)
@@ -100,14 +107,14 @@ static void register_vector_handle(starpu_data_handle_t handle, unsigned home_no
 		if (node == home_node)
 		{
 			local_interface->ptr = vector_interface->ptr;
-                        local_interface->dev_handle = vector_interface->dev_handle;
-                        local_interface->offset = vector_interface->offset;
+			local_interface->dev_handle = vector_interface->dev_handle;
+			local_interface->offset = vector_interface->offset;
 		}
 		else
 		{
 			local_interface->ptr = 0;
-                        local_interface->dev_handle = 0;
-                        local_interface->offset = 0;
+			local_interface->dev_handle = 0;
+			local_interface->offset = 0;
 		}
 
 		local_interface->id = vector_interface->id;
@@ -128,9 +135,9 @@ void starpu_vector_data_register_allocsize(starpu_data_handle_t *handleptr, int
 		.ptr = ptr,
 		.nx = nx,
 		.elemsize = elemsize,
-                .dev_handle = ptr,
+		.dev_handle = ptr,
 		.slice_base = 0,
-                .offset = 0,
+		.offset = 0,
 		.allocsize = allocsize,
 	};
 #if (!defined(STARPU_SIMGRID) && !defined(STARPU_OPENMP))
@@ -219,7 +226,7 @@ static int pack_vector_handle(starpu_data_handle_t handle, unsigned node, void *
 	return 0;
 }
 
-static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -229,6 +236,12 @@ static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void
 	STARPU_ASSERT(count == vector_interface->elemsize * vector_interface->nx);
 	memcpy((void*)vector_interface->ptr, ptr, count);
 
+	return 0;
+}
+
+static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_vector_handle(handle, node, ptr, count);
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 
 	return 0;
@@ -339,7 +352,7 @@ static starpu_ssize_t allocate_vector_buffer_on_node(void *data_interface_, unsi
 	/* update the data properly in consequence */
 	vector_interface->ptr = addr;
 	vector_interface->dev_handle = handle;
-        vector_interface->offset = 0;
+	vector_interface->offset = 0;
 
 	return allocated_memory;
 }
@@ -349,10 +362,76 @@ static void free_vector_buffer_on_node(void *data_interface, unsigned node)
 	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *) data_interface;
 
 	starpu_free_on_node(node, vector_interface->dev_handle, vector_interface->allocsize);
+	vector_interface->ptr = 0;
+	vector_interface->dev_handle = 0;
+}
+
+static void cache_vector_buffer_on_node(void *new_data_interface, void *data_interface, unsigned node STARPU_ATTRIBUTE_UNUSED)
+{
+	struct starpu_vector_interface *new_vector_interface = new_data_interface;
+	struct starpu_vector_interface *vector_interface = data_interface;
+
+	new_vector_interface->ptr = vector_interface->ptr;
+	vector_interface->ptr = 0;
+	new_vector_interface->dev_handle = vector_interface->dev_handle;
+	vector_interface->dev_handle = 0;
+	new_vector_interface->allocsize = vector_interface->allocsize;
+	STARPU_ASSERT(vector_interface->offset == 0);
+}
+
+static void reuse_vector_buffer_on_node(void *data_interface, const void *new_data_interface, unsigned node STARPU_ATTRIBUTE_UNUSED)
+{
+	struct starpu_vector_interface *vector_interface = data_interface;
+	const struct starpu_vector_interface *new_vector_interface = new_data_interface;
+
+	vector_interface->ptr = new_vector_interface->ptr;
+	vector_interface->dev_handle = new_vector_interface->dev_handle;
+	vector_interface->offset = 0;
+}
+
+static int map_vector(void *src_interface, unsigned src_node,
+		      void *dst_interface, unsigned dst_node)
+{
+	struct starpu_vector_interface *src_vector = src_interface;
+	struct starpu_vector_interface *dst_vector = dst_interface;
+	int ret;
+	uintptr_t mapped;
+
+	mapped = starpu_interface_map(src_vector->dev_handle, src_vector->offset, src_node, dst_node, src_vector->nx*src_vector->elemsize, &ret);
+	if (mapped)
+	{
+		dst_vector->dev_handle = mapped;
+		dst_vector->offset = 0;
+		if (starpu_node_get_kind(dst_node) != STARPU_OPENCL_RAM)
+			dst_vector->ptr = mapped;
+		return 0;
+	}
+	return ret;
+}
+
+static int unmap_vector(void *src_interface, unsigned src_node,
+			void *dst_interface, unsigned dst_node)
+{
+	struct starpu_vector_interface *src_vector = src_interface;
+	struct starpu_vector_interface *dst_vector = dst_interface;
+
+	int ret = starpu_interface_unmap(src_vector->dev_handle, src_vector->offset, src_node, dst_vector->dev_handle, dst_node, src_vector->nx*src_vector->elemsize);
+	dst_vector->dev_handle = 0;
+
+	return ret;
+}
+
+static int update_map(void *src_interface, unsigned src_node,
+		      void *dst_interface, unsigned dst_node)
+{
+	struct starpu_vector_interface *src_vector = src_interface;
+	struct starpu_vector_interface *dst_vector = dst_interface;
+
+	return starpu_interface_update_map(src_vector->dev_handle, src_vector->offset, src_node, dst_vector->dev_handle, dst_vector->offset, dst_node, src_vector->nx*src_vector->elemsize);
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node,
-                           void *dst_interface, unsigned dst_node, void *async_data)
+			   void *dst_interface, unsigned dst_node, void *async_data)
 {
 	struct starpu_vector_interface *src_vector = src_interface;
 	struct starpu_vector_interface *dst_vector = dst_interface;
