@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,23 +15,29 @@
  */
 
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 
 static int dummy_copy(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+static int map_void(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int unmap_void(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int update_map_void(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
 
 static const struct starpu_data_copy_methods void_copy_data_methods_s =
 {
 	.any_to_any = dummy_copy,
 };
 
-static void register_void_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_void_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static starpu_ssize_t allocate_void_buffer_on_node(void *data_interface_, unsigned dst_node);
-static int void_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_void_buffer_on_node(void *data_interface, unsigned node);
 static size_t void_interface_get_size(starpu_data_handle_t handle);
 static uint32_t footprint_void_interface_crc32(starpu_data_handle_t handle);
 static int void_compare(void *data_interface_a, void *data_interface_b);
 static void display_void_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_void_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_void_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_void_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
@@ -40,6 +46,9 @@ struct starpu_data_interface_ops starpu_interface_void_ops =
 	.register_data_handle = register_void_handle,
 	.allocate_data_on_node = allocate_void_buffer_on_node,
 	.free_data_on_node = free_void_buffer_on_node,
+	.map_data = map_void,
+	.unmap_data = unmap_void,
+	.update_map = update_map_void,
 	.copy_methods = &void_copy_data_methods_s,
 	.get_size = void_interface_get_size,
 	.footprint = footprint_void_interface_crc32,
@@ -48,19 +57,17 @@ struct starpu_data_interface_ops starpu_interface_void_ops =
 	.interface_size = 0,
 	.display = display_void_interface,
 	.pack_data = pack_void_handle,
+	.peek_data = peek_void_handle,
 	.unpack_data = unpack_void_handle,
 	.describe = describe,
-	.pointer_is_inside = void_pointer_is_inside,
-	.name = "STARPU_VOID_INTERFACE"
+	.name = "STARPU_VOID_INTERFACE",
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
-static int void_pointer_is_inside(void *data_interface STARPU_ATTRIBUTE_UNUSED, unsigned node STARPU_ATTRIBUTE_UNUSED, void *ptr STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0;
-}
-
 static void register_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
-				unsigned home_node STARPU_ATTRIBUTE_UNUSED,
+				int home_node STARPU_ATTRIBUTE_UNUSED,
 				void *data_interface STARPU_ATTRIBUTE_UNUSED)
 {
 	/* Since there is no real data to register, we don't do anything */
@@ -101,6 +108,14 @@ static int pack_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
 	return 0;
 }
 
+static int peek_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
+			      unsigned node STARPU_ATTRIBUTE_UNUSED,
+			      void *ptr STARPU_ATTRIBUTE_UNUSED,
+			      size_t count STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0;
+}
+
 static int unpack_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
 			      unsigned node STARPU_ATTRIBUTE_UNUSED,
 			      void *ptr STARPU_ATTRIBUTE_UNUSED,
@@ -120,7 +135,7 @@ static size_t void_interface_get_size(starpu_data_handle_t handle STARPU_ATTRIBU
 static starpu_ssize_t allocate_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSED,
 						   unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
 {
-	/* Successfuly allocated 0 bytes */
+	/* Successfully allocated 0 bytes */
 	return 0;
 }
 
@@ -130,6 +145,30 @@ static void free_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSE
 	/* There is no buffer actually */
 }
 
+static int map_void(void *src_interface STARPU_ATTRIBUTE_UNUSED,
+		    unsigned src_node STARPU_ATTRIBUTE_UNUSED,
+		    void *dst_interface STARPU_ATTRIBUTE_UNUSED,
+		    unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0;
+}
+
+static int unmap_void(void *src_interface STARPU_ATTRIBUTE_UNUSED,
+		      unsigned src_node STARPU_ATTRIBUTE_UNUSED,
+		      void *dst_interface STARPU_ATTRIBUTE_UNUSED,
+		      unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0;
+}
+
+static int update_map_void(void *src_interface STARPU_ATTRIBUTE_UNUSED,
+			   unsigned src_node STARPU_ATTRIBUTE_UNUSED,
+			   void *dst_interface STARPU_ATTRIBUTE_UNUSED,
+			   unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0;
+}
+
 static int dummy_copy(void *src_interface STARPU_ATTRIBUTE_UNUSED,
 			unsigned src_node STARPU_ATTRIBUTE_UNUSED,
 			void *dst_interface STARPU_ATTRIBUTE_UNUSED,
