@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2021       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -55,7 +55,7 @@ LIST_TYPE(_starpu_mem_chunk,
 
 	/** Whether StarPU automatically allocated this memory, or the application did */
 	unsigned automatically_allocated:1;
-	/** A buffer that is used for SCRATCH or reduction cannnot be used with
+	/** A buffer that is used for SCRATCH or reduction cannot be used with
 	 * filters. */
 	unsigned relaxed_coherency:2;
 	/** Whether this is the home chunk, or there is no home chunk (and it is thus always clean) */
@@ -90,6 +90,7 @@ int _starpu_allocate_memory_on_node(starpu_data_handle_t handle, struct _starpu_
 size_t _starpu_free_all_automatically_allocated_buffers(unsigned node);
 void _starpu_memchunk_recently_used(struct _starpu_mem_chunk *mc, unsigned node);
 void _starpu_memchunk_wont_use(struct _starpu_mem_chunk *m, unsigned nodec);
+void _starpu_memchunk_clean(struct _starpu_mem_chunk *mc, unsigned node);
 void _starpu_memchunk_dirty(struct _starpu_mem_chunk *mc, unsigned node);
 
 size_t _starpu_memory_reclaim_generic(unsigned node, unsigned force, size_t reclaim, enum starpu_is_prefetch is_prefetch);
