@@ -1,10 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2010       Mehdi Juhoor
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
- * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2020,2021  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,16 +33,19 @@
 #include <drivers/disk/driver_disk.h>
 #include <drivers/opencl/driver_opencl.h>
 #include <drivers/cuda/driver_cuda.h>
-#include <drivers/mic/driver_mic_source.h>
-#include <drivers/mpi/driver_mpi_source.h>
+#include <drivers/mp_common/source_common.h>
 #include <drivers/disk/driver_disk.h>
+#include <drivers/max/driver_max_fpga.h>
 #include <core/sched_policy.h>
 #include <datawizard/memory_manager.h>
 #include <datawizard/memory_nodes.h>
 #include <datawizard/malloc.h>
+#include <datawizard/datawizard.h>
 #include <core/simgrid.h>
 #include <core/task.h>
 #include <core/disk.h>
+#include <common/knobs.h>
+#include <profiling/callbacks.h>
 
 #ifdef STARPU_HAVE_HWLOC
 #include <hwloc.h>
@@ -58,6 +61,269 @@
 #include <windows.h>
 #endif
 
+static unsigned already_busy_cpus;
+
+static struct _starpu_driver_info driver_info =
+{
+	.name_upper = "CPU",
+	.name_var = "CPU",
+	.name_lower = "cpu",
+	.memory_kind = STARPU_CPU_RAM,
+	.alpha = 0.5f,
+	.wait_for_worker_initialization = 1,
+#ifdef STARPU_USE_CPU
+	.driver_ops = &_starpu_driver_cpu_ops,
+	.run_worker = _starpu_cpu_worker,
+#endif
+	.init_worker_binding = _starpu_cpu_init_worker_binding,
+	.init_worker_memory = _starpu_cpu_init_worker_memory,
+};
+
+static struct _starpu_memory_driver_info memory_driver_info =
+{
+	.name_upper = "NUMA",
+	.worker_archtype = STARPU_CPU_WORKER,
+	.ops = &_starpu_driver_cpu_node_ops,
+};
+
+/* Early library initialization, before anything else, just initialize data */
+void _starpu_cpu_preinit(void)
+{
+	_starpu_driver_info_register(STARPU_CPU_WORKER, &driver_info);
+	_starpu_memory_driver_info_register(STARPU_CPU_RAM, &memory_driver_info);
+	already_busy_cpus = 0;
+}
+
+void _starpu_cpu_busy_cpu(unsigned num)
+{
+	already_busy_cpus += num;
+}
+
+#if defined(STARPU_USE_CPU) || defined(STARPU_SIMGRID)
+/* Determine which devices we will use */
+void _starpu_init_cpu_config(struct _starpu_machine_topology *topology, struct _starpu_machine_config *config)
+{
+	int ncpu = config->conf.ncpus;
+
+	if (ncpu != 0)
+	{
+		STARPU_ASSERT_MSG(ncpu >= -1, "ncpus can not be negative and different from -1 (is is %d)", ncpu);
+
+		int nhyperthreads = topology->nhwpus / topology->nhwworker[STARPU_CPU_WORKER][0];
+		long avail_cpus = (long) (topology->nusedpus / nhyperthreads) - (long) already_busy_cpus;
+		if (avail_cpus < 0)
+			avail_cpus = 0;
+		int nth_per_core = starpu_getenv_number_default("STARPU_NTHREADS_PER_CORE", 1);
+		avail_cpus *= nth_per_core;
+
+		_starpu_topology_check_ndevices(&ncpu, avail_cpus, 1, STARPU_MAXCPUS, config->conf.reserve_ncpus, "ncpus", "CPU cores", "maxcpus");
+	}
+
+	topology->ndevices[STARPU_CPU_WORKER] = 1;
+	unsigned homogeneous = starpu_getenv_number_default("STARPU_PERF_MODEL_HOMOGENEOUS_CPU", 1);
+
+	_starpu_topology_configure_workers(topology, config,
+					   STARPU_CPU_WORKER,
+					   0, 0, homogeneous, 1,
+					   ncpu, 1, NULL, NULL);
+}
+#endif
+
+/* Bind the driver on a CPU core */
+void _starpu_cpu_init_worker_binding(struct _starpu_machine_config *config STARPU_ATTRIBUTE_UNUSED, int no_mp_config STARPU_ATTRIBUTE_UNUSED, struct _starpu_worker *workerarg)
+{
+	/* Dedicate a cpu core to that worker */
+	workerarg->bindid = _starpu_get_next_bindid(config, STARPU_THREAD_ACTIVE, NULL, 0);;
+}
+
+/* Set up memory and buses */
+void _starpu_cpu_init_worker_memory(struct _starpu_machine_config *config STARPU_ATTRIBUTE_UNUSED, int no_mp_config STARPU_ATTRIBUTE_UNUSED, struct _starpu_worker *workerarg)
+{
+	unsigned memory_node = -1;
+	int numa_logical_id = _starpu_get_logical_numa_node_worker(workerarg->workerid);
+	int numa_starpu_id =  starpu_memory_nodes_numa_hwloclogid_to_id(numa_logical_id);
+	if (numa_starpu_id < 0 || numa_starpu_id >= STARPU_MAXNUMANODES)
+		numa_starpu_id = STARPU_MAIN_RAM;
+
+#if defined(STARPU_HAVE_HWLOC) && !defined(STARPU_SIMGRID)
+	hwloc_obj_t pu_obj = hwloc_get_obj_by_type(config->topology.hwtopology, HWLOC_OBJ_PU, workerarg->bindid);
+	struct _starpu_hwloc_userdata *userdata = pu_obj->userdata;
+	userdata->pu_worker = workerarg;
+#endif
+
+	workerarg->numa_memory_node = memory_node = numa_starpu_id;
+
+	_starpu_memory_node_add_nworkers(memory_node);
+
+	_starpu_worker_drives_memory_node(workerarg, numa_starpu_id);
+
+	workerarg->memory_node = memory_node;
+}
+
+#ifdef STARPU_USE_CPU
+/* This is run from the driver thread to initialize the driver CUDA context */
+int _starpu_cpu_driver_init(struct _starpu_worker *cpu_worker)
+{
+	int devid = cpu_worker->devid;
+
+#ifdef STARPU_PROF_TOOL
+	struct starpu_prof_tool_info pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_driver_init, devid, cpu_worker->workerid, starpu_prof_tool_driver_cpu, -1, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_driver_init(&pi, NULL, NULL);
+
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_driver_init_start, devid, cpu_worker->workerid, starpu_prof_tool_driver_cpu, -1, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_driver_init_start(&pi, NULL, NULL);
+#endif
+
+	_starpu_driver_start(cpu_worker, STARPU_CPU_WORKER, 1);
+	snprintf(cpu_worker->name, sizeof(cpu_worker->name), "CPU %d", devid);
+	snprintf(cpu_worker->short_name, sizeof(cpu_worker->short_name), "CPU %d", devid);
+	starpu_pthread_setname(cpu_worker->short_name);
+
+	_STARPU_TRACE_WORKER_INIT_END(cpu_worker->workerid);
+
+	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&cpu_worker->sched_mutex);
+	cpu_worker->status = STATUS_UNKNOWN;
+	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&cpu_worker->sched_mutex);
+
+	/* tell the main thread that we are ready */
+	STARPU_PTHREAD_MUTEX_LOCK(&cpu_worker->mutex);
+	cpu_worker->worker_is_initialized = 1;
+	STARPU_PTHREAD_COND_SIGNAL(&cpu_worker->ready_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&cpu_worker->mutex);
+
+#ifdef STARPU_PROF_TOOL
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_driver_init_end, devid, cpu_worker->workerid, starpu_prof_tool_driver_cpu, -1, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_driver_init_end(&pi, NULL, NULL);
+#endif
+	return 0;
+}
+
+int _starpu_cpu_driver_deinit(struct _starpu_worker *cpu_worker)
+{
+	_STARPU_TRACE_WORKER_DEINIT_START;
+
+	unsigned memnode = cpu_worker->memory_node;
+	_starpu_datawizard_handle_all_pending_node_data_requests(memnode);
+
+	/* In case there remains some memory that was automatically
+	 * allocated by StarPU, we release it now. Note that data
+	 * coherency is not maintained anymore at that point ! */
+	_starpu_free_all_automatically_allocated_buffers(memnode);
+
+	cpu_worker->worker_is_initialized = 0;
+	_STARPU_TRACE_WORKER_DEINIT_END(STARPU_CPU_WORKER);
+
+#ifdef STARPU_PROF_TOOL
+ 	int workerid = cpu_worker->workerid;
+	struct starpu_prof_tool_info pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_driver_deinit, workerid, workerid, starpu_prof_tool_driver_cpu, memnode, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_driver_deinit(&pi, NULL, NULL);
+#endif
+
+	return 0;
+}
+#endif /* STARPU_USE_CPU */
+
+uintptr_t _starpu_cpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
+{
+	uintptr_t addr = 0;
+	_starpu_malloc_flags_on_node(dst_node, (void**) &addr, size,
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+				     /* without memcpy_peer, we can not
+				      * allocated pinned memory, since it
+				      * requires waiting for a task, and we
+				      * may be called with a spinlock held
+				      */
+				     flags & ~STARPU_MALLOC_PINNED
+#else
+				     flags
+#endif
+				     );
+	return addr;
+}
+
+void _starpu_cpu_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
+{
+	_starpu_free_flags_on_node(dst_node, (void*)addr, size,
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+				   flags & ~STARPU_MALLOC_PINNED
+#else
+				   flags
+#endif
+				   );
+}
+
+int _starpu_cpu_copy_interface(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_CPU_RAM);
+
+	int ret = 0;
+	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
+	if (copy_methods->ram_to_ram)
+		copy_methods->ram_to_ram(src_interface, src_node, dst_interface, dst_node);
+	else
+	{
+		STARPU_ASSERT_MSG(copy_methods->any_to_any, "the interface '%s' does define neither ram_to_ram nor any_to_any copy method", handle->ops->name);
+		copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, req ? &req->async_channel : NULL);
+	}
+	return ret;
+}
+
+int _starpu_cpu_copy_data(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+{
+	int src_kind = starpu_node_get_kind(src_node);
+	int dst_kind = starpu_node_get_kind(dst_node);
+	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_CPU_RAM);
+
+	(void) async_channel;
+
+	memcpy((void *) (dst + dst_offset), (void *) (src + src_offset), size);
+	return 0;
+}
+
+int _starpu_cpu_is_direct_access_supported(unsigned node, unsigned handling_node)
+{
+	(void) node;
+	(void) handling_node;
+	return 1;
+}
+
+uintptr_t _starpu_cpu_map(uintptr_t src, size_t src_offset, unsigned src_node, unsigned dst_node, size_t size, int *ret)
+{
+	(void) src_node;
+	(void) dst_node;
+	(void) size;
+
+	*ret = 0;
+	return src + src_offset;
+}
+
+int _starpu_cpu_unmap(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, unsigned dst_node, size_t size)
+{
+	(void) src;
+	(void) src_offset;
+	(void) src_node;
+	(void) dst;
+	(void) dst_node;
+	(void) size;
+
+	return 0;
+}
+
+int _starpu_cpu_update_map(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size)
+{
+	(void) src;
+	(void) src_offset;
+	(void) src_node;
+	(void) dst;
+	(void) dst_offset;
+	(void) dst_node;
+	(void) size;
+
+	/* Memory mappings are cache-coherent */
+	return 0;
+}
 
 #ifdef STARPU_USE_CPU
 /* Actually launch the job on a cpu worker.
@@ -70,6 +336,10 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 	int profiling = starpu_profiling_status_get();
 	struct starpu_task *task = j->task;
 	struct starpu_codelet *cl = task->cl;
+#ifdef STARPU_PROF_TOOL
+	struct starpu_prof_tool_info pi;
+	int devid = cpu_args->devid;
+#endif
 
 	STARPU_ASSERT(cl);
 
@@ -85,18 +355,23 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 	/* Give profiling variable */
 	_starpu_driver_start_job(cpu_args, j, perf_arch, rank, profiling);
 
+	_starpu_cl_func_t func = _starpu_task_get_cpu_nth_implementation(cl, j->nimpl);
+
 	/* In case this is a Fork-join parallel task, the worker does not
 	 * execute the kernel at all. */
 	if ((rank == 0) || (cl->type != STARPU_FORKJOIN))
 	{
-		_starpu_cl_func_t func = _starpu_task_get_cpu_nth_implementation(cl, j->nimpl);
 		if (is_parallel_task && cl->type == STARPU_FORKJOIN)
 			/* bind to parallel worker */
 			_starpu_bind_thread_on_cpus(_starpu_get_combined_worker_struct(j->combined_workerid));
 		STARPU_ASSERT_MSG(func, "when STARPU_CPU is defined in 'where', cpu_func or cpu_funcs has to be defined");
 		if (_starpu_get_disable_kernels() <= 0)
 		{
-			_STARPU_TRACE_START_EXECUTING();
+			_STARPU_TRACE_START_EXECUTING(j);
+#ifdef STARPU_PROF_TOOL
+			pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_start_cpu_exec, devid, worker_task->workerid, starpu_prof_tool_driver_cpu, -1, (void*)func);
+			starpu_prof_tool_callbacks.starpu_prof_tool_event_start_cpu_exec(&pi, NULL, NULL);
+#endif
 #ifdef STARPU_SIMGRID
 			if (cl->flags & STARPU_CODELET_SIMGRID_EXECUTE)
 				func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
@@ -107,11 +382,26 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 				_SIMGRID_TIMER_END;
 			}
 			else
-				_starpu_simgrid_submit_job(cpu_args->workerid, j, perf_arch, NAN, NULL);
+			{
+				struct _starpu_sched_ctx *sched_ctx = _starpu_sched_ctx_get_sched_ctx_for_worker_and_job(cpu_args, j);
+				_starpu_simgrid_submit_job(cpu_args->workerid, sched_ctx->id, j, perf_arch, NAN, NAN, NULL);
+			}
 #else
-			func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
+#ifdef STARPU_PAPI
+			if (rank == 0)
+				_starpu_profiling_papi_task_start_counters(task);
 #endif
-			_STARPU_TRACE_END_EXECUTING();
+			func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
+#ifdef STARPU_PAPI
+			if (rank == 0)
+				_starpu_profiling_papi_task_stop_counters(task);
+#endif
+#endif
+#ifdef STARPU_PROF_TOOL
+			pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_cpu_exec, devid, worker_task->workerid, starpu_prof_tool_driver_cpu, -1, (void*)func);
+			starpu_prof_tool_callbacks.starpu_prof_tool_event_end_cpu_exec(&pi, NULL, NULL);
+#endif
+			_STARPU_TRACE_END_EXECUTING(j);
 		}
 		if (is_parallel_task && cl->type == STARPU_FORKJOIN)
 			/* rebind to single CPU */
@@ -119,14 +409,26 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 	}
 	else
 	{
-		_STARPU_TRACE_START_EXECUTING();
+#ifdef STARPU_PROF_TOOL
+		pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_cpu_exec, devid, worker_task->workerid, starpu_prof_tool_driver_cpu, -1, (void*)func);
+		starpu_prof_tool_callbacks.starpu_prof_tool_event_end_cpu_exec(&pi, NULL, NULL);
+#endif
+		_STARPU_TRACE_START_EXECUTING(j);
 	}
 
 	if (is_parallel_task)
 	{
+		_STARPU_TRACE_START_PARALLEL_SYNC(j);
 		STARPU_PTHREAD_BARRIER_WAIT(&j->after_work_barrier);
+		_STARPU_TRACE_END_PARALLEL_SYNC(j);
 		if (rank != 0)
-			_STARPU_TRACE_END_EXECUTING();
+		{
+#ifdef STARPU_PROF_TOOL
+			pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_cpu_exec, devid, worker_task->workerid, starpu_prof_tool_driver_cpu, -1, (void*)func);
+			starpu_prof_tool_callbacks.starpu_prof_tool_event_end_cpu_exec(&pi, NULL, NULL);
+#endif
+			_STARPU_TRACE_END_EXECUTING(j);
+		}
 	}
 
 	_starpu_driver_end_job(cpu_args, j, perf_arch, rank, profiling);
@@ -173,29 +475,6 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 	return 0;
 }
 
-int _starpu_cpu_driver_init(struct _starpu_worker *cpu_worker)
-{
-	int devid = cpu_worker->devid;
-
-	_starpu_driver_start(cpu_worker, _STARPU_FUT_CPU_KEY, 1);
-	snprintf(cpu_worker->name, sizeof(cpu_worker->name), "CPU %d", devid);
-	snprintf(cpu_worker->short_name, sizeof(cpu_worker->short_name), "CPU %d", devid);
-	starpu_pthread_setname(cpu_worker->short_name);
-
-	_STARPU_TRACE_WORKER_INIT_END(cpu_worker->workerid);
-
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&cpu_worker->sched_mutex);
-	cpu_worker->status = STATUS_UNKNOWN;
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&cpu_worker->sched_mutex);
-
-	/* tell the main thread that we are ready */
-	STARPU_PTHREAD_MUTEX_LOCK(&cpu_worker->mutex);
-	cpu_worker->worker_is_initialized = 1;
-	STARPU_PTHREAD_COND_SIGNAL(&cpu_worker->ready_cond);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&cpu_worker->mutex);
-	return 0;
-}
-
 static int _starpu_cpu_driver_execute_task(struct _starpu_worker *cpu_worker, struct starpu_task *task, struct _starpu_job *j)
 {
 	int res;
@@ -241,7 +520,14 @@ static int _starpu_cpu_driver_execute_task(struct _starpu_worker *cpu_worker, st
 
 	_starpu_set_current_task(j->task);
 	cpu_worker->current_task = j->task;
+	j->workerid = cpu_worker->workerid;
 
+#ifdef STARPU_BUBBLE_VERBOSE
+	struct timespec tp;
+	clock_gettime(CLOCK_MONOTONIC, &tp);
+	unsigned long long timestamp = 1000000000ULL*tp.tv_sec + tp.tv_nsec;
+	_STARPU_DEBUG("{%llu} [%s(%p)]\n", timestamp, starpu_task_get_name(task), task);
+#endif
 	res = execute_job_on_cpu(j, task, cpu_worker, rank, perf_arch);
 
 	_starpu_set_current_task(NULL);
@@ -273,16 +559,17 @@ static int _starpu_cpu_driver_execute_task(struct _starpu_worker *cpu_worker, st
 	return 0;
 }
 
+/* One iteration of the main driver loop */
 int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 {
 	unsigned memnode = cpu_worker->memory_node;
 	int workerid = cpu_worker->workerid;
-
+#ifdef STARPU_PROF_TOOL
+	struct starpu_prof_tool_info pi;
+#endif
 	int res;
-
 	struct _starpu_job *j;
 	struct starpu_task *task = NULL, *pending_task;
-
 	int rank = 0;
 
 #ifdef STARPU_SIMGRID
@@ -296,30 +583,37 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 		int ret;
 		STARPU_RMB();
 		_STARPU_TRACE_END_PROGRESS(memnode);
+#ifdef STARPU_PROF_TOOL
+		pi = _starpu_prof_tool_get_info_d(starpu_prof_tool_event_end_transfer, workerid, workerid, starpu_prof_tool_driver_cpu, memnode, cpu_worker->nb_buffers_totransfer, cpu_worker->nb_buffers_transferred);
+		starpu_prof_tool_callbacks.starpu_prof_tool_event_end_transfer(&pi, NULL, NULL);
+#endif
 		j = _starpu_get_job_associated_to_task(pending_task);
 
 		_starpu_fetch_task_input_tail(pending_task, j, cpu_worker);
-		_starpu_set_worker_status(cpu_worker, STATUS_UNKNOWN);
 		/* Reset it */
 		cpu_worker->task_transferring = NULL;
 
 		ret = _starpu_cpu_driver_execute_task(cpu_worker, pending_task, j);
 		_STARPU_TRACE_START_PROGRESS(memnode);
+#ifdef STARPU_PROF_TOOL
+		pi = _starpu_prof_tool_get_info_d(starpu_prof_tool_event_start_transfer, workerid, workerid, starpu_prof_tool_driver_cpu, memnode, cpu_worker->nb_buffers_totransfer, cpu_worker->nb_buffers_transferred);
+		starpu_prof_tool_callbacks.starpu_prof_tool_event_start_transfer(&pi, NULL, NULL);
+#endif
 		return ret;
 	}
 
-	res = __starpu_datawizard_progress(1, 1);
+	res = __starpu_datawizard_progress(_STARPU_DATAWIZARD_DO_ALLOC, 1);
 
 	if (!pending_task)
 		task = _starpu_get_worker_task(cpu_worker, workerid, memnode);
 
 #ifdef STARPU_SIMGRID
- #ifndef STARPU_OPENMP
+#ifndef STARPU_OPENMP
 	if (!res && !task)
 		/* No progress, wait */
 		starpu_pthread_wait_wait(&cpu_worker->wait);
- #else
-  #if SIMGRID_VERSION >= 31800
+#else
+#if SIMGRID_VERSION >= 31800
 	if (!res && !task)
 	{
 		/* No progress, wait (but at most 1s for OpenMP support) */
@@ -329,11 +623,11 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 		abstime.tv_sec++;
 		starpu_pthread_wait_timedwait(&cpu_worker->wait, &abstime);
 	}
-  #else
+#else
 	/* Previous simgrid versions don't really permit to use wait_timedwait in C */
 	starpu_sleep(0.001);
-  #endif
- #endif
+#endif
+#endif
 #endif
 
 	if (!task)
@@ -348,13 +642,17 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	 * job. */
 
 	/* can a cpu perform that task ? */
-	if (!_STARPU_CPU_MAY_PERFORM(j))
+	if (!_STARPU_MAY_PERFORM(j, CPU))
 	{
 		/* put it and the end of the queue ... XXX */
 		_starpu_push_task_to_workers(task);
 		return 0;
 	}
 
+#ifdef STARPU_PROF_TOOL
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_transfer, workerid, workerid, starpu_prof_tool_driver_cpu, memnode, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_end_transfer(&pi, NULL, NULL);
+#endif
 	_STARPU_TRACE_END_PROGRESS(memnode);
 	/* Get the rank in case it is a parallel task */
 	if (j->task_size > 1)
@@ -384,43 +682,42 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	else
 	{
 		int ret = _starpu_cpu_driver_execute_task(cpu_worker, task, j);
+#ifdef STARPU_PROF_TOOL
+		pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_transfer, workerid, cpu_worker->workerid, starpu_prof_tool_driver_cpu, memnode, NULL);
+		starpu_prof_tool_callbacks.starpu_prof_tool_event_end_transfer(&pi, NULL, NULL);
+#endif
 		_STARPU_TRACE_END_PROGRESS(memnode);
 		return ret;
 	}
+#ifdef STARPU_PROF_TOOL
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_transfer, workerid, cpu_worker->workerid, starpu_prof_tool_driver_cpu, memnode, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_end_transfer(&pi, NULL, NULL);
+#endif
 	_STARPU_TRACE_END_PROGRESS(memnode);
 	return 0;
 }
 
-int _starpu_cpu_driver_deinit(struct _starpu_worker *cpu_worker)
-{
-	_STARPU_TRACE_WORKER_DEINIT_START;
-
-	unsigned memnode = cpu_worker->memory_node;
-	_starpu_handle_all_pending_node_data_requests(memnode);
-
-	/* In case there remains some memory that was automatically
-	 * allocated by StarPU, we release it now. Note that data
-	 * coherency is not maintained anymore at that point ! */
-	_starpu_free_all_automatically_allocated_buffers(memnode);
-
-	cpu_worker->worker_is_initialized = 0;
-	_STARPU_TRACE_WORKER_DEINIT_END(_STARPU_FUT_CPU_KEY);
-
-	return 0;
-}
-
 void *_starpu_cpu_worker(void *arg)
 {
 	struct _starpu_worker *worker = arg;
 
 	_starpu_cpu_driver_init(worker);
 	_STARPU_TRACE_START_PROGRESS(worker->memory_node);
+#ifdef STARPU_PROF_TOOL
+	struct starpu_prof_tool_info pi;
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_start_transfer, worker->workerid, worker->workerid, starpu_prof_tool_driver_cpu, worker->memory_node, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_start_transfer(&pi, NULL, NULL);
+#endif
 	while (_starpu_machine_is_running())
 	{
 		_starpu_may_pause();
 		_starpu_cpu_driver_run_once(worker);
 	}
 	_STARPU_TRACE_END_PROGRESS(worker->memory_node);
+#ifdef STARPU_PROF_TOOL
+	pi = _starpu_prof_tool_get_info(starpu_prof_tool_event_end_transfer, worker->workerid, worker->workerid, starpu_prof_tool_driver_cpu, worker->memory_node, NULL);
+	starpu_prof_tool_callbacks.starpu_prof_tool_event_end_transfer(&pi, NULL, NULL);
+#endif
 	_starpu_cpu_driver_deinit(worker);
 
 	return NULL;
@@ -428,167 +725,48 @@ void *_starpu_cpu_worker(void *arg)
 
 int _starpu_cpu_driver_run(struct _starpu_worker *worker)
 {
-	worker->set = NULL;
-	worker->worker_is_initialized = 0;
 	_starpu_cpu_worker(worker);
 
 	return 0;
 }
 
+int _starpu_cpu_driver_set_devid(struct starpu_driver *driver, struct _starpu_worker *worker)
+{
+	driver->id.cpu_id = worker->devid;
+
+	return 0;
+}
+
+int _starpu_cpu_driver_is_devid(struct starpu_driver *driver, struct _starpu_worker *worker)
+{
+	return driver->id.cpu_id == worker->devid;
+}
+
 struct _starpu_driver_ops _starpu_driver_cpu_ops =
 {
 	.init = _starpu_cpu_driver_init,
 	.run = _starpu_cpu_driver_run,
 	.run_once = _starpu_cpu_driver_run_once,
-	.deinit = _starpu_cpu_driver_deinit
+	.deinit = _starpu_cpu_driver_deinit,
+	.set_devid = _starpu_cpu_driver_set_devid,
+	.is_devid = _starpu_cpu_driver_is_devid,
 };
 #endif /* STARPU_USE_CPU */
 
-int _starpu_cpu_copy_interface(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
-{
-	int src_kind = starpu_node_get_kind(src_node);
-	int dst_kind = starpu_node_get_kind(dst_node);
-	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_CPU_RAM);
-
-	int ret = 0;
-	const struct starpu_data_copy_methods *copy_methods = handle->ops->copy_methods;
-	if (copy_methods->ram_to_ram)
-		copy_methods->ram_to_ram(src_interface, src_node, dst_interface, dst_node);
-	else
-		copy_methods->any_to_any(src_interface, src_node, dst_interface, dst_node, req ? &req->async_channel : NULL);
-	return ret;
-}
-
-int _starpu_cpu_copy_data(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
-{
-	int src_kind = starpu_node_get_kind(src_node);
-	int dst_kind = starpu_node_get_kind(dst_node);
-	STARPU_ASSERT(src_kind == STARPU_CPU_RAM && dst_kind == STARPU_CPU_RAM);
-
-	(void) async_channel;
-
-	memcpy((void *) (dst + dst_offset), (void *) (src + src_offset), size);
-	return 0;
-}
-
-int _starpu_cpu_is_direct_access_supported(unsigned node, unsigned handling_node)
-{
-	(void) node;
-	(void) handling_node;
-	return 1;
-}
-
-uintptr_t _starpu_cpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
-{
-	uintptr_t addr = 0;
-	_starpu_malloc_flags_on_node(dst_node, (void**) &addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
-				     /* without memcpy_peer, we can not
-				      * allocated pinned memory, since it
-				      * requires waiting for a task, and we
-				      * may be called with a spinlock held
-				      */
-				     flags & ~STARPU_MALLOC_PINNED
-#else
-				     flags
-#endif
-				     );
-	return addr;
-}
-
-void _starpu_cpu_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
-{
-	_starpu_free_flags_on_node(dst_node, (void*)addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
-				   flags & ~STARPU_MALLOC_PINNED
-#else
-				   flags
-#endif
-				   );
-}
-
 struct _starpu_node_ops _starpu_driver_cpu_node_ops =
 {
-	.copy_interface_to[STARPU_UNUSED] = NULL,
-	.copy_interface_to[STARPU_CPU_RAM] = _starpu_cpu_copy_interface,
-#ifdef STARPU_USE_CUDA
-	.copy_interface_to[STARPU_CUDA_RAM] = _starpu_cuda_copy_interface_from_cpu_to_cuda,
-#else
-	.copy_interface_to[STARPU_CUDA_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_OPENCL
-	.copy_interface_to[STARPU_OPENCL_RAM] = _starpu_opencl_copy_interface_from_cpu_to_opencl,
-#else
-	.copy_interface_to[STARPU_OPENCL_RAM] = NULL,
-#endif
-	.copy_interface_to[STARPU_DISK_RAM] = _starpu_disk_copy_interface_from_cpu_to_disk,
-#ifdef STARPU_USE_MIC
-	.copy_interface_to[STARPU_MIC_RAM] = _starpu_mic_copy_interface_from_cpu_to_mic,
-#else
-	.copy_interface_to[STARPU_MIC_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_MPI_MASTER_SLAVE
-	.copy_interface_to[STARPU_MPI_MS_RAM] = _starpu_mpi_copy_interface_from_cpu_to_mpi,
-#else
-	.copy_interface_to[STARPU_MPI_MS_RAM] = NULL,
-#endif
+	.name = "cpu driver",
 
-	.copy_data_to[STARPU_UNUSED] = NULL,
-	.copy_data_to[STARPU_CPU_RAM] = _starpu_cpu_copy_data,
-#ifdef STARPU_USE_CUDA
-	.copy_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy_data_from_cpu_to_cuda,
-#else
-	.copy_data_to[STARPU_CUDA_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_OPENCL
-	.copy_data_to[STARPU_OPENCL_RAM] = _starpu_opencl_copy_data_from_cpu_to_opencl,
-#else
-	.copy_data_to[STARPU_OPENCL_RAM] = NULL,
-#endif
-	.copy_data_to[STARPU_DISK_RAM] = _starpu_disk_copy_data_from_cpu_to_disk,
-#ifdef STARPU_USE_MIC
-	.copy_data_to[STARPU_MIC_RAM] = _starpu_mic_copy_data_from_cpu_to_mic,
-#else
-	.copy_data_to[STARPU_MIC_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_MPI_MASTER_SLAVE
-	.copy_data_to[STARPU_MPI_MS_RAM] = _starpu_mpi_copy_data_from_cpu_to_mpi,
-#else
-	.copy_data_to[STARPU_MPI_MS_RAM] = NULL,
-#endif
-
-	.copy2d_data_to[STARPU_UNUSED] = NULL,
-	.copy2d_data_to[STARPU_CPU_RAM] = NULL,
-#ifdef STARPU_USE_CUDA
-	.copy2d_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy2d_data_from_cpu_to_cuda,
-#else
-	.copy2d_data_to[STARPU_CUDA_RAM] = NULL,
-#endif
-	.copy2d_data_to[STARPU_OPENCL_RAM] = NULL,
-	.copy2d_data_to[STARPU_DISK_RAM] = NULL,
-	.copy2d_data_to[STARPU_MIC_RAM] = NULL,
-	.copy2d_data_to[STARPU_MPI_MS_RAM] = NULL,
-
-	.copy3d_data_to[STARPU_UNUSED] = NULL,
-	.copy3d_data_to[STARPU_CPU_RAM] = NULL,
-#if 0
-#ifdef STARPU_USE_CUDA
-	.copy3d_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy3d_data_from_cpu_to_cuda,
-#else
-	.copy3d_data_to[STARPU_CUDA_RAM] = NULL,
-#endif
-#else
-	.copy3d_data_to[STARPU_CUDA_RAM] = NULL,
-#endif
-	.copy3d_data_to[STARPU_OPENCL_RAM] = NULL,
-	.copy3d_data_to[STARPU_DISK_RAM] = NULL,
-	.copy3d_data_to[STARPU_MIC_RAM] = NULL,
-	.copy3d_data_to[STARPU_MPI_MS_RAM] = NULL,
-
-	.wait_request_completion = NULL,
-	.test_request_completion = NULL,
-	.is_direct_access_supported = _starpu_cpu_is_direct_access_supported,
 	.malloc_on_node = _starpu_cpu_malloc_on_node,
 	.free_on_node = _starpu_cpu_free_on_node,
-	.name = "cpu driver"
+
+	.is_direct_access_supported = _starpu_cpu_is_direct_access_supported,
+
+	.copy_interface_to[STARPU_CPU_RAM] = _starpu_cpu_copy_interface,
+
+	.copy_data_to[STARPU_CPU_RAM] = _starpu_cpu_copy_data,
+
+	.map[STARPU_CPU_RAM] = _starpu_cpu_map,
+	.unmap[STARPU_CPU_RAM] = _starpu_cpu_unmap,
+	.update_map[STARPU_CPU_RAM] = _starpu_cpu_update_map,
 };
