@@ -34,9 +34,12 @@
 #ifdef HAVE_CUDA_GL_INTEROP_H
 #include <cuda_gl_interop.h>
 #endif
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
+#ifdef STARPU_HAVE_NVML_H
 #include <nvml.h>
 #endif
+#ifdef HAVE_DLOPEN
+#include <dlfcn.h>
+#endif
 #ifdef STARPU_USE_CUDA
 #include <starpu_cublas_v2.h>
 #include <starpu_cusolver.h>
@@ -78,8 +81,19 @@
 static int ncudagpus = -1;
 
 static size_t global_mem[STARPU_MAXCUDADEVS];
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
+#ifdef STARPU_HAVE_NVML_H
 static nvmlDevice_t nvmlDev[STARPU_MAXCUDADEVS];
+__typeof__(nvmlInit) *_starpu_nvmlInit;
+__typeof__(nvmlDeviceGetNvLinkState) *_starpu_nvmlDeviceGetNvLinkState;
+__typeof__(nvmlDeviceGetNvLinkRemotePciInfo) *_starpu_nvmlDeviceGetNvLinkRemotePciInfo;
+__typeof__(nvmlDeviceGetHandleByIndex) *_starpu_nvmlDeviceGetHandleByIndex;
+__typeof__(nvmlDeviceGetHandleByPciBusId) *_starpu_nvmlDeviceGetHandleByPciBusId;
+__typeof__(nvmlDeviceGetIndex) *_starpu_nvmlDeviceGetIndex;
+__typeof__(nvmlDeviceGetPciInfo) *_starpu_nvmlDeviceGetPciInfo;
+__typeof__(nvmlDeviceGetUUID) *_starpu_nvmlDeviceGetUUID;
+#ifdef HAVE_DECL_NVMLDEVICEGETTOTALENERGYCONSUMPTION
+__typeof__(nvmlDeviceGetTotalEnergyConsumption) *_starpu_nvmlDeviceGetTotalEnergyConsumption;
+#endif
 #endif
 int _starpu_cuda_bus_ids[STARPU_MAXCUDADEVS+STARPU_MAXNUMANODES][STARPU_MAXCUDADEVS+STARPU_MAXNUMANODES];
 #ifdef STARPU_USE_CUDA
@@ -234,19 +248,54 @@ void _starpu_cuda_discover_devices(struct _starpu_machine_config *config)
 	if (STARPU_UNLIKELY(cures != cudaSuccess))
 		cnt = 0;
 	config->topology.nhwdevices[STARPU_CUDA_WORKER] = cnt;
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
-	nvmlInit();
+#ifdef STARPU_HAVE_NVML_H
+	void *nvml = dlopen("libnvidia-ml.so.1", RTLD_LAZY);
+
+	if (nvml)
+	{
+		_starpu_nvmlInit = dlsym(nvml, "nvmlInit_v2");
+		if (!_starpu_nvmlInit)
+			_starpu_nvmlInit = dlsym(nvml, "nvmlInit");
+	}
+	if (_starpu_nvmlInit)
+	{
+		_starpu_nvmlDeviceGetNvLinkState = dlsym(nvml, "nvmlDeviceGetNvLinkState");
+		_starpu_nvmlDeviceGetHandleByIndex = dlsym(nvml, "nvmlDeviceGetHandleByIndex_v2");
+		if (!_starpu_nvmlDeviceGetHandleByIndex)
+			_starpu_nvmlDeviceGetHandleByIndex = dlsym(nvml, "nvmlDeviceGetHandleByIndex");
+		_starpu_nvmlDeviceGetNvLinkRemotePciInfo = dlsym(nvml, "nvmlDeviceGetNvLinkRemotePciInfo_v2");
+		if (!_starpu_nvmlDeviceGetNvLinkRemotePciInfo)
+			_starpu_nvmlDeviceGetNvLinkRemotePciInfo = dlsym(nvml, "nvmlDeviceGetNvLinkRemotePciInfo");
+		_starpu_nvmlDeviceGetHandleByPciBusId = dlsym(nvml, "nvmlDeviceGetHandleByPciBusId_v2");
+		if (!_starpu_nvmlDeviceGetHandleByPciBusId)
+			_starpu_nvmlDeviceGetHandleByPciBusId = dlsym(nvml, "nvmlDeviceGetHandleByPciBusId");
+		_starpu_nvmlDeviceGetIndex = dlsym(nvml, "nvmlDeviceGetIndex");
+		_starpu_nvmlDeviceGetPciInfo = dlsym(nvml, "nvmlDeviceGetPciInfo_v3");
+		if (!_starpu_nvmlDeviceGetPciInfo)
+			_starpu_nvmlDeviceGetPciInfo = dlsym(nvml, "nvmlDeviceGetPciInfo_v2");
+		if (!_starpu_nvmlDeviceGetPciInfo)
+			_starpu_nvmlDeviceGetPciInfo = dlsym(nvml, "nvmlDeviceGetPciInfo");
+		_starpu_nvmlDeviceGetUUID = dlsym(nvml, "nvmlDeviceGetUUID");
+#ifdef HAVE_DECL_NVMLDEVICEGETTOTALENERGYCONSUMPTION
+		_starpu_nvmlDeviceGetTotalEnergyConsumption = dlsym(nvml, "nvmlDeviceGetTotalEnergyConsumption");
+#endif
+		_starpu_nvmlInit();
+	}
 #endif
 #endif
 }
 
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
-static int _starpu_cuda_direct_link(unsigned devid1, unsigned devid2)
+#ifdef STARPU_HAVE_NVML_H
+static int _starpu_cuda_direct_link(struct _starpu_machine_config *config, unsigned devid1, unsigned devid2)
 {
 	unsigned i;
 	struct cudaDeviceProp props_dev1;
 	struct cudaDeviceProp props_dev2;
 	cudaError_t cures;
+	int nvswitch = 0;
+
+	if (!_starpu_nvmlDeviceGetNvLinkState || !_starpu_nvmlDeviceGetNvLinkRemotePciInfo)
+		return 0;
 
 	cures = cudaGetDeviceProperties(&props_dev1, devid1);
 	if (cures != cudaSuccess)
@@ -263,14 +312,27 @@ static int _starpu_cuda_direct_link(unsigned devid1, unsigned devid2)
 	for (i = 0; i < NVML_NVLINK_MAX_LINKS; i++) {
 		nvmlEnableState_t active;
 		nvmlReturn_t ret;
-		ret = nvmlDeviceGetNvLinkState(nvml_dev1, i, &active);
+		ret = _starpu_nvmlDeviceGetNvLinkState(nvml_dev1, i, &active);
 		if (ret == NVML_ERROR_NOT_SUPPORTED)
 			continue;
 		if (active != NVML_FEATURE_ENABLED)
 			continue;
 
 		nvmlPciInfo_t pci;
-		nvmlDeviceGetNvLinkRemotePciInfo(nvml_dev1, i, &pci);
+		_starpu_nvmlDeviceGetNvLinkRemotePciInfo(nvml_dev1, i, &pci);
+
+		hwloc_obj_t obj = hwloc_get_pcidev_by_busid(config->topology.hwtopology,
+				pci.domain, pci.bus, pci.device, 0);
+		if (obj && obj->type == HWLOC_OBJ_PCI_DEVICE
+			&& (obj->attr->pcidev.class_id >> 8 == 0x06)
+			&& (obj->attr->pcidev.vendor_id == 0x10de))
+		{
+			/* This is an NVIDIA PCI bridge, i.e. an NVSwitch */
+			/* NVSwitch */
+			nvswitch = 1;
+			break;
+		}
+
 		if ((int) pci.domain == props_dev2.pciDomainID &&
 		    (int) pci.bus == props_dev2.pciBusID &&
 		    (int) pci.device == props_dev2.pciDeviceID)
@@ -278,7 +340,44 @@ static int _starpu_cuda_direct_link(unsigned devid1, unsigned devid2)
 			return 1;
 	}
 
-	/* No direct NVLink found */
+	if (!nvswitch)
+	{
+		/* No direct NVLink or NVSwitch found for dev1 */
+		return 0;
+	}
+
+	nvmlDevice_t nvml_dev2 = _starpu_cuda_get_nvmldev(&props_dev2);
+
+	if (!nvml_dev2)
+		return 0;
+
+	for (i = 0; i < NVML_NVLINK_MAX_LINKS; i++)
+	{
+		nvmlEnableState_t active;
+		nvmlReturn_t ret;
+		ret = _starpu_nvmlDeviceGetNvLinkState(nvml_dev2, i, &active);
+		if (ret == NVML_ERROR_NOT_SUPPORTED)
+			continue;
+		if (active != NVML_FEATURE_ENABLED)
+			continue;
+
+		nvmlPciInfo_t pci;
+		_starpu_nvmlDeviceGetNvLinkRemotePciInfo(nvml_dev2, i, &pci);
+
+		hwloc_obj_t obj = hwloc_get_pcidev_by_busid(config->topology.hwtopology,
+				pci.domain, pci.bus, pci.device, 0);
+		if (obj && obj->type == HWLOC_OBJ_PCI_DEVICE
+			&& (obj->attr->pcidev.class_id >> 8 == 0x06)
+			&& (obj->attr->pcidev.vendor_id == 0x10de))
+		{
+			/* This is an NVIDIA PCI bridge, i.e. an NVSwitch */
+			/* NVSwitch */
+			/* TODO: follow answers to https://forums.developer.nvidia.com/t/how-to-distinguish-different-nvswitch/241983 */
+			return 1;
+		}
+	}
+
+	/* No NVSwitch found for dev2 */
 	return 0;
 }
 #endif
@@ -534,8 +633,8 @@ void _starpu_cuda_init_worker_memory(struct _starpu_machine_config *config, int
 					_starpu_cuda_bus_ids[devid2+STARPU_MAXNUMANODES][devid+STARPU_MAXNUMANODES] = bus21;
 					_starpu_cuda_bus_ids[devid+STARPU_MAXNUMANODES][devid2+STARPU_MAXNUMANODES] = bus12;
 #ifndef STARPU_SIMGRID
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
-					if (_starpu_cuda_direct_link(devid, devid2))
+#ifdef STARPU_HAVE_NVML_H
+					if (_starpu_cuda_direct_link(config, devid, devid2))
 					{
 						starpu_bus_set_ngpus(bus21, 1);
 						starpu_bus_set_ngpus(bus12, 1);
@@ -844,14 +943,14 @@ static void deinit_worker_context(unsigned workerid, unsigned devid STARPU_ATTRI
 #endif /* STARPU_SIMGRID */
 }
 
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
+#ifdef STARPU_HAVE_NVML_H
 nvmlDevice_t _starpu_cuda_get_nvmldev(struct cudaDeviceProp *dev_props)
 {
 	char busid[13];
 	nvmlDevice_t ret;
 
 	snprintf(busid, sizeof(busid), "%04x:%02x:%02x.0", dev_props->pciDomainID, dev_props->pciBusID, dev_props->pciDeviceID);
-	if (nvmlDeviceGetHandleByPciBusId(busid, &ret) != NVML_SUCCESS)
+	if (!_starpu_nvmlDeviceGetHandleByPciBusId || _starpu_nvmlDeviceGetHandleByPciBusId(busid, &ret) != NVML_SUCCESS)
 		ret = NULL;
 
 	return ret;
@@ -937,7 +1036,7 @@ int _starpu_cuda_driver_init(struct _starpu_worker *worker)
 
 #if defined(STARPU_HAVE_BUSID) && !defined(STARPU_SIMGRID)
 #if defined(STARPU_HAVE_DOMAINID) && !defined(STARPU_SIMGRID)
-#ifdef STARPU_HAVE_LIBNVIDIA_ML
+#ifdef STARPU_HAVE_NVML_H
 		nvmlDev[devid] = _starpu_cuda_get_nvmldev(&props[devid]);
 #endif
 		if (props[devid].pciDomainID)
@@ -1970,9 +2069,9 @@ static void start_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *work
 #ifdef HAVE_NVMLDEVICEGETTOTALENERGYCONSUMPTION
 		unsigned long long energy_start = 0;
 		nvmlReturn_t nvmlRet = -1;
-		if (profiling && task->profiling_info)
+		if (profiling && _starpu_energy_profiling && task->profiling_info && _starpu_nvmlDeviceGetTotalEnergyConsumption)
 		{
-			nvmlRet = nvmlDeviceGetTotalEnergyConsumption(nvmlDev[worker->devid], &energy_start);
+			nvmlRet = _starpu_nvmlDeviceGetTotalEnergyConsumption(nvmlDev[worker->devid], &energy_start);
 			if (nvmlRet == NVML_SUCCESS)
 				task->profiling_info->energy_consumed = energy_start / 1000.;
 		}
@@ -2054,11 +2153,11 @@ static void finish_job_on_cuda(struct _starpu_job *j, struct _starpu_worker *wor
 
 
 #ifdef HAVE_NVMLDEVICEGETTOTALENERGYCONSUMPTION
-	if (profiling && j->task->profiling_info && j->task->profiling_info->energy_consumed)
+	if (profiling && _starpu_energy_profiling && j->task->profiling_info && j->task->profiling_info->energy_consumed && _starpu_nvmlDeviceGetTotalEnergyConsumption)
 	{
 		unsigned long long energy_end;
 		nvmlReturn_t nvmlRet;
-		nvmlRet = nvmlDeviceGetTotalEnergyConsumption(nvmlDev[worker->devid], &energy_end);
+		nvmlRet = _starpu_nvmlDeviceGetTotalEnergyConsumption(nvmlDev[worker->devid], &energy_end);
 #ifdef STARPU_DEVEL
 #warning TODO: measure idle consumption to subtract it
 #endif
@@ -2411,7 +2510,7 @@ void starpu_cusolver_report_error(const char *func, const char *file, int line,
 		REPORT(CUSOLVER_STATUS_ZERO_PIVOT);
 		REPORT(CUSOLVER_STATUS_INVALID_LICENSE);
 
-#if CUSOLVER_VER_MAJOR >= 11
+#if defined(CUSOLVER_VER_MAJOR) && (CUSOLVER_VER_MAJOR >= 11)
 		REPORT(CUSOLVER_STATUS_IRS_PARAMS_NOT_INITIALIZED);
 		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID);
 		REPORT(CUSOLVER_STATUS_IRS_PARAMS_INVALID_PREC);
