@@ -383,9 +383,10 @@ static int _starpu_max_fpga_copy_ram_to_max_fpga(void *src, void *dst, size_t si
 	// LMemLoopback_writeLMem(dst, size, src);
 }
 
-/* Transfert SIZE bytes from the address pointed by SRC in the SRC_NODE memory
- *  * node to the address pointed by DST in the DST_NODE memory node
- *   */
+/**
+   Transfer SIZE bytes from the address pointed by SRC in the SRC_NODE memory
+   node to the address pointed by DST in the DST_NODE memory node
+*/
 void copy_ram_to_max_fpga(void *src, void *dst, size_t size)
 {
 	(void) src; (void) dst; (void) size;
@@ -400,9 +401,10 @@ void copy_max_fpga_to_ram(void *src, void *dst, size_t size)
 	//LMemLoopback_readLMem(size, src, dst);
 }
 
-/* Transfert SIZE bytes from the address pointed by SRC in the SRC_NODE memory
- * node to the address pointed by DST in the DST_NODE memory node
- */
+/**
+   Transfer SIZE bytes from the address pointed by SRC in the SRC_NODE memory
+   node to the address pointed by DST in the DST_NODE memory node
+*/
 int _starpu_max_fpga_copy_max_fpga_to_ram(void *src, void *dst, size_t size)
 {
 	printf("fpga to ram, fpga @= %p\n",src);
@@ -411,9 +413,10 @@ int _starpu_max_fpga_copy_max_fpga_to_ram(void *src, void *dst, size_t size)
 	//LMemLoopback_readLMem(src, size, dst);
 }
 
-/* Transfert SIZE bytes from the address pointed by SRC in the SRC_NODE memory
- * node to the address pointed by DST in the DST_NODE memory node
- */
+/**
+   Transfer SIZE bytes from the address pointed by SRC in the SRC_NODE memory
+   node to the address pointed by DST in the DST_NODE memory node
+*/
 int _starpu_max_fpga_copy_fpga_to_fpga(void *src, void *dst, size_t size)
 {
 	printf("fpga to ram, fpga @= %p\n",src);
@@ -559,9 +562,9 @@ static int execute_job_on_fpga(struct _starpu_job *j, struct starpu_task *worker
 		STARPU_ASSERT_MSG(func, "when STARPU_MAX_FPGA is defined in 'where', fpga_func or max_fpga_funcs has to be defined");
 		if (_starpu_get_disable_kernels() <= 0)
 		{
-			_STARPU_TRACE_START_EXECUTING();
+			_STARPU_TRACE_START_EXECUTING(j);
 			func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
-			_STARPU_TRACE_END_EXECUTING();
+			_STARPU_TRACE_END_EXECUTING(j);
 		}
 	}
 
