@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2012-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013	    Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,44 +20,43 @@
 #include <datawizard/interfaces/data_interface.h>
 #include <drivers/mp_common/mp_common.h>
 #include <drivers/mp_common/sink_common.h>
-#include <drivers/mic/driver_mic_common.h>
-#include <drivers/mic/driver_mic_source.h>
-#include <drivers/mic/driver_mic_sink.h>
+#include <drivers/mp_common/source_common.h>
 #include <drivers/mpi/driver_mpi_common.h>
 #include <drivers/mpi/driver_mpi_source.h>
 #include <drivers/mpi/driver_mpi_sink.h>
+#include <drivers/tcpip/driver_tcpip_common.h>
+#include <drivers/tcpip/driver_tcpip_source.h>
+#include <drivers/tcpip/driver_tcpip_sink.h>
 
 #include <common/list.h>
 
-const char *_starpu_mp_common_command_to_string(const int command)
+const char *_starpu_mp_common_command_to_string(const enum _starpu_mp_command command)
 {
 	switch(command)
 	{
+		/* Commands from master to slave */
 		case STARPU_MP_COMMAND_EXIT:
 			return "EXIT";
 		case STARPU_MP_COMMAND_EXECUTE:
 			return "EXECUTE";
 		case STARPU_MP_COMMAND_EXECUTE_DETACHED:
 			return "EXECUTE_DETACHED";
-		case STARPU_MP_COMMAND_ERROR_EXECUTE:
-			return "ERROR_EXECUTE";
-		case STARPU_MP_COMMAND_ERROR_EXECUTE_DETACHED:
-			return "ERROR_EXECUTE_DETACHED";
+		case STARPU_MP_COMMAND_SINK_NBCORES:
+			return "SINK_NBCORES";
 		case STARPU_MP_COMMAND_LOOKUP:
 			return "LOOKUP";
-		case STARPU_MP_COMMAND_ANSWER_LOOKUP:
-			return "ANSWER_LOOKUP";
-		case STARPU_MP_COMMAND_ERROR_LOOKUP:
-			return "ERROR_LOOKUP";
 		case STARPU_MP_COMMAND_ALLOCATE:
 			return "ALLOCATE";
-		case STARPU_MP_COMMAND_ANSWER_ALLOCATE:
-			return "ANSWER_ALLOCATE";
-		case STARPU_MP_COMMAND_ERROR_ALLOCATE:
-			return "ERROR_ALLOCATE";
 		case STARPU_MP_COMMAND_FREE:
 			return "FREE";
-		/* Synchronous send */
+		case STARPU_MP_COMMAND_MAP:
+			return "MAP";
+		case STARPU_MP_COMMAND_UNMAP:
+			return "UNMAP";
+		case STARPU_MP_COMMAND_SYNC_WORKERS:
+			return "SYNC_WORKERS";
+
+		/* Note: synchronous send */
 		case STARPU_MP_COMMAND_RECV_FROM_HOST:
 			return "RECV_FROM_HOST";
 		case STARPU_MP_COMMAND_SEND_TO_HOST:
@@ -66,42 +65,59 @@ const char *_starpu_mp_common_command_to_string(const int command)
 			return "RECV_FROM_SINK";
 		case STARPU_MP_COMMAND_SEND_TO_SINK:
 			return "SEND_TO_SINK";
-		/* Asynchronous send */
+
+		/* Note: Asynchronous send */
 		case STARPU_MP_COMMAND_RECV_FROM_HOST_ASYNC:
 			return "RECV_FROM_HOST_ASYNC";
-		case STARPU_MP_COMMAND_RECV_FROM_HOST_ASYNC_COMPLETED:
-			return "RECV_FROM_HOST_ASYNC_COMPLETED";
 		case STARPU_MP_COMMAND_SEND_TO_HOST_ASYNC:
 			return "SEND_TO_HOST_ASYNC";
-		case STARPU_MP_COMMAND_SEND_TO_HOST_ASYNC_COMPLETED:
-			return "SEND_TO_HOST_ASYNC_COMPLETED";
 		case STARPU_MP_COMMAND_RECV_FROM_SINK_ASYNC:
 			return "RECV_FROM_SINK_ASYNC";
-		case STARPU_MP_COMMAND_RECV_FROM_SINK_ASYNC_COMPLETED:
-			return "RECV_FROM_SINK_ASYNC_COMPLETED";
 		case STARPU_MP_COMMAND_SEND_TO_SINK_ASYNC:
 			return "SEND_TO_SINK_ASYNC";
-		case STARPU_MP_COMMAND_SEND_TO_SINK_ASYNC_COMPLETED:
-			return "SEND_TO_SINK_ASYNC_COMPLETED";
 
-		case STARPU_MP_COMMAND_TRANSFER_COMPLETE:
-			return "TRANSFER_COMPLETE";
-		case STARPU_MP_COMMAND_SINK_NBCORES:
-			return "SINK_NBCORES";
+		/* Synchronous answers from slave to master */
+		case STARPU_MP_COMMAND_ERROR_EXECUTE:
+			return "ERROR_EXECUTE";
+		case STARPU_MP_COMMAND_ERROR_EXECUTE_DETACHED:
+			return "ERROR_EXECUTE_DETACHED";
+		case STARPU_MP_COMMAND_ANSWER_LOOKUP:
+			return "ANSWER_LOOKUP";
+		case STARPU_MP_COMMAND_ERROR_LOOKUP:
+			return "ERROR_LOOKUP";
+		case STARPU_MP_COMMAND_ANSWER_ALLOCATE:
+			return "ANSWER_ALLOCATE";
+		case STARPU_MP_COMMAND_ERROR_ALLOCATE:
+			return "ERROR_ALLOCATE";
+		case STARPU_MP_COMMAND_ANSWER_MAP:
+			return "ANSWER_MAP";
+		case STARPU_MP_COMMAND_ERROR_MAP:
+			return "ERROR_MAP";
+		case STARPU_MP_COMMAND_ANSWER_TRANSFER_COMPLETE:
+			return "ANSWER_TRANSFER_COMPLETE";
 		case STARPU_MP_COMMAND_ANSWER_SINK_NBCORES:
 			return "ANSWER_SINK_NBCORES";
-		case STARPU_MP_COMMAND_EXECUTION_SUBMITTED:
-			return "EXECUTION_SUBMITTED";
-		case STARPU_MP_COMMAND_EXECUTION_COMPLETED:
-			return "EXECUTION_COMPLETED";
-		case STARPU_MP_COMMAND_EXECUTION_DETACHED_SUBMITTED:
-			return "EXECUTION_SUBMITTED_DETACHED";
-		case STARPU_MP_COMMAND_EXECUTION_DETACHED_COMPLETED:
-			return "EXECUTION_DETACHED_COMPLETED";
-		case STARPU_MP_COMMAND_PRE_EXECUTION:
-			return "PRE_EXECUTION";
-		case STARPU_MP_COMMAND_SYNC_WORKERS:
-			return "SYNC_WORKERS";
+		case STARPU_MP_COMMAND_ANSWER_EXECUTION_SUBMITTED:
+			return "ANSWER_EXECUTION_SUBMITTED";
+		case STARPU_MP_COMMAND_ANSWER_EXECUTION_DETACHED_SUBMITTED:
+			return "ANSWER_EXECUTION_DETACHED_SUBMITTED";
+
+		/* Asynchronous notifications from slave to master */
+		case STARPU_MP_COMMAND_NOTIF_RECV_FROM_HOST_ASYNC_COMPLETED:
+			return "NOTIF_RECV_FROM_HOST_ASYNC_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_SEND_TO_HOST_ASYNC_COMPLETED:
+			return "NOTIF_SEND_TO_HOST_ASYNC_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_RECV_FROM_SINK_ASYNC_COMPLETED:
+			return "NOTIF_RECV_FROM_SINK_ASYNC_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_SEND_TO_SINK_ASYNC_COMPLETED:
+			return "NOTIF_SEND_TO_SINK_ASYNC_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_EXECUTION_COMPLETED:
+			return "NOTIF_EXECUTION_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_EXECUTION_DETACHED_COMPLETED:
+			return "NOTIF_EXECUTION_DETACHED_COMPLETED";
+		case STARPU_MP_COMMAND_NOTIF_PRE_EXECUTION:
+			return "NOTIF_PRE_EXECUTION";
+
 		default:
 			return "<invalid command code>";
 	}
@@ -111,14 +127,14 @@ const char *_starpu_mp_common_node_kind_to_string(const int kind)
 {
 	switch(kind)
 	{
-		case STARPU_NODE_MIC_SINK:
-			return "MIC_SINK";
-		case STARPU_NODE_MIC_SOURCE:
-			return "MIC_SOURCE";
 		case STARPU_NODE_MPI_SINK:
 			return "MPI_SINK";
 		case STARPU_NODE_MPI_SOURCE:
 			return "MPI_SOURCE";
+		case STARPU_NODE_TCPIP_SINK:
+			return "TCPIP_SINK";
+		case STARPU_NODE_TCPIP_SOURCE:
+			return "TCPIP_SOURCE";
 		default:
 			return "<invalid command code>";
 	}
@@ -140,60 +156,6 @@ _starpu_mp_common_node_create(enum _starpu_mp_node_kind node_kind,
 
 	switch(node->kind)
 	{
-#ifdef STARPU_USE_MIC
-		case STARPU_NODE_MIC_SOURCE:
-		{
-			node->nb_mp_sinks = starpu_mic_worker_get_count();
-			node->devid = peer_id;
-
-			node->init = _starpu_mic_src_init;
-			node->launch_workers= NULL;
-			node->deinit = _starpu_mic_src_deinit;
-			node->report_error = _starpu_mic_src_report_scif_error;
-
-			node->mp_recv_is_ready = _starpu_mic_common_recv_is_ready;
-			node->mp_send = _starpu_mic_common_send;
-			node->mp_recv = _starpu_mic_common_recv;
-			node->dt_send = _starpu_mic_common_dt_send;
-			node->dt_recv = _starpu_mic_common_dt_recv;
-
-			node->get_kernel_from_job =_starpu_mic_src_get_kernel_from_job;
-			node->lookup = NULL;
-			node->bind_thread = NULL;
-			node->execute = NULL;
-			node->allocate = NULL;
-			node->free = NULL;
-		}
-		break;
-
-		case STARPU_NODE_MIC_SINK:
-		{
-			node->devid = atoi(starpu_getenv("_STARPU_MIC_DEVID"));
-			node->nb_mp_sinks = atoi(starpu_getenv("_STARPU_MIC_NB"));
-
-			node->init = _starpu_mic_sink_init;
-			node->launch_workers = _starpu_mic_sink_launch_workers;
-			node->deinit = _starpu_mic_sink_deinit;
-			node->report_error = _starpu_mic_sink_report_error;
-
-			node->mp_recv_is_ready = _starpu_mic_common_recv_is_ready;
-			node->mp_send = _starpu_mic_common_send;
-			node->mp_recv = _starpu_mic_common_recv;
-			node->dt_send = _starpu_mic_common_dt_send;
-			node->dt_recv = _starpu_mic_common_dt_recv;
-
-			node->dt_test = NULL; /* Not used now */
-
-			node->get_kernel_from_job = NULL;
-			node->lookup = _starpu_mic_sink_lookup;
-			node->bind_thread = _starpu_mic_sink_bind_thread;
-			node->execute = _starpu_sink_common_execute;
-			node->allocate = _starpu_mic_sink_allocate;
-			node->free = _starpu_mic_sink_free;
-		}
-		break;
-#endif /* STARPU_USE_MIC */
-
 #ifdef STARPU_USE_MPI_MASTER_SLAVE
 		case STARPU_NODE_MPI_SOURCE:
 		{
@@ -212,21 +174,29 @@ _starpu_mp_common_node_create(enum _starpu_mp_node_kind node_kind,
 			node->mp_recv_is_ready = _starpu_mpi_common_recv_is_ready;
 			node->mp_send = _starpu_mpi_common_mp_send;
 			node->mp_recv = _starpu_mpi_common_mp_recv;
+			node->nt_recv_is_ready = _starpu_mpi_common_notif_recv_is_ready;
+			node->nt_send_is_ready = _starpu_mpi_common_notif_send_is_ready;
+			node->mp_wait = NULL;
+			node->mp_signal = NULL;
+			node->nt_send = _starpu_mpi_common_nt_send;
+			node->nt_recv = _starpu_mpi_common_nt_recv;
 			node->dt_send = _starpu_mpi_common_send;
 			node->dt_recv = _starpu_mpi_common_recv;
 			node->dt_send_to_device = _starpu_mpi_common_send_to_device;
 			node->dt_recv_from_device = _starpu_mpi_common_recv_from_device;
 
-			node->get_kernel_from_job = _starpu_mpi_ms_src_get_kernel_from_job;
+			node->get_kernel_from_job = _starpu_src_common_get_cpu_func_from_job;
 			node->lookup = NULL;
 			node->bind_thread = NULL;
 			node->execute = NULL;
 			node->allocate = NULL;
 			node->free = NULL;
+			node->map = NULL;
+			node->unmap = NULL;
 		}
 		break;
 
-	        case STARPU_NODE_MPI_SINK:
+		case STARPU_NODE_MPI_SINK:
 		{
 			/*
 			  node->nb_mp_sinks =
@@ -235,13 +205,19 @@ _starpu_mp_common_node_create(enum _starpu_mp_node_kind node_kind,
 			node->mp_connection.mpi_remote_nodeid = _starpu_mpi_common_get_src_node();
 
 			node->init = _starpu_mpi_sink_init;
-			node->launch_workers = _starpu_mpi_sink_launch_workers;
-			node->deinit = _starpu_mpi_sink_deinit;
+			node->launch_workers = _starpu_sink_launch_workers;
+			node->deinit = _starpu_sink_deinit;
 			/*    node->report_error =  */
 
 			node->mp_recv_is_ready = _starpu_mpi_common_recv_is_ready;
 			node->mp_send = _starpu_mpi_common_mp_send;
 			node->mp_recv = _starpu_mpi_common_mp_recv;
+			node->nt_recv_is_ready = _starpu_mpi_common_notif_recv_is_ready;
+			node->nt_send_is_ready = _starpu_mpi_common_notif_send_is_ready;
+			node->mp_wait = NULL;
+			node->mp_signal = NULL;
+			node->nt_send = _starpu_mpi_common_nt_send;
+			node->nt_recv = _starpu_mpi_common_nt_recv;
 			node->dt_send = _starpu_mpi_common_send;
 			node->dt_recv = _starpu_mpi_common_recv;
 			node->dt_send_to_device = _starpu_mpi_common_send_to_device;
@@ -250,15 +226,99 @@ _starpu_mp_common_node_create(enum _starpu_mp_node_kind node_kind,
 			node->dt_test = _starpu_mpi_common_test_event;
 
 			node->get_kernel_from_job = NULL;
-			node->lookup = _starpu_mpi_sink_lookup;
+			node->lookup = _starpu_sink_common_cpu_lookup;
 			node->bind_thread = _starpu_mpi_sink_bind_thread;
 			node->execute = _starpu_sink_common_execute;
 			node->allocate = _starpu_sink_common_allocate;
 			node->free = _starpu_sink_common_free;
+			node->map = _starpu_sink_common_map;
+			node->unmap = _starpu_sink_common_unmap;
 		}
 		break;
 #endif /* STARPU_USE_MPI_MASTER_SLAVE */
 
+#ifdef STARPU_USE_TCPIP_MASTER_SLAVE
+		case STARPU_NODE_TCPIP_SOURCE:
+		{
+			/*
+			  node->nb_mp_sinks =
+			  node->devid =
+			*/
+			node->peer_id = (0 <= peer_id ? peer_id+1 : peer_id);
+
+			node->mp_connection.tcpip_mp_connection = &tcpip_sock[node->peer_id];
+
+			node->init = _starpu_tcpip_source_init;
+			node->launch_workers = NULL;
+			node->deinit = _starpu_tcpip_source_deinit;
+			/*     node->report_error = */
+
+			node->mp_recv_is_ready = _starpu_tcpip_common_recv_is_ready;
+			node->mp_send = _starpu_tcpip_common_mp_send;
+			node->mp_recv = _starpu_tcpip_common_mp_recv;
+			node->nt_recv_is_ready = _starpu_tcpip_common_notif_recv_is_ready;
+			node->nt_send_is_ready = _starpu_tcpip_common_notif_send_is_ready;
+			node->mp_wait = _starpu_tcpip_common_wait;
+			node->mp_signal = _starpu_tcpip_common_signal;
+			node->nt_send = _starpu_tcpip_common_nt_send;
+			node->nt_recv = _starpu_tcpip_common_nt_recv;
+			node->dt_send = _starpu_tcpip_common_send;
+			node->dt_recv = _starpu_tcpip_common_recv;
+			node->dt_send_to_device = _starpu_tcpip_common_send_to_device;
+			node->dt_recv_from_device = _starpu_tcpip_common_recv_from_device;
+
+			node->get_kernel_from_job = _starpu_src_common_get_cpu_func_from_job;
+			node->lookup = NULL;
+			node->bind_thread = NULL;
+			node->execute = NULL;
+			node->allocate = NULL;
+			node->free = NULL;
+			node->map = NULL;
+			node->unmap = NULL;
+		}
+		break;
+
+		case STARPU_NODE_TCPIP_SINK:
+		{
+			/*
+			  node->nb_mp_sinks =
+			  node->devid =
+			*/
+			node->mp_connection.tcpip_mp_connection = &tcpip_sock[0];
+
+			node->init = _starpu_tcpip_sink_init;
+			node->launch_workers = _starpu_sink_launch_workers;
+			node->deinit = _starpu_sink_deinit;
+			/*    node->report_error =  */
+
+			node->mp_recv_is_ready = _starpu_tcpip_common_recv_is_ready;
+			node->mp_send = _starpu_tcpip_common_mp_send;
+			node->mp_recv = _starpu_tcpip_common_mp_recv;
+			node->nt_recv_is_ready = _starpu_tcpip_common_notif_recv_is_ready;
+			node->nt_send_is_ready = _starpu_tcpip_common_notif_send_is_ready;
+			node->mp_wait = _starpu_tcpip_common_wait;
+			node->mp_signal = _starpu_tcpip_common_signal;
+			node->nt_send = _starpu_tcpip_common_nt_send;
+			node->nt_recv = _starpu_tcpip_common_nt_recv;
+			node->dt_send = _starpu_tcpip_common_send;
+			node->dt_recv = _starpu_tcpip_common_recv;
+			node->dt_send_to_device = _starpu_tcpip_common_send_to_device;
+			node->dt_recv_from_device = _starpu_tcpip_common_recv_from_device;
+
+			node->dt_test = _starpu_tcpip_common_test_event;
+
+			node->get_kernel_from_job = NULL;
+			node->lookup = _starpu_sink_common_cpu_lookup;
+			node->bind_thread = _starpu_tcpip_sink_bind_thread;
+			node->execute = _starpu_sink_common_execute;
+			node->allocate = _starpu_sink_common_allocate;
+			node->free = _starpu_sink_common_free;
+			node->map = _starpu_sink_common_map;
+			node->unmap = _starpu_sink_common_unmap;
+		}
+		break;
+#endif /* STARPU_USE_TCPIP_MASTER_SLAVE */
+
 		default:
 			STARPU_ASSERT(0);
 	}
@@ -273,14 +333,16 @@ _starpu_mp_common_node_create(enum _starpu_mp_node_kind node_kind,
 	mp_message_list_init(&node->message_queue);
 	STARPU_PTHREAD_MUTEX_INIT(&node->message_queue_mutex,NULL);
 
-        STARPU_PTHREAD_MUTEX_INIT(&node->connection_mutex, NULL);
+	STARPU_PTHREAD_MUTEX_INIT(&node->connection_mutex, NULL);
 
-        _starpu_mp_event_list_init(&node->event_list);
+	_starpu_mp_event_list_init(&node->event_list);
+	_starpu_mp_event_list_init(&node->event_queue);
 
 	/* If the node is a sink then we must initialize some field */
-	if(node->kind == STARPU_NODE_MIC_SINK || node->kind == STARPU_NODE_MPI_SINK)
+	if(node->kind == STARPU_NODE_MPI_SINK || node->kind == STARPU_NODE_TCPIP_SINK)
 	{
 		int i;
+		STARPU_HG_DISABLE_CHECKING(node->is_running);
 		node->is_running = 1;
 		_STARPU_MALLOC(node->run_table, sizeof(struct mp_task *)*node->nb_cores);
 		_STARPU_MALLOC(node->run_table_detached, sizeof(struct mp_task *)*node->nb_cores);
@@ -311,7 +373,7 @@ void _starpu_mp_common_node_destroy(struct _starpu_mp_node *node)
 	STARPU_PTHREAD_MUTEX_DESTROY(&node->message_queue_mutex);
 
 	/* If the node is a sink then we must destroy some field */
-	if(node->kind == STARPU_NODE_MIC_SINK || node->kind == STARPU_NODE_MPI_SINK)
+	if(node->kind == STARPU_NODE_MPI_SINK || node->kind == STARPU_NODE_TCPIP_SINK)
 	{
 		int i;
 		for(i=0; i<node->nb_cores; i++)
@@ -332,13 +394,13 @@ void _starpu_mp_common_node_destroy(struct _starpu_mp_node *node)
 }
 
 /* Send COMMAND to RECIPIENT, along with ARG if ARG_SIZE is non-zero */
-void _starpu_mp_common_send_command(const struct _starpu_mp_node *node, const enum _starpu_mp_command command, void *arg, int arg_size)
+static void __starpu_mp_common_send_command(const struct _starpu_mp_node *node, const enum _starpu_mp_command command, void *arg, int arg_size, int notif)
 {
-	STARPU_ASSERT_MSG(arg_size <= BUFFER_SIZE, "Too much data (%d) for the static MIC buffer (%d), increase BUFFER_SIZE perhaps?", arg_size, BUFFER_SIZE);
+	STARPU_ASSERT_MSG(arg_size <= BUFFER_SIZE, "Too much data (%d) for the static buffer (%d), increase BUFFER_SIZE perhaps?", arg_size, BUFFER_SIZE);
 
-        //printf("SEND CMD : %d - arg_size %d by %lu \n", command, arg_size, starpu_pthread_self());
+	//printf("SEND %s: %d/%s - arg_size %d by %lu \n", notif?"NOTIF":"CMD", command, _starpu_mp_common_command_to_string(command), arg_size, starpu_pthread_self());
 
-	/* MIC and MPI sizes are given through a int */
+	/* MPI sizes are given through a int */
 	int command_size = sizeof(enum _starpu_mp_command);
 	int arg_size_size = sizeof(int);
 
@@ -346,10 +408,30 @@ void _starpu_mp_common_send_command(const struct _starpu_mp_node *node, const en
 	memcpy(node->buffer, &command, command_size);
 	memcpy((void*) ((uintptr_t)node->buffer + command_size), &arg_size, arg_size_size);
 
-	node->mp_send(node, node->buffer, command_size + arg_size_size);
+	if (!notif)
+		node->mp_send(node, node->buffer, command_size + arg_size_size);
+	else
+		node->nt_send(node, node->buffer, command_size + arg_size_size);
 
 	if (arg_size)
-		node->mp_send(node, arg, arg_size);
+	{
+		if (!notif)
+			node->mp_send(node, arg, arg_size);
+		else
+			node->nt_send(node, arg, arg_size);
+	}
+}
+
+/* Send COMMAND to RECIPIENT, along with ARG if ARG_SIZE is non-zero */
+void _starpu_mp_common_send_command(const struct _starpu_mp_node *node, const enum _starpu_mp_command command, void *arg, int arg_size)
+{
+	__starpu_mp_common_send_command(node, command, arg, arg_size, 0);
+}
+
+/* Send NOTIF COMMAND to RECIPIENT, along with ARG if ARG_SIZE is non-zero */
+void _starpu_nt_common_send_command(const struct _starpu_mp_node *node, const enum _starpu_mp_command command, void *arg, int arg_size)
+{
+	__starpu_mp_common_send_command(node, command, arg, arg_size, 1);
 }
 
 /* Return the command received from SENDER. In case SENDER sent an argument
@@ -358,20 +440,23 @@ void _starpu_mp_common_send_command(const struct _starpu_mp_node *node, const en
  * However, the data pointed by arg shouldn't be relied on after a new call to
  * STARPU_MP_COMMON_RECV_COMMAND as it might corrupt it.
  */
-enum _starpu_mp_command _starpu_mp_common_recv_command(const struct _starpu_mp_node *node, void **arg, int *arg_size)
+static enum _starpu_mp_command __starpu_mp_common_recv_command(const struct _starpu_mp_node *node, void **arg, int *arg_size, int notif)
 {
 	enum _starpu_mp_command command;
 
-	/* MIC and MPI sizes are given through a int */
+	/* MPI sizes are given through a int */
 	int command_size = sizeof(enum _starpu_mp_command);
 	int arg_size_size = sizeof(int);
 
-	node->mp_recv(node, node->buffer, command_size + arg_size_size);
+	if (!notif)
+		node->mp_recv(node, node->buffer, command_size + arg_size_size);
+	else
+		node->nt_recv(node, node->buffer, command_size + arg_size_size);
 
 	command = *((enum _starpu_mp_command *) node->buffer);
 	*arg_size = *((int *) ((uintptr_t)node->buffer + command_size));
 
-        //printf("RECV command : %d - arg_size %d by %lu \n", command, *arg_size, starpu_pthread_self());
+	//printf("RECV %s : %d/%s - arg_size %d by %lu \n", notif?"NOTIF":"CMD", command, _starpu_mp_common_command_to_string(command), *arg_size, starpu_pthread_self());
 
 	/* If there is no argument (ie. arg_size == 0),
 	 * let's return the command right now */
@@ -383,9 +468,80 @@ enum _starpu_mp_command _starpu_mp_common_recv_command(const struct _starpu_mp_n
 
 	STARPU_ASSERT(*arg_size <= BUFFER_SIZE);
 
-	node->mp_recv(node, node->buffer, *arg_size);
+	if (!notif)
+		node->mp_recv(node, node->buffer, *arg_size);
+	else
+		node->nt_recv(node, node->buffer, *arg_size);
 
 	*arg = node->buffer;
 
 	return command;
 }
+
+/* Return the command received from SENDER*/
+enum _starpu_mp_command _starpu_mp_common_recv_command(const struct _starpu_mp_node *node, void **arg, int *arg_size)
+{
+	return __starpu_mp_common_recv_command(node, arg, arg_size, 0);
+}
+
+/* Return the notif command received from SENDER*/
+enum _starpu_mp_command _starpu_nt_common_recv_command(const struct _starpu_mp_node *node, void **arg, int *arg_size)
+{
+	return __starpu_mp_common_recv_command(node, arg, arg_size, 1);
+}
+
+void _starpu_sink_deinit(struct _starpu_mp_node *node)
+{
+	int i;
+	node->is_running = 0;
+	for(i=0; i<node->nb_cores; i++)
+	{
+		sem_post(&node->sem_run_table[i]);
+		STARPU_PTHREAD_JOIN(((starpu_pthread_t *)node->thread_table)[i],NULL);
+	}
+	free(node->thread_table);
+}
+
+void _starpu_sink_launch_workers(struct _starpu_mp_node *node)
+{
+	//TODO
+	int i;
+	struct arg_sink_thread * arg;
+	cpu_set_t cpuset;
+	starpu_pthread_attr_t attr;
+	starpu_pthread_t thread;
+
+	for(i=0; i < node->nb_cores; i++)
+	{
+		int ret;
+
+		ret = starpu_pthread_attr_init(&attr);
+		STARPU_ASSERT(ret == 0);
+
+#if defined(HAVE_PTHREAD_SETAFFINITY_NP) && defined(__linux__)
+		//init the set
+		CPU_ZERO(&cpuset);
+		CPU_SET(i,&cpuset);
+
+		int nobind = starpu_getenv_number("STARPU_WORKERS_NOBIND");
+
+		if (nobind <= 0)
+		{
+			ret = pthread_attr_setaffinity_np(&attr, sizeof(cpu_set_t), &cpuset);
+			STARPU_ASSERT(ret == 0);
+		}
+#else
+#warning no CPU binding support
+#endif
+
+		/*prepare the argument for the thread*/
+		_STARPU_MALLOC(arg, sizeof(struct arg_sink_thread));
+		arg->coreid = i;
+		arg->node = node;
+
+		STARPU_PTHREAD_CREATE(&thread, &attr, _starpu_sink_thread, arg);
+		starpu_pthread_attr_destroy(&attr);
+		((starpu_pthread_t *)node->thread_table)[i] = thread;
+
+	}
+}
