@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Simon Archipoff
  *
@@ -38,7 +38,7 @@
  *
  *
  *
- * its possible that a _starpu_task_grid wont have task, because it have been
+ * its possible that a _starpu_task_grid won't have task, because it have been
  * poped by a worker.
  *
  * N = no task
@@ -93,7 +93,7 @@ struct _starpu_worker_task_list
 {
 	double exp_start, exp_len, exp_end, pipeline_len;
 	struct _starpu_task_grid *first, *last;
-	unsigned ntasks;
+	unsigned ntasks, pipeline_ntasks;
 	starpu_pthread_mutex_t mutex;
 };
 
@@ -129,7 +129,7 @@ static void _starpu_worker_task_list_started(struct _starpu_worker_task_list *l,
 		l->exp_len -= model;
 		l->pipeline_len += model;
 		l->exp_start = starpu_timing_now() + l->pipeline_len;
-                l->exp_end= l->exp_start + l->exp_len;
+		l->exp_end= l->exp_start + l->exp_len;
 	}
 }
 
@@ -139,11 +139,14 @@ static void _starpu_worker_task_list_finished(struct _starpu_worker_task_list *l
 	if(!isnan(task->predicted))
 		/* The execution is over, remove it from pipelined */
 		l->pipeline_len -= task->predicted;
+	if (!l->pipeline_ntasks)
+		_STARPU_DISP("warning: bogus computation of pipeline_ntasks?\n");
+	else
+		l->pipeline_ntasks--;
 	l->exp_start = STARPU_MAX(starpu_timing_now() + l->pipeline_len, l->exp_start);
 	l->exp_end = l->exp_start + l->exp_len;
 }
 
-
 struct _starpu_worker_component_data
 {
 	union
@@ -257,6 +260,8 @@ static inline void _starpu_worker_task_list_add(struct _starpu_worker_task_list
 
 	task->predicted = predicted;
 	task->predicted_transfer = predicted_transfer;
+
+	l->pipeline_ntasks++;
 }
 
 static inline void _starpu_worker_task_list_push(struct _starpu_worker_task_list * l, struct _starpu_task_grid * t)
@@ -301,7 +306,7 @@ static inline void _starpu_task_grid_unset_left_right_member(struct _starpu_task
 
 static inline struct starpu_task * _starpu_worker_task_list_pop(struct _starpu_worker_task_list * l)
 {
- 	if(!l->first)
+	if(!l->first)
 	{
 		l->exp_len = l->pipeline_len = 0.0;
 		l->exp_start = l->exp_end = starpu_timing_now();
@@ -410,7 +415,7 @@ static int simple_worker_push_task(struct starpu_sched_component * component, st
 	t->task = task;
 	t->ntasks = 1;
 
-	task->workerid = starpu_bitmap_first(component->workers);
+	task->workerid = starpu_bitmap_first(&component->workers);
 #if 1 /* dead lock problem? */
 	if (starpu_get_prefetch_flag() && !task->prefetched)
 		starpu_prefetch_task_input_for(task, task->workerid);
@@ -499,7 +504,7 @@ static double simple_worker_estimated_end(struct starpu_sched_component * compon
 {
 	struct _starpu_worker_component_data * data = component->data;
 	double now = starpu_timing_now();
-	if (now + data->list->pipeline_len > data->list->exp_start )
+	if (now + data->list->pipeline_len > data->list->exp_start)
 	{
 		data->list->exp_start = now + data->list->pipeline_len;
 		data->list->exp_end = data->list->exp_start + data->list->exp_len;
@@ -512,19 +517,19 @@ static double simple_worker_estimated_load(struct starpu_sched_component * compo
 	struct _starpu_worker * worker = _starpu_sched_component_worker_get_worker(component);
 	int nb_task = 0;
 	STARPU_COMPONENT_MUTEX_LOCK(&worker->mutex);
-	struct starpu_task_list list = worker->local_tasks;
+	struct starpu_task_prio_list *list = &worker->local_tasks;
 	struct starpu_task * task;
-	for(task = starpu_task_list_front(&list);
-	    task != starpu_task_list_end(&list);
-	    task = starpu_task_list_next(task))
+	for(task = starpu_task_prio_list_begin(list);
+	    task != starpu_task_prio_list_end(list);
+	    task = starpu_task_prio_list_next(list, task))
 		nb_task++;
 	STARPU_COMPONENT_MUTEX_UNLOCK(&worker->mutex);
 	struct _starpu_worker_component_data * d = component->data;
 	struct _starpu_worker_task_list * l = d->list;
-	int ntasks_in_fifo = l ? l->ntasks : 0;
+	int ntasks_in_fifo = l ? l->ntasks + l->pipeline_ntasks : 0;
 	return (double) (nb_task + ntasks_in_fifo)
 		/ starpu_worker_get_relative_speedup(
-				starpu_worker_get_perf_archtype(starpu_bitmap_first(component->workers), component->tree->sched_ctx_id));
+				starpu_worker_get_perf_archtype(starpu_bitmap_first(&component->workers), component->tree->sched_ctx_id));
 }
 
 static void _worker_component_deinit_data(struct starpu_sched_component * component)
@@ -569,8 +574,8 @@ static struct starpu_sched_component * starpu_sched_component_worker_create(stru
 	component->estimated_end = simple_worker_estimated_end;
 	component->estimated_load = simple_worker_estimated_load;
 	component->deinit_data = _worker_component_deinit_data;
-	starpu_bitmap_set(component->workers, workerid);
-	starpu_bitmap_or(component->workers_in_ctx, component->workers);
+	starpu_bitmap_set(&component->workers, workerid);
+	starpu_bitmap_or(&component->workers_in_ctx, &component->workers);
 	_worker_components[tree->sched_ctx_id][workerid] = component;
 
 	/*
@@ -618,7 +623,7 @@ static int combined_worker_push_task(struct starpu_sched_component * component,
 	struct _starpu_worker_component_data * data = component->data;
 	STARPU_ASSERT(data->parallel_worker.worker_size >= 1);
 	struct _starpu_task_grid * task_alias[data->parallel_worker.worker_size];
-	starpu_parallel_task_barrier_init(task, starpu_bitmap_first(component->workers));
+	starpu_parallel_task_barrier_init(task, starpu_bitmap_first(&component->workers));
 	task_alias[0] = _starpu_task_grid_create();
 	task_alias[0]->task = starpu_task_dup(task);
 	task_alias[0]->task->workerid = data->parallel_worker.workerids[0];
@@ -752,8 +757,8 @@ static struct starpu_sched_component  * starpu_sched_component_combined_worker_c
 
 	struct starpu_sched_component *component = starpu_sched_component_parallel_worker_create(tree, combined_worker->worker_size, (unsigned *) combined_worker->combined_workerid);
 
-	starpu_bitmap_set(component->workers, workerid);
-	starpu_bitmap_or(component->workers_in_ctx, component->workers);
+	starpu_bitmap_set(&component->workers, workerid);
+	starpu_bitmap_or(&component->workers_in_ctx, &component->workers);
 
 	_worker_components[tree->sched_ctx_id][workerid] = component;
 
@@ -805,8 +810,8 @@ int starpu_sched_component_worker_get_workerid(struct starpu_sched_component * w
 #ifndef STARPU_NO_ASSERT
 	STARPU_ASSERT(_worker_consistant(worker_component));
 #endif
-	STARPU_ASSERT(1 == starpu_bitmap_cardinal(worker_component->workers));
-	return starpu_bitmap_first(worker_component->workers);
+	STARPU_ASSERT(1 == starpu_bitmap_cardinal(&worker_component->workers));
+	return starpu_bitmap_first(&worker_component->workers);
 }
 
 void starpu_sched_component_worker_pre_exec_hook(struct starpu_task * task, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
