@@ -1,11 +1,12 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Joris Pablo
  * Copyright (C) 2013       Simon Archipoff
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
+ * Copyright (C) 2020       Télécom-Sud Paris
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,15 +24,20 @@
 
 #include <starpu_config.h>
 #include <starpu_scheduler.h>
+#include <schedulers/starpu_scheduler_toolbox.h>
 
 #include <common/fxt.h>
 #include <core/task.h>
 #include <core/workers.h>
 #include <core/sched_policy.h>
 #include <core/debug.h>
-
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 #include <sched_policies/fifo_queues.h>
+
 #include <limits.h>
+#include <math.h> /* for fpclassify() checks on knob values */
 
 
 #ifndef DBL_MIN
@@ -51,7 +57,7 @@ struct _starpu_dmda_data
 	double _gamma;
 	double idle_power;
 
-	struct _starpu_fifo_taskq **queue_array;
+	struct starpu_st_fifo_taskq queue_array[STARPU_NMAXWORKERS];
 
 	long int total_task_cnt;
 	long int ready_task_cnt;
@@ -59,6 +65,104 @@ struct _starpu_dmda_data
 	int num_priorities;
 };
 
+/* performance steering knobs */
+
+/* . per-scheduler knobs */
+static int __s_alpha_knob;
+static int __s_beta_knob;
+static int __s_gamma_knob;
+static int __s_idle_power_knob;
+
+/* . knob variables */
+static double __s_alpha__value = 1.0;
+static double __s_beta__value = 1.0;
+static double __s_gamma__value = 1.0;
+static double __s_idle_power__value = 1.0;
+
+/* . per-scheduler knob group */
+static struct starpu_perf_knob_group * __kg_starpu_dmda__per_scheduler;
+
+static void sched_knobs__set(const struct starpu_perf_knob * const knob, void *context, const struct starpu_perf_knob_value * const value)
+{
+	const char * const sched_policy_name = *(const char **)context;
+	(void) sched_policy_name;
+	if (knob->id == __s_alpha_knob)
+	{
+		STARPU_ASSERT(fpclassify(value->val_double) == FP_NORMAL);
+		__s_alpha__value = value->val_double;
+	}
+	else if (knob->id == __s_beta_knob)
+	{
+		STARPU_ASSERT(fpclassify(value->val_double) == FP_NORMAL);
+		__s_beta__value = value->val_double;
+	}
+	else if (knob->id == __s_gamma_knob)
+	{
+		STARPU_ASSERT(fpclassify(value->val_double) == FP_NORMAL);
+		__s_gamma__value = value->val_double;
+	}
+	else if (knob->id == __s_idle_power_knob)
+	{
+		STARPU_ASSERT(fpclassify(value->val_double) == FP_NORMAL);
+		__s_idle_power__value = value->val_double;
+	}
+	else
+	{
+		STARPU_ASSERT(0);
+		abort();
+	}
+}
+
+static void sched_knobs__get(const struct starpu_perf_knob * const knob, void *context,       struct starpu_perf_knob_value * const value)
+{
+	const char * const sched_policy_name = *(const char **)context;
+	(void) sched_policy_name;
+	if (knob->id == __s_alpha_knob)
+	{
+		value->val_double = __s_alpha__value;
+	}
+	else if (knob->id == __s_beta_knob)
+	{
+		value->val_double = __s_beta__value;
+	}
+	else if (knob->id == __s_gamma_knob)
+	{
+		value->val_double = __s_gamma__value;
+	}
+	else if (knob->id == __s_idle_power_knob)
+	{
+		value->val_double = __s_idle_power__value;
+	}
+	else
+	{
+		STARPU_ASSERT(0);
+		abort();
+	}
+}
+
+void _starpu__dmda_c__register_knobs(void)
+{
+	{
+		const enum starpu_perf_knob_scope scope = starpu_perf_knob_scope_per_scheduler;
+		__kg_starpu_dmda__per_scheduler = _starpu_perf_knob_group_register(scope, sched_knobs__set, sched_knobs__get);
+
+		/* TODO: priority capping knobs actually work globally for now, the sched policy name is ignored */
+		__STARPU_PERF_KNOB_REG("starpu.dmda", __kg_starpu_dmda__per_scheduler, s_alpha_knob, double, "alpha constant multiplier");
+
+		__STARPU_PERF_KNOB_REG("starpu.dmda", __kg_starpu_dmda__per_scheduler, s_beta_knob, double, "beta constant multiplier");
+
+		__STARPU_PERF_KNOB_REG("starpu.dmda", __kg_starpu_dmda__per_scheduler, s_gamma_knob, double, "gamma constant multiplier");
+
+		__STARPU_PERF_KNOB_REG("starpu.dmda", __kg_starpu_dmda__per_scheduler, s_idle_power_knob, double, "idle_power constant multiplier");
+	}
+}
+
+void _starpu__dmda_c__unregister_knobs(void)
+{
+	_starpu_perf_knob_group_unregister(__kg_starpu_dmda__per_scheduler);
+	__kg_starpu_dmda__per_scheduler = NULL;
+}
+
 /* The dmda scheduling policy uses
  *
  * alpha * T_computation + beta * T_communication + gamma * Consumption
@@ -71,7 +175,7 @@ struct _starpu_dmda_data
 #define _STARPU_SCHED_GAMMA_DEFAULT 1000.0
 
 /* This is called when a transfer request is actually pushed to the worker */
-static void _starpu_fifo_task_transfer_started(struct _starpu_fifo_taskq *fifo, struct starpu_task *task, int num_priorities)
+static void _starpu_fifo_task_transfer_started(struct starpu_st_fifo_taskq *fifo, struct starpu_task *task, int num_priorities)
 {
 	double transfer_model = task->predicted_transfer;
 	if (isnan(transfer_model))
@@ -85,14 +189,14 @@ static void _starpu_fifo_task_transfer_started(struct _starpu_fifo_taskq *fifo,
 	if(num_priorities != -1)
 	{
 		int i;
-		int task_prio = _starpu_normalize_prio(task->priority, num_priorities, task->sched_ctx);
+		int task_prio = starpu_st_normalize_prio(task->priority, num_priorities, task->sched_ctx);
 		for(i = 0; i <= task_prio; i++)
 			fifo->exp_len_per_priority[i] -= transfer_model;
 	}
 }
 
 /* This is called when a task is actually pushed to the worker (i.e. the transfer finished */
-static void _starpu_fifo_task_started(struct _starpu_fifo_taskq *fifo, struct starpu_task *task, int num_priorities)
+static void _starpu_fifo_task_started(struct starpu_st_fifo_taskq *fifo, struct starpu_task *task, int num_priorities)
 {
 	double model = task->predicted;
 	double transfer_model = task->predicted_transfer;
@@ -106,11 +210,11 @@ static void _starpu_fifo_task_started(struct _starpu_fifo_taskq *fifo, struct st
 		fifo->exp_len -= model;
 		fifo->pipeline_len += model;
 		fifo->exp_start = starpu_timing_now() + fifo->pipeline_len;
-                fifo->exp_end= fifo->exp_start + fifo->exp_len;
+		fifo->exp_end= fifo->exp_start + fifo->exp_len;
 		if(num_priorities != -1)
 		{
 			int i;
-			int task_prio = _starpu_normalize_prio(task->priority, num_priorities, task->sched_ctx);
+			int task_prio = starpu_st_normalize_prio(task->priority, num_priorities, task->sched_ctx);
 			for(i = 0; i <= task_prio; i++)
 				fifo->exp_len_per_priority[i] -= model;
 		}
@@ -118,60 +222,28 @@ static void _starpu_fifo_task_started(struct _starpu_fifo_taskq *fifo, struct st
 }
 
 /* This is called when a task is actually finished */
-static void _starpu_fifo_task_finished(struct _starpu_fifo_taskq *fifo, struct starpu_task *task, int num_priorities STARPU_ATTRIBUTE_UNUSED)
+static void _starpu_fifo_task_finished(struct starpu_st_fifo_taskq *fifo, struct starpu_task *task, int num_priorities STARPU_ATTRIBUTE_UNUSED)
 {
 	if(!isnan(task->predicted))
 		/* The execution is over, remove it from pipelined */
 		fifo->pipeline_len -= task->predicted;
+	if (!fifo->pipeline_ntasks)
+		_STARPU_DISP("warning: bogus computation of pipeline_ntasks?\n");
+	else
+		fifo->pipeline_ntasks--;
 	fifo->exp_start = STARPU_MAX(starpu_timing_now() + fifo->pipeline_len, fifo->exp_start);
 	fifo->exp_end = fifo->exp_start + fifo->exp_len;
 }
 
 
-
-static struct starpu_task *dmda_pop_ready_task(unsigned sched_ctx_id)
+static struct starpu_task *_dmda_pop_task(unsigned sched_ctx_id, int ready)
 {
 	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
 	struct starpu_task *task;
 
 	unsigned workerid = starpu_worker_get_id_check();
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
-
-	/* Take the opportunity to update start time */
-	fifo->exp_start = STARPU_MAX(starpu_timing_now(), fifo->exp_start);
-	fifo->exp_end = fifo->exp_start + fifo->exp_len;
-
-	task = _starpu_fifo_pop_first_ready_task(fifo, workerid, dt->num_priorities);
-	if (task)
-	{
-		_starpu_fifo_task_transfer_started(fifo, task, dt->num_priorities);
-
-		starpu_sched_ctx_list_task_counters_decrement(sched_ctx_id, workerid);
-
-#ifdef STARPU_VERBOSE
-		if (task->cl)
-		{
-			int non_ready = _starpu_count_non_ready_buffers(task, workerid);
-			if (non_ready == 0)
-				dt->ready_task_cnt++;
-		}
-
-		dt->total_task_cnt++;
-#endif
-	}
-
-	return task;
-}
-
-static struct starpu_task *dmda_pop_task(unsigned sched_ctx_id)
-{
-	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-
-	struct starpu_task *task;
-
-	unsigned workerid = starpu_worker_get_id_check();
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
+	struct starpu_st_fifo_taskq *fifo = &dt->queue_array[workerid];
 
 	/* Take the opportunity to update start time */
 	fifo->exp_start = STARPU_MAX(starpu_timing_now(), fifo->exp_start);
@@ -179,17 +251,20 @@ static struct starpu_task *dmda_pop_task(unsigned sched_ctx_id)
 
 	STARPU_ASSERT_MSG(fifo, "worker %u does not belong to ctx %u anymore.\n", workerid, sched_ctx_id);
 
-	task = _starpu_fifo_pop_local_task(fifo);
+	if (ready)
+		task = starpu_st_fifo_taskq_pop_first_ready_task(fifo, workerid, dt->num_priorities);
+	else
+		task = starpu_st_fifo_taskq_pop_local_task(fifo);
 	if (task)
 	{
 		_starpu_fifo_task_transfer_started(fifo, task, dt->num_priorities);
 
 		starpu_sched_ctx_list_task_counters_decrement(sched_ctx_id, workerid);
-		  
+
 #ifdef STARPU_VERBOSE
 		if (task->cl)
 		{
-			int non_ready = _starpu_count_non_ready_buffers(task, workerid);
+			int non_ready = starpu_st_non_ready_buffers_count(task, workerid);
 			if (non_ready == 0)
 				dt->ready_task_cnt++;
 		}
@@ -201,29 +276,14 @@ static struct starpu_task *dmda_pop_task(unsigned sched_ctx_id)
 	return task;
 }
 
-static struct starpu_task *dmda_pop_every_task(unsigned sched_ctx_id)
+static struct starpu_task *dmda_pop_ready_task(unsigned sched_ctx_id)
 {
-	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
+	return _dmda_pop_task(sched_ctx_id, 1);
+}
 
-	struct starpu_task *new_list, *task;
-
-	unsigned workerid = starpu_worker_get_id_check();
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
-
-	/* Take the opportunity to update start time */
-	fifo->exp_start = STARPU_MAX(starpu_timing_now(), fifo->exp_start);
-	fifo->exp_end = fifo->exp_start + fifo->exp_len;
-
-	starpu_worker_lock_self();
-	new_list = _starpu_fifo_pop_every_task(fifo, workerid);
-	starpu_worker_unlock_self();
-
-	starpu_sched_ctx_list_task_counters_reset(sched_ctx_id, workerid);
-
-	for (task = new_list; task; task = task->next)
-		_starpu_fifo_task_transfer_started(fifo, task, dt->num_priorities);
-
-	return new_list;
+static struct starpu_task *dmda_pop_task(unsigned sched_ctx_id)
+{
+	return _dmda_pop_task(sched_ctx_id, 0);
 }
 
 static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
@@ -247,7 +307,7 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 			return 0;
 	}
 
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[best_workerid];
+	struct starpu_st_fifo_taskq *fifo = &dt->queue_array[best_workerid];
 
 	double now = starpu_timing_now();
 
@@ -257,10 +317,13 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 
 	starpu_worker_lock(best_workerid);
 
-        /* Sometimes workers didn't take the tasks as early as we expected */
+	fifo->pipeline_ntasks++;
+
+	/* Sometimes workers didn't take the tasks as early as we expected */
 	fifo->exp_start = isnan(fifo->exp_start) ? now + fifo->pipeline_len : STARPU_MAX(fifo->exp_start, now);
 	fifo->exp_end = fifo->exp_start + fifo->exp_len;
 
+	/* FIXME: We don't have overlap when running CPU-CPU transfers */
 	if ((now + predicted_transfer) < fifo->exp_end)
 	{
 		/* We may hope that the transfer will be finished by
@@ -280,7 +343,7 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 		if(dt->num_priorities != -1)
 		{
 			int i;
-			int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+			int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 			for(i = 0; i <= task_prio; i++)
 				fifo->exp_len_per_priority[i] += predicted_transfer;
 		}
@@ -293,7 +356,7 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 		if(dt->num_priorities != -1)
 		{
 			int i;
-			int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+			int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 			for(i = 0; i <= task_prio; i++)
 				fifo->exp_len_per_priority[i] += predicted;
 		}
@@ -329,13 +392,13 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 	if (prio)
 	{
 		starpu_worker_lock(best_workerid);
-		ret =_starpu_fifo_push_sorted_task(dt->queue_array[best_workerid], task);
+		ret =starpu_st_fifo_taskq_push_sorted_task(&dt->queue_array[best_workerid], task);
 		if(dt->num_priorities != -1)
 		{
 			int i;
-			int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+			int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 			for(i = 0; i <= task_prio; i++)
-				dt->queue_array[best_workerid]->ntasks_per_priority[i]++;
+				dt->queue_array[best_workerid].ntasks_per_priority[i]++;
 		}
 
 
@@ -348,9 +411,9 @@ static int push_task_on_best_worker(struct starpu_task *task, int best_workerid,
 	else
 	{
 		starpu_worker_lock(best_workerid);
-		starpu_task_list_push_back (&dt->queue_array[best_workerid]->taskq, task);
-		dt->queue_array[best_workerid]->ntasks++;
-		dt->queue_array[best_workerid]->nprocessed++;
+		starpu_task_list_push_back (&dt->queue_array[best_workerid].taskq, task);
+		dt->queue_array[best_workerid].ntasks++;
+		dt->queue_array[best_workerid].nprocessed++;
 #if !defined(STARPU_NON_BLOCKING_DRIVERS) || defined(STARPU_SIMGRID)
 		starpu_wake_worker_locked(best_workerid);
 #endif
@@ -368,15 +431,15 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 						unsigned nworkers,
 						double local_task_length[nworkers][STARPU_MAXIMPLEMENTATIONS],
 						double exp_end[nworkers][STARPU_MAXIMPLEMENTATIONS],
-						double *max_exp_endp,
-						double *best_exp_endp,
+						double *max_exp_endp_of_workers,
+						double *min_exp_endp_of_task,
 						double local_data_penalty[nworkers][STARPU_MAXIMPLEMENTATIONS],
 						double local_energy[nworkers][STARPU_MAXIMPLEMENTATIONS],
 						int *forced_worker, int *forced_impl, unsigned sched_ctx_id, unsigned sorted_decision)
 {
 	int calibrating = 0;
-	double max_exp_end = DBL_MIN;
-	double best_exp_end = DBL_MAX;
+	double max_exp_end_of_workers = DBL_MIN;
+	double best_exp_end_of_task = DBL_MAX;
 	int ntasks_best = -1;
 	int nimpl_best = 0;
 	double ntasks_best_end = 0.0;
@@ -391,7 +454,7 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
 	if(sorted_decision && dt->num_priorities != -1)
-		task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, sched_ctx_id);
+		task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, sched_ctx_id);
 
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
 	double now = starpu_timing_now();
@@ -403,7 +466,7 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 		unsigned nimpl;
 		unsigned impl_mask;
 		unsigned workerid = workers->get_next(workers, &it);
-		struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
+		struct starpu_st_fifo_taskq *fifo = &dt->queue_array[workerid];
 		struct starpu_perfmodel_arch* perf_arch = starpu_worker_get_perf_archtype(workerid, sched_ctx_id);
 		unsigned memory_node = starpu_worker_get_memory_node(workerid);
 
@@ -423,7 +486,7 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 				continue;
 			}
 
-			int fifo_ntasks = fifo->ntasks;
+			int fifo_ntasks = fifo->ntasks + fifo->pipeline_ntasks;
 			double prev_exp_len = fifo->exp_len;
 			/* consider the priority of the task when deciding on which workerid to schedule,
 			   compute the expected_end of the task if it is inserted before other tasks already scheduled */
@@ -437,14 +500,14 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 				else
 				{
 					starpu_worker_lock(workerid);
-					prev_exp_len = _starpu_fifo_get_exp_len_prev_task_list(fifo, task, workerid, nimpl, &fifo_ntasks);
+					prev_exp_len = starpu_st_fifo_taskq_get_exp_len_prev_task_list(fifo, task, workerid, nimpl, &fifo_ntasks);
 					starpu_worker_unlock(workerid);
 				}
 			}
 
 			exp_end[worker_ctx][nimpl] = exp_start + prev_exp_len;
-			if (exp_end[worker_ctx][nimpl] > max_exp_end)
-				max_exp_end = exp_end[worker_ctx][nimpl];
+			if (exp_end[worker_ctx][nimpl] > max_exp_end_of_workers)
+				max_exp_end_of_workers = exp_end[worker_ctx][nimpl];
 
 			//_STARPU_DEBUG("Scheduler dmda: task length (%lf) workerid (%u) kernel (%u) \n", local_task_length[workerid][nimpl],workerid,nimpl);
 
@@ -460,11 +523,11 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 			}
 			else
 			{
-				local_task_length[worker_ctx][nimpl] = starpu_task_expected_length(task, perf_arch, nimpl);
+				local_task_length[worker_ctx][nimpl] = starpu_task_worker_expected_length(task, workerid, sched_ctx_id, nimpl);
 				if (local_data_penalty)
 					local_data_penalty[worker_ctx][nimpl] = starpu_task_expected_data_transfer_time_for(task, workerid);
 				if (local_energy)
-					local_energy[worker_ctx][nimpl] = starpu_task_expected_energy(task, perf_arch,nimpl);
+					local_energy[worker_ctx][nimpl] = starpu_task_worker_expected_energy(task, workerid, sched_ctx_id,nimpl);
 				double conversion_time = starpu_task_expected_conversion_time(task, perf_arch, nimpl);
 				if (conversion_time > 0.0)
 					local_task_length[worker_ctx][nimpl] += conversion_time;
@@ -528,10 +591,10 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 
 			exp_end[worker_ctx][nimpl] = task_starting_time + local_task_length[worker_ctx][nimpl];
 
-			if (exp_end[worker_ctx][nimpl] < best_exp_end)
+			if (exp_end[worker_ctx][nimpl] < best_exp_end_of_task)
 			{
 				/* a better solution was found */
-				best_exp_end = exp_end[worker_ctx][nimpl];
+				best_exp_end_of_task = exp_end[worker_ctx][nimpl];
 				nimpl_best = nimpl;
 			}
 
@@ -553,8 +616,8 @@ static void compute_all_performance_predictions(struct starpu_task *task,
 	}
 #endif
 
-	*best_exp_endp = best_exp_end;
-	*max_exp_endp = max_exp_end;
+	*min_exp_endp_of_task = best_exp_end_of_task;
+	*max_exp_endp_of_workers = max_exp_end_of_workers;
 }
 
 static double _dmda_push_task(struct starpu_task *task, unsigned prio, unsigned sched_ctx_id, unsigned da, unsigned simulate, unsigned sorted_decision)
@@ -581,10 +644,10 @@ static double _dmda_push_task(struct starpu_task *task, unsigned prio, unsigned
 	double exp_end[nworkers_ctx][STARPU_MAXIMPLEMENTATIONS];
 
 	/* This is the minimum among the exp_end[] matrix */
-	double best_exp_end;
+	double min_exp_end_of_task;
 
 	/* This is the maximum termination time of already-scheduled tasks over all workers */
-	double max_exp_end = 0.0;
+	double max_exp_end_of_workers = 0.0;
 
 	double fitness[nworkers_ctx][STARPU_MAXIMPLEMENTATIONS];
 
@@ -593,8 +656,8 @@ static double _dmda_push_task(struct starpu_task *task, unsigned prio, unsigned
 					    nworkers_ctx,
 					    local_task_length,
 					    exp_end,
-					    &max_exp_end,
-					    &best_exp_end,
+					    &max_exp_end_of_workers,
+					    &min_exp_end_of_task,
 					    da ? local_data_penalty : NULL,
 					    da ? local_energy : NULL,
 					    &forced_best,
@@ -624,18 +687,20 @@ static double _dmda_push_task(struct starpu_task *task, unsigned prio, unsigned
 					continue;
 				}
 				if (da)
-					fitness[worker_ctx][nimpl] = dt->alpha*(exp_end[worker_ctx][nimpl] - best_exp_end)
-						+ dt->beta*(local_data_penalty[worker_ctx][nimpl])
-						+ dt->_gamma*(local_energy[worker_ctx][nimpl]);
+					fitness[worker_ctx][nimpl] = dt->alpha * __s_alpha__value *(exp_end[worker_ctx][nimpl] - min_exp_end_of_task)
+						+ dt->beta * __s_beta__value *(local_data_penalty[worker_ctx][nimpl])
+						+ dt->_gamma * __s_gamma__value *(local_energy[worker_ctx][nimpl]);
 				else
-					fitness[worker_ctx][nimpl] = exp_end[worker_ctx][nimpl] - best_exp_end;
+					fitness[worker_ctx][nimpl] = exp_end[worker_ctx][nimpl] - min_exp_end_of_task;
 
-				if (da && exp_end[worker_ctx][nimpl] > max_exp_end)
+				if (da && exp_end[worker_ctx][nimpl] > max_exp_end_of_workers)
 				{
 					/* This placement will make the computation
 					 * longer, take into account the idle
 					 * consumption of other cpus */
-					fitness[worker_ctx][nimpl] += dt->_gamma * dt->idle_power * (exp_end[worker_ctx][nimpl] - max_exp_end) / 1000000.0;
+					fitness[worker_ctx][nimpl] += dt->_gamma * __s_gamma__value * dt->idle_power * __s_idle_power__value * (exp_end[worker_ctx][nimpl] - max_exp_end_of_workers) / 1000000.0; /* Since gamma is the cost in us of one Joules,
+																									  then  d->idle_power * (exp_end - max_exp_end_of_workers)
+																									  must be in Joules, thus the / 1000000.0 */
 				}
 
 				if (best == -1 || fitness[worker_ctx][nimpl] < best_fitness)
@@ -646,7 +711,7 @@ static double _dmda_push_task(struct starpu_task *task, unsigned prio, unsigned
 					best_in_ctx = worker_ctx;
 					selected_impl = nimpl;
 
-					//_STARPU_DEBUG("best fitness (worker %d) %e = alpha*(%e) + beta(%e) +gamma(%e)\n", worker, best_fitness, exp_end[worker][nimpl] - best_exp_end, local_data_penalty[worker][nimpl], local_energy[worker][nimpl]);
+					//_STARPU_DEBUG("best fitness (worker %d) %e = alpha*(%e) + beta(%e) +gamma(%e)\n", worker, best_fitness, exp_end[worker][nimpl] - min_exp_end_of_task, local_data_penalty[worker][nimpl], local_energy[worker][nimpl]);
 
 				}
 			}
@@ -758,19 +823,16 @@ static void dmda_add_workers(unsigned sched_ctx_id, int *workerids, unsigned nwo
 	unsigned i;
 	for (i = 0; i < nworkers; i++)
 	{
-		struct _starpu_fifo_taskq *q;
+		struct starpu_st_fifo_taskq *q;
 		int workerid = workerids[i];
 		/* if the worker has alreadry belonged to this context
 		   the queue and the synchronization variables have been already initialized */
-		q = dt->queue_array[workerid];
-		if(q == NULL)
-		{
-			q = dt->queue_array[workerid] = _starpu_create_fifo();
-			/* These are only stats, they can be read with races */
-			STARPU_HG_DISABLE_CHECKING(q->exp_start);
-			STARPU_HG_DISABLE_CHECKING(q->exp_len);
-			STARPU_HG_DISABLE_CHECKING(q->exp_end);
-		}
+		q = &dt->queue_array[workerid];
+		starpu_st_fifo_taskq_init(q);
+		/* These are only stats, they can be read with races */
+		STARPU_HG_DISABLE_CHECKING(q->exp_start);
+		STARPU_HG_DISABLE_CHECKING(q->exp_len);
+		STARPU_HG_DISABLE_CHECKING(q->exp_end);
 
 		if(dt->num_priorities != -1)
 		{
@@ -794,16 +856,10 @@ static void dmda_remove_workers(unsigned sched_ctx_id, int *workerids, unsigned
 	for (i = 0; i < nworkers; i++)
 	{
 		int workerid = workerids[i];
-		if(dt->queue_array[workerid] != NULL)
+		if(dt->num_priorities != -1)
 		{
-			if(dt->num_priorities != -1)
-			{
-				free(dt->queue_array[workerid]->exp_len_per_priority);
-				free(dt->queue_array[workerid]->ntasks_per_priority);
-			}
-
-			_starpu_destroy_fifo(dt->queue_array[workerid]);
-			dt->queue_array[workerid] = NULL;
+			free(dt->queue_array[workerid].exp_len_per_priority);
+			free(dt->queue_array[workerid].ntasks_per_priority);
 		}
 	}
 }
@@ -815,21 +871,16 @@ static void initialize_dmda_policy(unsigned sched_ctx_id)
 
 	starpu_sched_ctx_set_policy_data(sched_ctx_id, (void*)dt);
 
-	_STARPU_MALLOC(dt->queue_array, STARPU_NMAXWORKERS*sizeof(struct _starpu_fifo_taskq*));
-
-	int i;
-	for(i = 0; i < STARPU_NMAXWORKERS; i++)
-		dt->queue_array[i] = NULL;
-
-	dt->alpha = starpu_get_env_float_default("STARPU_SCHED_ALPHA", _STARPU_SCHED_ALPHA_DEFAULT);
-	dt->beta = starpu_get_env_float_default("STARPU_SCHED_BETA", _STARPU_SCHED_BETA_DEFAULT);
+	dt->alpha = starpu_getenv_float_default("STARPU_SCHED_ALPHA", _STARPU_SCHED_ALPHA_DEFAULT);
+	dt->beta = starpu_getenv_float_default("STARPU_SCHED_BETA", _STARPU_SCHED_BETA_DEFAULT);
 	/* data->_gamma: cost of one Joule in us. If gamma is set to 10^6, then one Joule cost 1s */
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 	if (starpu_getenv("STARPU_SCHED_GAMMA"))
 		_STARPU_DISP("Warning: STARPU_SCHED_GAMMA was used, but --enable-blocking-drivers configuration was not set, CPU cores will not actually be sleeping\n");
 #endif
-	dt->_gamma = starpu_get_env_float_default("STARPU_SCHED_GAMMA", _STARPU_SCHED_GAMMA_DEFAULT);
-	dt->idle_power = starpu_get_env_float_default("STARPU_IDLE_POWER", 0.0);
+	dt->_gamma = starpu_getenv_float_default("STARPU_SCHED_GAMMA", _STARPU_SCHED_GAMMA_DEFAULT);
+	/* data->idle_power: Idle power of the whole machine in Watt */
+	dt->idle_power = starpu_getenv_float_default("STARPU_IDLE_POWER", 0.0);
 
 	if(starpu_sched_ctx_min_priority_is_set(sched_ctx_id) != 0 && starpu_sched_ctx_max_priority_is_set(sched_ctx_id) != 0)
 		dt->num_priorities = starpu_sched_ctx_get_max_priority(sched_ctx_id) - starpu_sched_ctx_get_min_priority(sched_ctx_id) + 1;
@@ -871,7 +922,6 @@ static void deinitialize_dmda_policy(unsigned sched_ctx_id)
 	}
 #endif
 
-	free(dt->queue_array);
 	free(dt);
 }
 
@@ -882,7 +932,7 @@ static void dmda_pre_exec_hook(struct starpu_task *task, unsigned sched_ctx_id)
 {
 	unsigned workerid = starpu_worker_get_id_check();
 	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
+	struct starpu_st_fifo_taskq *fifo = &dt->queue_array[workerid];
 	const double now = starpu_timing_now();
 
 	/* Once the task is executing, we can update the predicted amount
@@ -901,12 +951,11 @@ static void dmda_pre_exec_hook(struct starpu_task *task, unsigned sched_ctx_id)
 static void _dm_push_task_notify(struct starpu_task *task, int workerid, int perf_workerid, unsigned sched_ctx_id, int da)
 {
 	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
-	/* Compute the expected penality */
-	struct starpu_perfmodel_arch *perf_arch = starpu_worker_get_perf_archtype(perf_workerid, sched_ctx_id);
+	struct starpu_st_fifo_taskq *fifo = &dt->queue_array[workerid];
 
-	double predicted = starpu_task_expected_length(task, perf_arch,
-						       starpu_task_get_implementation(task));
+	/* Compute the expected penalty */
+	double predicted = starpu_task_worker_expected_length(task, perf_workerid, sched_ctx_id,
+							      starpu_task_get_implementation(task));
 	double predicted_transfer = NAN;
 
 	if (da)
@@ -916,6 +965,9 @@ static void _dm_push_task_notify(struct starpu_task *task, int workerid, int per
 
 	/* Update the predictions */
 	starpu_worker_lock(workerid);
+
+	fifo->pipeline_ntasks++;
+
 	/* Sometimes workers didn't take the tasks as early as we expected */
 	fifo->exp_start = isnan(fifo->exp_start) ? now + fifo->pipeline_len : STARPU_MAX(fifo->exp_start, now);
 	fifo->exp_end = fifo->exp_start + fifo->exp_len;
@@ -943,7 +995,7 @@ static void _dm_push_task_notify(struct starpu_task *task, int workerid, int per
 			if(dt->num_priorities != -1)
 			{
 				int i;
-				int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+				int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 				for(i = 0; i <= task_prio; i++)
 					fifo->exp_len_per_priority[i] += predicted_transfer;
 			}
@@ -960,7 +1012,7 @@ static void _dm_push_task_notify(struct starpu_task *task, int workerid, int per
 		if(dt->num_priorities != -1)
 		{
 			int i;
-			int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+			int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 			for(i = 0; i <= task_prio; i++)
 				fifo->exp_len_per_priority[i] += predicted;
 		}
@@ -969,7 +1021,7 @@ static void _dm_push_task_notify(struct starpu_task *task, int workerid, int per
 	if(dt->num_priorities != -1)
 	{
 		int i;
-		int task_prio = _starpu_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
+		int task_prio = starpu_st_normalize_prio(task->priority, dt->num_priorities, task->sched_ctx);
 		for(i = 0; i <= task_prio; i++)
 			fifo->ntasks_per_priority[i]++;
 	}
@@ -993,7 +1045,7 @@ static void dmda_post_exec_hook(struct starpu_task * task, unsigned sched_ctx_id
 {
 	struct _starpu_dmda_data *dt = (struct _starpu_dmda_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 	unsigned workerid = starpu_worker_get_id_check();
-	struct _starpu_fifo_taskq *fifo = dt->queue_array[workerid];
+	struct starpu_st_fifo_taskq *fifo = &dt->queue_array[workerid];
 	starpu_worker_lock_self();
 	_starpu_fifo_task_finished(fifo, task, dt->num_priorities);
 	starpu_worker_unlock_self();
@@ -1011,10 +1063,10 @@ struct starpu_sched_policy _starpu_sched_dm_policy =
 	.pop_task = dmda_pop_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dm",
 	.policy_description = "performance model",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
 
 struct starpu_sched_policy _starpu_sched_dmda_policy =
@@ -1029,10 +1081,10 @@ struct starpu_sched_policy _starpu_sched_dmda_policy =
 	.pop_task = dmda_pop_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dmda",
 	.policy_description = "data-aware performance model",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
 
 struct starpu_sched_policy _starpu_sched_dmda_prio_policy =
@@ -1047,10 +1099,10 @@ struct starpu_sched_policy _starpu_sched_dmda_prio_policy =
 	.pop_task = dmda_pop_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dmdap",
 	.policy_description = "data-aware performance model (priority)",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
 
 struct starpu_sched_policy _starpu_sched_dmda_sorted_policy =
@@ -1065,10 +1117,10 @@ struct starpu_sched_policy _starpu_sched_dmda_sorted_policy =
 	.pop_task = dmda_pop_ready_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dmdas",
 	.policy_description = "data-aware performance model (sorted)",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
 
 struct starpu_sched_policy _starpu_sched_dmda_sorted_decision_policy =
@@ -1083,10 +1135,10 @@ struct starpu_sched_policy _starpu_sched_dmda_sorted_decision_policy =
 	.pop_task = dmda_pop_ready_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dmdasd",
 	.policy_description = "data-aware performance model (sorted decision)",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
 
 struct starpu_sched_policy _starpu_sched_dmda_ready_policy =
@@ -1101,8 +1153,8 @@ struct starpu_sched_policy _starpu_sched_dmda_ready_policy =
 	.pop_task = dmda_pop_ready_task,
 	.pre_exec_hook = dmda_pre_exec_hook,
 	.post_exec_hook = dmda_post_exec_hook,
-	.pop_every_task = dmda_pop_every_task,
 	.policy_name = "dmdar",
 	.policy_description = "data-aware performance model (ready)",
 	.worker_type = STARPU_WORKER_LIST,
+	.prefetches = 1,
 };
