@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016,2017                                Inria
- * Copyright (C) 2016-2019                                CNRS
- * Copyright (C) 2016-2018                                Université de Bordeaux
+ * Copyright (C) 2016-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,9 +27,11 @@ static const intptr_t fstarpu_w	= STARPU_W;
 static const intptr_t fstarpu_rw	= STARPU_RW;
 static const intptr_t fstarpu_scratch	= STARPU_SCRATCH;
 static const intptr_t fstarpu_redux	= STARPU_REDUX;
+static const intptr_t fstarpu_mpi_redux	= STARPU_MPI_REDUX;
 static const intptr_t fstarpu_commute	= STARPU_COMMUTE;
 static const intptr_t fstarpu_ssend	= STARPU_SSEND;
 static const intptr_t fstarpu_locality	= STARPU_LOCALITY;
+static const intptr_t fstarpu_nofootprint	= STARPU_NOFOOTPRINT;
 
 static const intptr_t fstarpu_data_array	= STARPU_DATA_ARRAY;
 static const intptr_t fstarpu_data_mode_array	= STARPU_DATA_MODE_ARRAY;
@@ -41,11 +41,15 @@ static const intptr_t fstarpu_task_deps_array	= STARPU_TASK_DEPS_ARRAY;
 static const intptr_t fstarpu_task_end_deps_array	= STARPU_TASK_END_DEPS_ARRAY;
 static const intptr_t fstarpu_callback	= STARPU_CALLBACK;
 static const intptr_t fstarpu_callback_with_arg	= STARPU_CALLBACK_WITH_ARG;
+static const intptr_t fstarpu_callback_with_arg_nfree	= STARPU_CALLBACK_WITH_ARG_NFREE;
 static const intptr_t fstarpu_callback_arg	= STARPU_CALLBACK_ARG;
+static const intptr_t fstarpu_callback_arg_nfree= STARPU_CALLBACK_ARG_NFREE;
 static const intptr_t fstarpu_prologue_callback	= STARPU_PROLOGUE_CALLBACK;
-static const intptr_t fstarpu_prologue_callback_arg	= STARPU_PROLOGUE_CALLBACK_ARG;
-static const intptr_t fstarpu_prologue_callback_pop	= STARPU_PROLOGUE_CALLBACK_POP;
-static const intptr_t fstarpu_prologue_callback_pop_arg	= STARPU_PROLOGUE_CALLBACK_POP_ARG;
+static const intptr_t fstarpu_prologue_callback_arg	  = STARPU_PROLOGUE_CALLBACK_ARG;
+static const intptr_t fstarpu_prologue_callback_arg_nfree = STARPU_PROLOGUE_CALLBACK_ARG_NFREE;
+static const intptr_t fstarpu_prologue_callback_pop	  	= STARPU_PROLOGUE_CALLBACK_POP;
+static const intptr_t fstarpu_prologue_callback_pop_arg	  	= STARPU_PROLOGUE_CALLBACK_POP_ARG;
+static const intptr_t fstarpu_prologue_callback_pop_arg_nfree	 = STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE;
 static const intptr_t fstarpu_priority	= STARPU_PRIORITY;
 static const intptr_t fstarpu_execute_on_node	= STARPU_EXECUTE_ON_NODE;
 static const intptr_t fstarpu_execute_on_data	= STARPU_EXECUTE_ON_DATA;
@@ -63,6 +67,13 @@ static const intptr_t fstarpu_handles_sequential_consistency	= STARPU_HANDLES_SE
 static const intptr_t fstarpu_task_end_dep	= STARPU_TASK_END_DEP;
 static const intptr_t fstarpu_task_synchronous	= STARPU_TASK_SYNCHRONOUS;
 static const intptr_t fstarpu_node_selection_policy	= STARPU_NODE_SELECTION_POLICY;
+static const intptr_t fstarpu_task_workerids = STARPU_TASK_WORKERIDS;
+static const intptr_t fstarpu_sequential_consistency = STARPU_SEQUENTIAL_CONSISTENCY;
+static const intptr_t fstarpu_task_profiling_info = STARPU_TASK_PROFILING_INFO;
+static const intptr_t fstarpu_task_no_submitorder = STARPU_TASK_NO_SUBMITORDER;
+static const intptr_t fstarpu_task_sched_data = STARPU_TASK_SCHED_DATA;
+static const intptr_t fstarpu_task_file = STARPU_TASK_FILE;
+static const intptr_t fstarpu_task_line = STARPU_TASK_LINE;
 
 static const intptr_t fstarpu_value = STARPU_VALUE;
 static const intptr_t fstarpu_sched_ctx = STARPU_SCHED_CTX;
@@ -70,8 +81,8 @@ static const intptr_t fstarpu_sched_ctx = STARPU_SCHED_CTX;
 static const intptr_t fstarpu_cpu_worker = STARPU_CPU_WORKER;
 static const intptr_t fstarpu_cuda_worker = STARPU_CUDA_WORKER;
 static const intptr_t fstarpu_opencl_worker = STARPU_OPENCL_WORKER;
-static const intptr_t fstarpu_mic_worker = STARPU_MIC_WORKER;
 static const intptr_t fstarpu_any_worker = STARPU_ANY_WORKER;
+static const intptr_t fstarpu_narch = STARPU_NARCH;
 
 static const intptr_t fstarpu_nmaxbufs = STARPU_NMAXBUFS;
 
@@ -89,13 +100,25 @@ static const intptr_t fstarpu_starpu_nowhere	= STARPU_NOWHERE;
 static const intptr_t fstarpu_starpu_cpu	= STARPU_CPU;
 static const intptr_t fstarpu_starpu_cuda	= STARPU_CUDA;
 static const intptr_t fstarpu_starpu_opencl	= STARPU_OPENCL;
-static const intptr_t fstarpu_starpu_mic	= STARPU_MIC;
 
 static const intptr_t fstarpu_starpu_codelet_simgrid_execute	= STARPU_CODELET_SIMGRID_EXECUTE;
 static const intptr_t fstarpu_starpu_codelet_simgrid_execute_and_inject	= STARPU_CODELET_SIMGRID_EXECUTE_AND_INJECT;
 static const intptr_t fstarpu_starpu_cuda_async	= STARPU_CUDA_ASYNC;
 static const intptr_t fstarpu_starpu_opencl_async	= STARPU_OPENCL_ASYNC;
 
+//static const intptr_t fstarpu_per_worker	= STARPU_PER_WORKER;
+//static const intptr_t fstarpu_per_arch		= STARPU_PER_ARCH;
+//static const intptr_t fstarpu_per_common	= STARPU_COMMON;
+static const intptr_t fstarpu_history_based	= STARPU_HISTORY_BASED;
+static const intptr_t fstarpu_regression_based	= STARPU_REGRESSION_BASED;
+static const intptr_t fstarpu_nl_regression_based	= STARPU_NL_REGRESSION_BASED;
+static const intptr_t fstarpu_multiple_regression_based	= STARPU_MULTIPLE_REGRESSION_BASED;
+
+static const intptr_t fstarpu_seq	= STARPU_SEQ;
+static const intptr_t fstarpu_spmd	= STARPU_SPMD;
+static const intptr_t fstarpu_forkjoin	= STARPU_FORKJOIN;
+
+static const intptr_t fstarpu_default_prio	= STARPU_DEFAULT_PRIO;
 
 intptr_t fstarpu_get_constant(char *s)
 {
@@ -104,9 +127,11 @@ intptr_t fstarpu_get_constant(char *s)
 	else if	(!strcmp(s, "FSTARPU_RW"))	{ return fstarpu_rw; }
 	else if	(!strcmp(s, "FSTARPU_SCRATCH"))	{ return fstarpu_scratch; }
 	else if	(!strcmp(s, "FSTARPU_REDUX"))	{ return fstarpu_redux; }
+	else if	(!strcmp(s, "FSTARPU_MPI_REDUX"))	{ return fstarpu_mpi_redux; }
 	else if	(!strcmp(s, "FSTARPU_COMMUTE"))	{ return fstarpu_commute; }
 	else if	(!strcmp(s, "FSTARPU_SSEND"))	{ return fstarpu_ssend; }
 	else if	(!strcmp(s, "FSTARPU_LOCALITY"))	{ return fstarpu_locality; }
+	else if	(!strcmp(s, "FSTARPU_NOFOOTPRINT"))	{ return fstarpu_nofootprint; }
 
 
 	else if	(!strcmp(s, "FSTARPU_DATA_ARRAY"))	{ return fstarpu_data_array; }
@@ -117,11 +142,15 @@ intptr_t fstarpu_get_constant(char *s)
 	else if	(!strcmp(s, "FSTARPU_TASK_END_DEPS_ARRAY"))	{ return fstarpu_task_end_deps_array; }
 	else if	(!strcmp(s, "FSTARPU_CALLBACK"))	{ return fstarpu_callback; }
 	else if	(!strcmp(s, "FSTARPU_CALLBACK_WITH_ARG"))	{ return fstarpu_callback_with_arg; }
-	else if	(!strcmp(s, "FSTARPU_CALLBACK_ARG"))	{ return fstarpu_callback_arg; }
+	else if	(!strcmp(s, "FSTARPU_CALLBACK_WITH_ARG_NFREE"))	{ return fstarpu_callback_with_arg_nfree; }
+	else if	(!strcmp(s, "FSTARPU_CALLBACK_ARG"))		{ return fstarpu_callback_arg; }
+	else if	(!strcmp(s, "FSTARPU_CALLBACK_ARG_NFREE"))	{ return fstarpu_callback_arg_nfree; }
 	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK"))	{ return fstarpu_prologue_callback; }
 	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK_ARG"))	{ return fstarpu_prologue_callback_arg; }
+	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK_ARG_NFREE"))	{ return fstarpu_prologue_callback_arg_nfree; }
 	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK_POP"))	{ return fstarpu_prologue_callback_pop; }
 	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK_POP_ARG"))	{ return fstarpu_prologue_callback_pop_arg; }
+	else if	(!strcmp(s, "FSTARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE"))	{ return fstarpu_prologue_callback_pop_arg_nfree; }
 	else if	(!strcmp(s, "FSTARPU_PRIORITY"))	{ return fstarpu_priority; }
 	else if	(!strcmp(s, "FSTARPU_EXECUTE_ON_NODE"))	{ return fstarpu_execute_on_node; }
 	else if	(!strcmp(s, "FSTARPU_EXECUTE_ON_DATA"))	{ return fstarpu_execute_on_data; }
@@ -140,13 +169,20 @@ intptr_t fstarpu_get_constant(char *s)
 	else if (!strcmp(s, "FSTARPU_TASK_COLOR"))	{ return fstarpu_task_color; }
 	else if (!strcmp(s, "FSTARPU_HANDLES_SEQUENTIAL_CONSISTENCY"))	{ return fstarpu_handles_sequential_consistency; }
 	else if (!strcmp(s, "FSTARPU_TASK_END_DEP"))	{ return fstarpu_task_end_dep; }
+	else if (!strcmp(s, "FSTARPU_TASK_WORKERIDS"))	{ return fstarpu_task_workerids; }
 	else if (!strcmp(s, "FSTARPU_TASK_SYNCHRONOUS"))	{ return fstarpu_task_synchronous; }
+	else if (!strcmp(s, "FSTARPU_SEQUENTIAL_CONSISTENCY"))	{ return fstarpu_sequential_consistency; }
+	else if (!strcmp(s, "FSTARPU_TASK_PROFILING_INFO"))	{ return fstarpu_task_profiling_info; }
+	else if (!strcmp(s, "FSTARPU_TASK_NO_SUBMITORDER"))	{ return fstarpu_task_no_submitorder; }
+	else if	(!strcmp(s, "FSTARPU_TASK_SCHED_DATA"))	{ return fstarpu_task_sched_data; }
+	else if	(!strcmp(s, "FSTARPU_TASK_FILE"))	{ return fstarpu_task_file; }
+	else if	(!strcmp(s, "FSTARPU_TASK_LINE"))	{ return fstarpu_task_line; }
 
 	else if (!strcmp(s, "FSTARPU_CPU_WORKER"))	{ return fstarpu_cpu_worker; }
 	else if (!strcmp(s, "FSTARPU_CUDA_WORKER"))	{ return fstarpu_cuda_worker; }
 	else if (!strcmp(s, "FSTARPU_OPENCL_WORKER"))	{ return fstarpu_opencl_worker; }
-	else if (!strcmp(s, "FSTARPU_MIC_WORKER"))	{ return fstarpu_mic_worker; }
 	else if (!strcmp(s, "FSTARPU_ANY_WORKER"))	{ return fstarpu_any_worker; }
+	else if (!strcmp(s, "FSTARPU_NARCH"))	{ return fstarpu_narch; }
 
 	else if (!strcmp(s, "FSTARPU_NMAXBUFS"))	{ return fstarpu_nmaxbufs; }
 
@@ -164,16 +200,31 @@ intptr_t fstarpu_get_constant(char *s)
 	else if (!strcmp(s, "FSTARPU_CPU"))	{ return fstarpu_starpu_cpu; }
 	else if (!strcmp(s, "FSTARPU_CUDA"))	{ return fstarpu_starpu_cuda; }
 	else if (!strcmp(s, "FSTARPU_OPENCL"))	{ return fstarpu_starpu_opencl; }
-	else if (!strcmp(s, "FSTARPU_MIC"))	{ return fstarpu_starpu_mic; }
 
 	else if (!strcmp(s, "FSTARPU_CODELET_SIMGRID_EXECUTE"))	{ return fstarpu_starpu_codelet_simgrid_execute; }
 	else if (!strcmp(s, "FSTARPU_CODELET_SIMGRID_EXECUTE_AND_INJECT"))	{ return fstarpu_starpu_codelet_simgrid_execute_and_inject; }
 	else if (!strcmp(s, "FSTARPU_CUDA_ASYNC"))	{ return fstarpu_starpu_cuda_async; }
 	else if (!strcmp(s, "FSTARPU_OPENCL_ASYNC"))	{ return fstarpu_starpu_opencl_async; }
 
+//	else if (!strcmp(s, "FSTARPU_PER_WORKER"))	{ return fstarpu_per_worker; }
+//	else if (!strcmp(s, "FSTARPU_PER_ARCH"))	{ return fstarpu_per_arch; }
+//	else if (!strcmp(s, "FSTARPU_COMMON"))	{ return fstarpu_per_common; }
+	else if (!strcmp(s, "FSTARPU_HISTORY_BASED"))	{ return fstarpu_history_based; }
+	else if (!strcmp(s, "FSTARPU_REGRESSION_BASED"))	{ return fstarpu_regression_based; }
+	else if (!strcmp(s, "FSTARPU_NL_REGRESSION_BASED"))	{ return fstarpu_nl_regression_based; }
+	else if (!strcmp(s, "FSTARPU_MULTIPLE_REGRESSION_BASED"))	{ return fstarpu_multiple_regression_based; }
+
+	else if (!strcmp(s, "FSTARPU_SEQ"))	{ return fstarpu_seq; }
+	else if (!strcmp(s, "FSTARPU_SPMD"))	{ return fstarpu_spmd; }
+	else if (!strcmp(s, "FSTARPU_FORKJOIN"))	{ return fstarpu_forkjoin; }
+
+	else if (!strcmp(s, "FSTARPU_DEFAULT_PRIO"))	{ return fstarpu_default_prio; }
+
 	else { _STARPU_ERROR("unknown constant"); }
+	return -1;
 }
 
+STARPU_ATTRIBUTE_MALLOC
 struct starpu_conf *fstarpu_conf_allocate(void)
 {
 	struct starpu_conf *conf;
@@ -221,12 +272,6 @@ void fstarpu_conf_set_nopencl(struct starpu_conf *conf, int nopencl)
 	conf->nopencl = nopencl;
 }
 
-void fstarpu_conf_set_nmic(struct starpu_conf *conf, int nmic)
-{
-	STARPU_ASSERT(nmic >= 0 && nmic <= STARPU_NMAXWORKERS);
-	conf->nmic = nmic;
-}
-
 void fstarpu_conf_set_calibrate(struct starpu_conf *conf, int calibrate)
 {
 	STARPU_ASSERT(calibrate == 0 || calibrate == 1);
@@ -244,6 +289,7 @@ void fstarpu_topology_print(void)
 	starpu_topology_print(stderr);
 }
 
+STARPU_ATTRIBUTE_MALLOC
 struct starpu_codelet *fstarpu_codelet_allocate(void)
 {
 	struct starpu_codelet *cl;
@@ -263,6 +309,22 @@ void fstarpu_codelet_set_name(struct starpu_codelet *cl, const char *cl_name)
 	cl->name = cl_name;
 }
 
+void fstarpu_codelet_set_color(struct starpu_codelet *cl, int cl_color)
+{
+	STARPU_ASSERT(cl_color >= 0);
+	cl->color = (unsigned)cl_color;
+}
+
+void fstarpu_codelet_set_model(struct starpu_codelet *cl, struct starpu_perfmodel *cl_perfmodel)
+{
+	cl->model = cl_perfmodel;
+}
+
+void fstarpu_codelet_set_energy_model(struct starpu_codelet *cl, struct starpu_perfmodel *cl_perfmodel)
+{
+	cl->energy_model = cl_perfmodel;
+}
+
 void fstarpu_codelet_add_cpu_func(struct starpu_codelet *cl, void *f_ptr)
 {
 	const size_t max_cpu_funcs = sizeof(cl->cpu_funcs)/sizeof(cl->cpu_funcs[0])-1;
@@ -338,21 +400,6 @@ void fstarpu_codelet_add_opencl_flags(struct starpu_codelet *cl, intptr_t flags)
 	_STARPU_ERROR("fstarpu: too many opencl flags in Fortran codelet");
 }
 
-void fstarpu_codelet_add_mic_func(struct starpu_codelet *cl, void *f_ptr)
-{
-	const size_t max_mic_funcs = sizeof(cl->mic_funcs)/sizeof(cl->mic_funcs[0])-1;
-	unsigned i;
-	for (i = 0; i < max_mic_funcs; i++)
-	{
-		if (cl->mic_funcs[i] == NULL)
-		{
-			cl->mic_funcs[i] = f_ptr;
-			return;
-		}
-	}
-	_STARPU_ERROR("fstarpu: too many mic functions in Fortran codelet");
-}
-
 void fstarpu_codelet_add_buffer(struct starpu_codelet *cl, intptr_t _mode)
 {
 
@@ -401,6 +448,49 @@ void fstarpu_codelet_set_where(struct starpu_codelet *cl, intptr_t where)
 	cl->where = (uint32_t)where;
 }
 
+void fstarpu_codelet_set_type(struct starpu_codelet *cl, intptr_t type_constant)
+{
+	STARPU_ASSERT(type_constant == STARPU_SEQ || type_constant == STARPU_SPMD || type_constant == STARPU_FORKJOIN);
+	cl->type = (int)type_constant;
+}
+
+void fstarpu_codelet_set_max_parallelism(struct starpu_codelet *cl, int max_parallelism)
+{
+	if (max_parallelism >= 1)
+	{
+		cl->max_parallelism = max_parallelism;
+	}
+	else
+	{
+		_STARPU_ERROR("fstarpu: invalid max_parallelism parameter");
+	}
+}
+
+STARPU_ATTRIBUTE_MALLOC
+struct starpu_perfmodel *fstarpu_perfmodel_allocate(void)
+{
+	struct starpu_perfmodel *model;
+	_STARPU_CALLOC(model, 1, sizeof(*model));
+	return model;
+}
+
+void fstarpu_perfmodel_free(struct starpu_perfmodel *model)
+{
+	memset(model, 0, sizeof(*model));
+	free(model);
+}
+
+void fstarpu_perfmodel_set_symbol(struct starpu_perfmodel *model, const char *model_symbol)
+{
+	model->symbol = model_symbol;
+}
+
+void fstarpu_perfmodel_set_type(struct starpu_perfmodel *model, intptr_t type)
+{
+	STARPU_ASSERT(type == fstarpu_history_based || type == fstarpu_regression_based || type == fstarpu_nl_regression_based || type == fstarpu_multiple_regression_based);
+	model->type = type;
+}
+
 void * fstarpu_variable_get_ptr(void *buffers[], int i)
 {
 	return (void *)STARPU_VARIABLE_GET_PTR(buffers[i]);
@@ -534,6 +624,7 @@ void fstarpu_worker_get_type_as_string(intptr_t type, char *dst, size_t maxlen)
 	snprintf(dst, maxlen, "%s", str);
 }
 
+STARPU_ATTRIBUTE_MALLOC
 starpu_data_handle_t *fstarpu_data_handle_array_alloc(int nb)
 {
 	void *ptr;
@@ -551,6 +642,7 @@ void fstarpu_data_handle_array_set(starpu_data_handle_t *handles, int i, starpu_
 	handles[i] = handle;
 }
 
+STARPU_ATTRIBUTE_MALLOC
 struct starpu_data_descr *fstarpu_data_descr_array_alloc(int nb)
 {
 	void *ptr;
@@ -558,6 +650,7 @@ struct starpu_data_descr *fstarpu_data_descr_array_alloc(int nb)
 	return ptr;
 }
 
+STARPU_ATTRIBUTE_MALLOC
 struct starpu_data_descr *fstarpu_data_descr_alloc(void)
 {
 	return fstarpu_data_descr_array_alloc(1);
@@ -584,6 +677,7 @@ void fstarpu_data_descr_set(struct starpu_data_descr *descr, starpu_data_handle_
 	fstarpu_data_descr_array_set(descr, 1, handle, mode);
 }
 
+STARPU_ATTRIBUTE_MALLOC
 struct starpu_data_filter *fstarpu_data_filter_allocate(void)
 {
 	struct starpu_data_filter *filter;
@@ -594,6 +688,7 @@ struct starpu_data_filter *fstarpu_data_filter_allocate(void)
 /* Note: use fstarpu_df_alloc_ prefix instead of fstarpu_data_filter_allocate_ to fit within the
  * Fortran id length limit */
 #define _FSTARPU_DATA_FILTER_ALLOCATOR(name) \
+STARPU_ATTRIBUTE_MALLOC \
 struct starpu_data_filter *fstarpu_df_alloc_##name(void) \
 { \
 	struct starpu_data_filter *filter = fstarpu_data_filter_allocate(); \
