@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,11 +24,31 @@
 
 void starpu_codelet_pack_args(void **arg_buffer, size_t *arg_buffer_size, ...)
 {
+	struct starpu_codelet_pack_arg_data state;
 	va_list varg_list;
+	int arg_type;
+
+	starpu_codelet_pack_arg_init(&state);
 
 	va_start(varg_list, arg_buffer_size);
-	_starpu_codelet_pack_args(arg_buffer, arg_buffer_size, varg_list);
+	while((arg_type = va_arg(varg_list, int)) != 0)
+	{
+		if (arg_type==STARPU_VALUE)
+		{
+			/* We have a constant value: this should be followed by a pointer to the cst value and the size of the constant */
+			void *ptr = va_arg(varg_list, void *);
+			size_t ptr_size = va_arg(varg_list, size_t);
+
+			starpu_codelet_pack_arg(&state, ptr, ptr_size);
+		}
+		else
+		{
+			STARPU_ABORT_MSG("Unrecognized argument %d, did you perhaps forget to end arguments with 0?\n", arg_type);
+		}
+	}
 	va_end(varg_list);
+
+	starpu_codelet_pack_arg_fini(&state, arg_buffer, arg_buffer_size);
 }
 
 void _starpu_codelet_unpack_args_and_copyleft(char *cl_arg, void *_buffer, size_t buffer_size, va_list varg_list)
@@ -36,7 +56,7 @@ void _starpu_codelet_unpack_args_and_copyleft(char *cl_arg, void *_buffer, size_
 	size_t current_arg_offset = 0;
 	int nargs, arg;
 
-     	/* We fill the different pointers with the appropriate arguments */
+	/* We fill the different pointers with the appropriate arguments */
 	memcpy(&nargs, cl_arg, sizeof(nargs));
 	current_arg_offset += sizeof(nargs);
 
@@ -67,7 +87,7 @@ void _starpu_codelet_unpack_args_and_copyleft(char *cl_arg, void *_buffer, size_
 		int current_buffer_offset = 0;
 		memcpy(buffer, (int *)&left, sizeof(left));
 		current_buffer_offset += sizeof(left);
-		for ( ; arg < nargs; arg++)
+		for (; arg < nargs; arg++)
 		{
 			size_t arg_size;
 			memcpy(&arg_size, cl_arg+current_arg_offset, sizeof(arg_size));
@@ -130,6 +150,7 @@ struct starpu_task *_starpu_task_build_v(struct starpu_task *ptask, struct starp
 	return (ret == 0) ? task : NULL;
 }
 
+#undef starpu_task_submit
 int _starpu_task_insert_v(struct starpu_codelet *cl, va_list varg_list)
 {
 	struct starpu_task *task;
@@ -152,6 +173,7 @@ int _starpu_task_insert_v(struct starpu_codelet *cl, va_list varg_list)
 	return ret;
 }
 
+#undef starpu_task_set
 int starpu_task_set(struct starpu_task *task, struct starpu_codelet *cl, ...)
 {
 	va_list varg_list;
@@ -162,6 +184,7 @@ int starpu_task_set(struct starpu_task *task, struct starpu_codelet *cl, ...)
 	return 0;
 }
 
+#undef starpu_task_insert
 int starpu_task_insert(struct starpu_codelet *cl, ...)
 {
 	va_list varg_list;
@@ -173,6 +196,7 @@ int starpu_task_insert(struct starpu_codelet *cl, ...)
 	return ret;
 }
 
+#undef starpu_insert_task
 int starpu_insert_task(struct starpu_codelet *cl, ...)
 {
 	va_list varg_list;
@@ -184,6 +208,7 @@ int starpu_insert_task(struct starpu_codelet *cl, ...)
 	return ret;
 }
 
+#undef starpu_task_build
 struct starpu_task *starpu_task_build(struct starpu_codelet *cl, ...)
 {
 	struct starpu_task *task;
