@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016,2017                      Inria
- * Copyright (C) 2011-2019                                CNRS
- * Copyright (C) 2011-2019                                Université de Bordeaux
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,6 +23,7 @@ void starpu_codelet_pack_arg_init(struct starpu_codelet_pack_arg_data *state)
 {
 	state->arg_buffer = NULL;
 	state->arg_buffer_size = 0;
+	state->arg_buffer_used = 0;
 	state->current_offset = sizeof(int);
 	state->nargs = 0;
 }
@@ -46,6 +45,7 @@ void starpu_codelet_pack_arg(struct starpu_codelet_pack_arg_data *state, const v
 	memcpy(state->arg_buffer+state->current_offset, ptr, ptr_size);
 	state->current_offset += ptr_size;
 	STARPU_ASSERT(state->current_offset <= state->arg_buffer_size);
+	state->arg_buffer_used = state->current_offset;
 	state->nargs++;
 }
 
@@ -62,203 +62,77 @@ void starpu_codelet_pack_arg_fini(struct starpu_codelet_pack_arg_data *state, vo
 	}
 
 	*cl_arg = state->arg_buffer;
-	*cl_arg_size = state->arg_buffer_size;
+	*cl_arg_size = state->arg_buffer_used;
 }
 
-int _starpu_codelet_pack_args(void **arg_buffer, size_t *arg_buffer_size, va_list varg_list)
+void starpu_codelet_unpack_arg_init(struct starpu_codelet_pack_arg_data *state, void *cl_arg, size_t cl_arg_size)
 {
-	int arg_type;
+	state->arg_buffer = cl_arg;
+	state->arg_buffer_size = cl_arg_size;
+	state->arg_buffer_used = cl_arg_size;
+	state->current_offset = sizeof(int);
+	state->nargs = 0;
+}
 
-	struct starpu_codelet_pack_arg_data state;
-	starpu_codelet_pack_arg_init(&state);
+void starpu_codelet_unpack_arg(struct starpu_codelet_pack_arg_data *state, void *ptr, size_t size)
+{
+	size_t ptr_size;
+	STARPU_ASSERT_MSG(state->current_offset + sizeof(size) <= state->arg_buffer_size, "The unpack brings offset %ld beyond the buffer size (%ld)\n", state->current_offset, (long)state->arg_buffer_size);
+	memcpy((void *)&ptr_size, state->arg_buffer+state->current_offset, sizeof(ptr_size));
+	STARPU_ASSERT_MSG(ptr_size==size, "The given size (%ld) is not the size of the next argument (%ld)\n", size, ptr_size);
+	state->current_offset += sizeof(size);
 
-	while((arg_type = va_arg(varg_list, int)) != 0)
+	STARPU_ASSERT_MSG(state->current_offset + size <= state->arg_buffer_size, "The recorded size (%ld) brings beyond the buffer size (%ld)\n", (long)size, (long)state->arg_buffer_size);
+	memcpy(ptr, state->arg_buffer+state->current_offset, ptr_size);
+	state->current_offset += size;
+
+	state->nargs++;
+}
+
+void starpu_codelet_dup_arg(struct starpu_codelet_pack_arg_data *state, void **ptr, size_t *size)
+{
+	STARPU_ASSERT_MSG(state->current_offset + sizeof(*size) <= state->arg_buffer_size, "The unpack brings offset %ld beyond the buffer size (%ld)\n", state->current_offset, (long)state->arg_buffer_size);
+	memcpy((void*)size, state->arg_buffer+state->current_offset, sizeof(*size));
+	state->current_offset += sizeof(*size);
+
+	STARPU_ASSERT_MSG(state->current_offset + *size <= state->arg_buffer_size, "The recorded size (%ld) brings beyond the buffer size (%ld)\n", *size, (long)state->arg_buffer_size);
+	_STARPU_MALLOC(*ptr, *size);
+	memcpy(*ptr, state->arg_buffer+state->current_offset, *size);
+	state->current_offset += *size;
+
+	state->nargs++;
+}
+
+void starpu_codelet_pick_arg(struct starpu_codelet_pack_arg_data *state, void **ptr, size_t *size)
+{
+	STARPU_ASSERT_MSG(state->current_offset + sizeof(*size) <= state->arg_buffer_size, "The unpack brings offset %ld beyond the buffer size (%ld)\n", state->current_offset, (long)state->arg_buffer_size);
+	memcpy((void*)size, state->arg_buffer+state->current_offset, sizeof(*size));
+	state->current_offset += sizeof(*size);
+
+	STARPU_ASSERT_MSG(state->current_offset + *size <= state->arg_buffer_size, "The recorded size (%ld) brings beyond the buffer size (%ld)\n", (long)(*size), (long)state->arg_buffer_size);
+	*ptr = state->arg_buffer+state->current_offset;
+	state->current_offset += *size;
+
+	state->nargs++;
+}
+
+void starpu_codelet_unpack_arg_fini(struct starpu_codelet_pack_arg_data *state)
+{
+	if (state->current_offset < state->arg_buffer_size)
 	{
-		if (arg_type & STARPU_R || arg_type & STARPU_W || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX)
-		{
-			(void)va_arg(varg_list, starpu_data_handle_t);
-		}
-		else if (arg_type==STARPU_DATA_ARRAY)
-		{
-			(void)va_arg(varg_list, starpu_data_handle_t*);
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_DATA_MODE_ARRAY)
-		{
-			(void)va_arg(varg_list, struct starpu_data_descr*);
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_VALUE)
-		{
-			/* We have a constant value: this should be followed by a pointer to the cst value and the size of the constant */
-			void *ptr = va_arg(varg_list, void *);
-			size_t ptr_size = va_arg(varg_list, size_t);
-
-			starpu_codelet_pack_arg(&state, ptr, ptr_size);
-		}
-		else if (arg_type==STARPU_CL_ARGS)
-		{
-			(void)va_arg(varg_list, void *);
-			(void)va_arg(varg_list, size_t);
-		}
-		else if (arg_type==STARPU_CL_ARGS_NFREE)
-		{
-			(void)va_arg(varg_list, void *);
-			(void)va_arg(varg_list, size_t);
-		}
-		else if (arg_type==STARPU_TASK_DEPS_ARRAY)
-		{
-			(void)va_arg(varg_list, unsigned);
-			(void)va_arg(varg_list, struct starpu_task **);
-		}
-		else if (arg_type==STARPU_TASK_END_DEPS_ARRAY)
-		{
-			(void)va_arg(varg_list, unsigned);
-			(void)va_arg(varg_list, struct starpu_task **);
-		}
-		else if (arg_type==STARPU_CALLBACK)
-		{
-			(void)va_arg(varg_list, _starpu_callback_func_t);
-		}
-		else if (arg_type==STARPU_CALLBACK_WITH_ARG)
-		{
-			va_arg(varg_list, _starpu_callback_func_t);
-			va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_CALLBACK_WITH_ARG_NFREE)
-		{
-			va_arg(varg_list, _starpu_callback_func_t);
-			va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_CALLBACK_ARG)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_CALLBACK_ARG_NFREE)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK)
-		{
-			va_arg(varg_list, _starpu_callback_func_t);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG_NFREE)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP)
-		{
-			va_arg(varg_list, _starpu_callback_func_t);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else if (arg_type==STARPU_PRIORITY)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_EXECUTE_ON_NODE)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_EXECUTE_ON_DATA)
-		{
-			(void)va_arg(varg_list, starpu_data_handle_t);
-		}
-		else if (arg_type==STARPU_EXECUTE_WHERE)
-		{
-			(void)va_arg(varg_list, unsigned long long);
-		}
-		else if (arg_type==STARPU_EXECUTE_ON_WORKER)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_WORKER_ORDER)
-		{
-			va_arg(varg_list, unsigned);
-		}
-		else if (arg_type==STARPU_SCHED_CTX)
-		{
-			(void)va_arg(varg_list, unsigned);
-		}
-		else if (arg_type==STARPU_HYPERVISOR_TAG)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_POSSIBLY_PARALLEL)
-		{
-			(void)va_arg(varg_list, unsigned);
-		}
-		else if (arg_type==STARPU_FLOPS)
-		{
-			(void)va_arg(varg_list, double);
-		}
-		else if (arg_type==STARPU_TAG || arg_type==STARPU_TAG_ONLY)
-		{
-			(void)va_arg(varg_list, starpu_tag_t);
-		}
-		else if (arg_type==STARPU_NAME)
-		{
-			(void)va_arg(varg_list, const char *);
-		}
-		else if (arg_type==STARPU_NODE_SELECTION_POLICY)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_TASK_COLOR)
-                {
-                        (void)va_arg(varg_list, int);
-                }
-		else if (arg_type==STARPU_TASK_SYNCHRONOUS)
-                {
-                        (void)va_arg(varg_list, int);
-                }
-		else if (arg_type==STARPU_HANDLES_SEQUENTIAL_CONSISTENCY)
-                {
-                        (void)va_arg(varg_list, unsigned char *);
-                }
-		else if (arg_type==STARPU_TASK_END_DEP)
-		{
-			(void)va_arg(varg_list, int);
-		}
-		else if (arg_type==STARPU_TASK_WORKERIDS)
-		{
-			(void)va_arg(varg_list, unsigned);
-			(void)va_arg(varg_list, uint32_t*);
-		}
-		else if (arg_type==STARPU_SEQUENTIAL_CONSISTENCY)
-		{
-			(void)va_arg(varg_list, unsigned);
-		}
-		else if (arg_type==STARPU_TASK_PROFILING_INFO)
-		{
-			(void)va_arg(varg_list, struct starpu_profiling_task_info *);
-		}
-		else if (arg_type==STARPU_TASK_NO_SUBMITORDER)
-		{
-			(void)va_arg(varg_list, unsigned);
-		}
-		else if (arg_type==STARPU_TASK_SCHED_DATA)
-		{
-			(void)va_arg(varg_list, void *);
-		}
-		else
-		{
-			STARPU_ABORT_MSG("Unrecognized argument %d, did you perhaps forget to end arguments with 0?\n", arg_type);
-		}
+		_STARPU_MSG("Arguments still need to be unpacked from the starpu_codelet_pack_arg_data (offset %ld - buffer_size %ld)\n", state->current_offset, (long)state->arg_buffer_size);
 	}
+}
 
-	starpu_codelet_pack_arg_fini(&state, arg_buffer, arg_buffer_size);
-	return 0;
+void starpu_codelet_unpack_discard_arg(struct starpu_codelet_pack_arg_data *state)
+{
+	size_t ptr_size;
+	memcpy((void *)&ptr_size, state->arg_buffer+state->current_offset, sizeof(ptr_size));
+
+	state->current_offset += sizeof(ptr_size);
+	state->current_offset += ptr_size;
+
+	state->nargs++;
 }
 
 void starpu_task_insert_data_make_room(struct starpu_codelet *cl, struct starpu_task *task, int *allocated_buffers, int current_buffer, int room)
@@ -298,21 +172,29 @@ void starpu_task_insert_data_make_room(struct starpu_codelet *cl, struct starpu_
 
 void starpu_task_insert_data_process_arg(struct starpu_codelet *cl, struct starpu_task *task, int *allocated_buffers, int *current_buffer, int arg_type, starpu_data_handle_t handle)
 {
-	enum starpu_data_access_mode mode = (enum starpu_data_access_mode) arg_type & ~STARPU_SSEND;
 	STARPU_ASSERT(cl != NULL);
 	STARPU_ASSERT_MSG(cl->nbuffers == STARPU_VARIABLE_NBUFFERS || *current_buffer < cl->nbuffers, "Too many data passed to starpu_task_insert");
 
 	starpu_task_insert_data_make_room(cl, task, allocated_buffers, *current_buffer, 1);
-
 	STARPU_TASK_SET_HANDLE(task, handle, *current_buffer);
+
+	enum starpu_data_access_mode arg_mode = (enum starpu_data_access_mode) arg_type & ~STARPU_SSEND & ~STARPU_NOFOOTPRINT;
+
+	/* MPI_REDUX should be interpreted as RW|COMMUTE by the "ground" StarPU layer.*/
+	if (arg_mode & STARPU_MPI_REDUX)
+	{
+		arg_mode = STARPU_RW|STARPU_COMMUTE;
+	}
 	if (cl->nbuffers == STARPU_VARIABLE_NBUFFERS || (cl->nbuffers > STARPU_NMAXBUFS && !cl->dyn_modes))
-		STARPU_TASK_SET_MODE(task, mode,* current_buffer);
+	{
+		STARPU_TASK_SET_MODE(task, arg_mode,* current_buffer);
+	}
 	else if (STARPU_CODELET_GET_MODE(cl, *current_buffer))
 	{
-		STARPU_ASSERT_MSG(STARPU_CODELET_GET_MODE(cl, *current_buffer) == mode,
-				"The codelet <%s> defines the access mode %d for the buffer %d which is different from the mode %d given to starpu_task_insert\n",
-				cl->name, STARPU_CODELET_GET_MODE(cl, *current_buffer),
-				*current_buffer, mode);
+		STARPU_ASSERT_MSG((STARPU_CODELET_GET_MODE(cl, *current_buffer) & ~STARPU_NOFOOTPRINT) == arg_mode,
+				  "The codelet <%s> defines the access mode %d for the buffer %d which is different from the mode %d given to starpu_task_insert\n",
+				  _starpu_codelet_get_name(cl), STARPU_CODELET_GET_MODE(cl, *current_buffer),
+				  *current_buffer, arg_mode);
 	}
 	else
 	{
@@ -320,7 +202,7 @@ void starpu_task_insert_data_process_arg(struct starpu_codelet *cl, struct starp
 #  warning shall we print a warning to the user
 		/* Morse uses it to avoid having to set it in the codelet structure */
 #endif
-		STARPU_CODELET_SET_MODE(cl, mode, *current_buffer);
+		STARPU_CODELET_SET_MODE(cl, arg_mode, *current_buffer);
 	}
 
 	(*current_buffer)++;
@@ -361,7 +243,7 @@ void starpu_task_insert_data_process_mode_array_arg(struct starpu_codelet *cl, s
 		{
 			STARPU_ASSERT_MSG(STARPU_CODELET_GET_MODE(cl, *current_buffer) == descrs[i].mode,
 					"The codelet <%s> defines the access mode %d for the buffer %d which is different from the mode %d given to starpu_task_insert\n",
-					cl->name, STARPU_CODELET_GET_MODE(cl, *current_buffer),
+					_starpu_codelet_get_name(cl), STARPU_CODELET_GET_MODE(cl, *current_buffer),
 					*current_buffer, descrs[i].mode);
 		}
 		else
@@ -394,7 +276,7 @@ int _starpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *ta
 
 	while((arg_type = va_arg(varg_list, int)) != 0)
 	{
-		if (arg_type & STARPU_R || arg_type & STARPU_W || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX)
+		if (arg_type & STARPU_R || arg_type & STARPU_W || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX || arg_type & STARPU_MPI_REDUX)
 		{
 			/* We have an access mode : we expect to find a handle */
 			starpu_data_handle_t handle = va_arg(varg_list, starpu_data_handle_t);
@@ -470,6 +352,15 @@ int _starpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *ta
 			task->callback_arg = va_arg(varg_list, void *);
 			task->callback_arg_free = 0;
 		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK)
+		{
+			task->epilogue_callback_func = va_arg(varg_list, _starpu_callback_func_t);
+		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK_ARG)
+		{
+			task->epilogue_callback_arg = va_arg(varg_list, void *);
+			task->epilogue_callback_arg_free = 1;
+		}
 		else if (arg_type==STARPU_PROLOGUE_CALLBACK)
 		{
 			task->prologue_callback_func = va_arg(varg_list, _starpu_callback_func_t);
@@ -575,17 +466,44 @@ int _starpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *ta
 			(void)va_arg(varg_list, int);
 		}
 		else if (arg_type==STARPU_TASK_COLOR)
-                {
-                        task->color = va_arg(varg_list, int);
-                }
+		{
+			task->color = va_arg(varg_list, int);
+		}
 		else if (arg_type==STARPU_TASK_SYNCHRONOUS)
-                {
-                        task->synchronous = va_arg(varg_list, int);
-                }
+		{
+			task->synchronous = va_arg(varg_list, int);
+		}
 		else if (arg_type==STARPU_HANDLES_SEQUENTIAL_CONSISTENCY)
-                {
-                        task->handles_sequential_consistency = va_arg(varg_list, unsigned char *);
-                }
+		{
+			task->handles_sequential_consistency = va_arg(varg_list, unsigned char *);
+		}
+#ifdef STARPU_BUBBLE
+		else if (arg_type==STARPU_BUBBLE_FUNC)
+		{
+			task->bubble_func = va_arg(varg_list, starpu_bubble_func_t);
+		}
+		else if (arg_type==STARPU_BUBBLE_FUNC_ARG)
+		{
+			task->bubble_func_arg = va_arg(varg_list, void*);
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC)
+		{
+			task->bubble_gen_dag_func = va_arg(varg_list, starpu_bubble_gen_dag_func_t);
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC_ARG)
+		{
+			task->bubble_gen_dag_func_arg = va_arg(varg_list,void*);
+		}
+		else if (arg_type==STARPU_BUBBLE_PARENT)
+		{
+			struct starpu_task *parent = va_arg(varg_list, struct starpu_task *);
+			if (parent)
+			{
+				struct _starpu_job *job = _starpu_get_job_associated_to_task(parent);
+				task->bubble_parent = job->job_id;
+			}
+		}
+#endif
 		else if (arg_type==STARPU_TASK_END_DEP)
 		{
 			int end_dep = va_arg(varg_list, int);
@@ -612,6 +530,19 @@ int _starpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *ta
 		{
 			task->sched_data = va_arg(varg_list, void *);
 		}
+		else if (arg_type==STARPU_TASK_FILE)
+		{
+			task->file = va_arg(varg_list, const char *);
+		}
+		else if (arg_type==STARPU_TASK_LINE)
+		{
+			task->line = va_arg(varg_list, int);
+		}
+		else if (arg_type==STARPU_TRANSACTION)
+		{
+			STARPU_ASSERT_MSG(task->transaction == NULL, "a transaction has already been set");
+			task->transaction = va_arg(varg_list, struct starpu_transaction *);
+		}
 		else
 		{
 			STARPU_ABORT_MSG("Unrecognized argument %d, did you perhaps forget to end arguments with 0?\n", arg_type);
@@ -679,7 +610,8 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 		if (arg_type & STARPU_R
 			|| arg_type & STARPU_W
 			|| arg_type & STARPU_SCRATCH
-			|| arg_type & STARPU_REDUX)
+			|| arg_type & STARPU_REDUX
+			|| arg_type & STARPU_MPI_REDUX)
 		{
 			arg_i++;
 			starpu_data_handle_t handle = arglist[arg_i];
@@ -774,6 +706,17 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 			task->callback_arg = arglist[arg_i];
 			task->callback_arg_free = 0;
 		}
+		else if (arg_type == STARPU_EPILOGUE_CALLBACK)
+		{
+			arg_i++;
+			task->epilogue_callback_func = (_starpu_callback_func_t)arglist[arg_i];
+		}
+		else if (arg_type == STARPU_EPILOGUE_CALLBACK_ARG)
+		{
+			arg_i++;
+			task->epilogue_callback_arg = arglist[arg_i];
+			task->epilogue_callback_arg_free = 1;
+		}
 		else if (arg_type == STARPU_PROLOGUE_CALLBACK)
 		{
 			arg_i++;
@@ -825,9 +768,8 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 		}
 		else if (arg_type == STARPU_EXECUTE_WHERE)
 		{
-			assert(0);
 			arg_i++;
-			unsigned long long where = *(unsigned long long *)arglist[arg_i];
+			int32_t where = (int32_t)(intptr_t)arglist[arg_i];
 			task->where = where;
 		}
 		else if (arg_type == STARPU_EXECUTE_ON_WORKER)
@@ -892,19 +834,49 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 			(void)arglist[arg_i];
 		}
 		else if (arg_type == STARPU_TASK_COLOR)
-                {
-                        arg_i++;
-                        task->color = *(int *)arglist[arg_i];
-                }
+		{
+			arg_i++;
+			task->color = *(int *)arglist[arg_i];
+		}
 		else if (arg_type == STARPU_TASK_SYNCHRONOUS)
-                {
-                        arg_i++;
-                        task->synchronous = *(int *)arglist[arg_i];
-                }
+		{
+			arg_i++;
+			task->synchronous = *(int *)arglist[arg_i];
+		}
 		else if (arg_type==STARPU_HANDLES_SEQUENTIAL_CONSISTENCY)
-                {
-                        task->handles_sequential_consistency = (unsigned char *)arglist[arg_i];
-                }
+		{
+			task->handles_sequential_consistency = (unsigned char *)arglist[arg_i];
+		}
+#ifdef STARPU_BUBBLE
+		else if (arg_type==STARPU_BUBBLE_FUNC)
+		{
+			arg_i++;
+			task->bubble_func = (starpu_bubble_func_t)arglist[arg_i];
+		}
+		else if (arg_type==STARPU_BUBBLE_FUNC_ARG)
+		{
+			arg_i++;
+			task->bubble_func_arg = (void *)arglist[arg_i];
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC)
+		{
+			arg_i++;
+			task->bubble_gen_dag_func = (starpu_bubble_gen_dag_func_t)arglist[arg_i];
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC_ARG)
+		{
+			arg_i++;
+			task->bubble_gen_dag_func_arg = (void*)arglist[arg_i];
+		}
+		else if (arg_type==STARPU_BUBBLE_PARENT)
+		{
+			arg_i++;
+			struct starpu_task *parent = (struct starpu_task *)arglist[arg_i];
+			struct _starpu_job *job = _starpu_get_job_associated_to_task(parent);
+			task->bubble_parent = job->job_id;
+
+		}
+#endif
 		else if (arg_type==STARPU_TASK_END_DEP)
 		{
 			arg_i++;
@@ -937,6 +909,22 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 			arg_i++;
 			task->sched_data = (void*)arglist[arg_i];
 		}
+		else if (arg_type == STARPU_TASK_FILE)
+		{
+			arg_i++;
+			task->file = arglist[arg_i];
+		}
+		else if (arg_type == STARPU_TASK_LINE)
+		{
+			arg_i++;
+			task->line = *(int *)arglist[arg_i];
+		}
+		else if (arg_type==STARPU_TRANSACTION)
+		{
+			STARPU_ASSERT_MSG(task->transaction == NULL, "a transaction has already been set");
+			arg_i++;
+			task->transaction = arglist[arg_i];
+		}
 		else
 		{
 			STARPU_ABORT_MSG("unknown/unsupported argument %d, did you perhaps forget to end arguments with 0?", arg_type);
@@ -983,6 +971,7 @@ int _fstarpu_task_insert_create(struct starpu_codelet *cl, struct starpu_task *t
 }
 
 /* Fortran interface to task_insert */
+#undef starpu_task_submit
 void fstarpu_task_insert(void **arglist)
 {
 	struct starpu_codelet *cl = arglist[0];
