@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -67,11 +67,12 @@ struct starpu_codelet work =
 static int x;
 static starpu_data_handle_t x_handle, f_handle;
 
-static
-void callback(void *arg)
+static void callback(void *arg)
 {
 	(void)arg;
-	starpu_task_insert(&work, STARPU_W, starpu_data_get_sub_data(f_handle, 1, x), 0);
+	int ret;
+	ret = starpu_task_insert(&work, STARPU_W, starpu_data_get_sub_data(f_handle, 1, x), 0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	starpu_data_release(x_handle);
 }
 
@@ -79,12 +80,23 @@ int main(int argc, char **argv)
 {
         int i, ret;
 	float *f;
+	struct starpu_conf conf;
 
-	ret = starpu_initialize(NULL, &argc, &argv);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	if(starpu_cpu_worker_get_count() == 0) return STARPU_TEST_SKIPPED;
+	if(starpu_cpu_worker_get_count() == 0)
+	{
+		starpu_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
 
 	/* Declare x */
 	starpu_variable_data_register(&x_handle, STARPU_MAIN_RAM, (uintptr_t)&x, sizeof(x));
@@ -144,7 +156,7 @@ int main(int argc, char **argv)
 	    f[X*(N/M)+2] != 3 || f[X*(N/M)+3] != 4)
 		ret = EXIT_FAILURE;
 
-	starpu_free(f);
+	starpu_free_noflag(f, N * sizeof(*f));
 	starpu_shutdown();
 	return ret;
 
