@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,11 +15,11 @@
  */
 
 #include <starpu.h>
-
 #include "../helper.h"
+#include "../variable/increment.h"
 
 /*
- * Call acquire/release in competition with inserting task working on the same data 
+ * Call acquire/release in competition with inserting task working on the same data
  */
 
 #ifdef STARPU_QUICK_CHECK
@@ -30,41 +30,10 @@ static unsigned ntasks = 4000;
 static unsigned ntasks = 40000;
 #endif
 
-#ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], void *_args);
-#endif
-#ifdef STARPU_USE_OPENCL
-extern void increment_opencl(void *buffers[], void *args);
-#endif
-
-void increment_cpu(void *descr[], void *arg)
-{
-	(void)arg;
-	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
-	(*tokenptr)++;
-}
-
-static struct starpu_codelet increment_cl =
-{
-	.modes = { STARPU_RW },
-	.cpu_funcs = {increment_cpu},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {increment_cuda},
-	.cuda_flags = {STARPU_CUDA_ASYNC},
-#endif
-#ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {increment_opencl},
-	.opencl_flags = {STARPU_OPENCL_ASYNC},
-#endif
-	.cpu_funcs_name = {"increment_cpu"},
-	.nbuffers = 1
-};
-
 unsigned token = 0;
 starpu_data_handle_t token_handle;
 
-static
-int increment_token(int synchronous)
+static int increment_token(int synchronous)
 {
 	struct starpu_task *task = starpu_task_create();
         task->synchronous = synchronous;
@@ -73,8 +42,7 @@ int increment_token(int synchronous)
 	return starpu_task_submit(task);
 }
 
-static
-void callback(void *arg)
+static void callback(void *arg)
 {
 	(void)arg;
         starpu_data_release(token_handle);
@@ -84,9 +52,6 @@ void callback(void *arg)
 #  warning TODO add threads
 #endif
 
-#ifdef STARPU_USE_OPENCL
-struct starpu_opencl_program opencl_program;
-#endif
 int main(int argc, char **argv)
 {
 	unsigned i;
@@ -96,11 +61,7 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-#ifdef STARPU_USE_OPENCL
-	ret = starpu_opencl_load_opencl_from_file("tests/datawizard/acquire_release_opencl_kernel.cl",
-						  &opencl_program, NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_load_opencl_from_file");
-#endif
+	increment_load_opencl();
 
 	starpu_variable_data_register(&token_handle, STARPU_MAIN_RAM, (uintptr_t)&token, sizeof(unsigned));
 
@@ -121,10 +82,7 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(token_handle);
 
-#ifdef STARPU_USE_OPENCL
-        ret = starpu_opencl_unload_opencl(&opencl_program);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
-#endif
+	increment_unload_opencl();
 	starpu_shutdown();
 
         FPRINTF(stderr, "Token: %u\n", token);
@@ -139,10 +97,7 @@ enodev:
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
-#ifdef STARPU_USE_OPENCL
-        ret = starpu_opencl_unload_opencl(&opencl_program);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
-#endif
+	increment_unload_opencl();
 	starpu_shutdown();
 	return STARPU_TEST_SKIPPED;
 }
