@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -38,7 +38,7 @@ static void cpu_memcpy(void *descr[], void *cl_arg)
 	if (me == 0)
 	{
 		/* let commute tasks potentially happen */
-		usleep(100000);
+		starpu_usleep(100000);
 		res = STARPU_ATOMIC_ADD(&cnt,1);
 		STARPU_ASSERT(res == 1);
 	}
@@ -61,10 +61,17 @@ int main(void)
 	double *res, *a;
 	unsigned n=100000, i;
 	starpu_data_handle_t res_handle, a_handle;
-	unsigned nb_tasks = 10, worker;
+	unsigned nb_tasks = 10;
 	int ret;
+	struct starpu_conf conf;
 
-	ret = starpu_init(NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -104,7 +111,7 @@ int main(void)
 	}
 
 	/* let commute tasks potentially happen */
-	usleep(100000);
+	starpu_usleep(100000);
 	starpu_data_release(a_handle);
 
 	starpu_task_wait_for_all ();
@@ -113,8 +120,8 @@ enodev:
 	starpu_data_unregister(res_handle);
 	starpu_data_unregister(a_handle);
 
-	starpu_free(res);
-	starpu_free(a);
+	starpu_free_noflag(res, n*sizeof(double));
+	starpu_free_noflag(a, n*sizeof(double));
 
 	starpu_shutdown();
 	return ret == -ENODEV ? STARPU_TEST_SKIPPED : EXIT_SUCCESS;
