@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2012,2015,2017                           CNRS
- * Copyright (C) 2013                                     Université de Bordeaux
+ * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,8 +16,6 @@
 #ifndef MULTIFORMAT_GENERIC_H
 #define MULTIFORMAT_GENERIC_H
 
-#include <starpu_mic.h>
-
 #define NX 16
 
 #ifdef STARPU_USE_CPU
@@ -34,10 +30,6 @@ void cuda_func(void *buffers[], void *args);
 void opencl_func(void *buffers[], void *args);
 #endif /* !STARPU_USE_OPENCL */
 
-#ifdef STARPU_USE_MIC
-starpu_mic_kernel_t mic_func();
-#endif
-
 extern struct starpu_multiformat_data_interface_ops ops;
 /* Counting the calls to the codelets */
 struct stats
@@ -55,11 +47,6 @@ struct stats
 	unsigned int cpu_to_opencl;
 	unsigned int opencl_to_cpu;
 #endif
-#ifdef STARPU_USE_MIC
-	unsigned int mic;
-	unsigned int cpu_to_mic;
-	unsigned int mic_to_cpu;
-#endif
 };
 
 void print_stats(struct stats *);
