@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,7 +40,7 @@ static unsigned k=K_DEF;
  *  - incrementer b
  *  - notify the modification of v
  *  - incrementer c n times
- *  - sync v 
+ *  - sync v
  *
  * At the end, we have to make sure that if we did k iterations,
  *  v == (kn, k, kn)
@@ -121,6 +121,11 @@ int main(int argc, char **argv)
 	k /= 8;
 #endif
 
+#ifdef STARPU_HAVE_VALGRIND_H
+	if(RUNNING_ON_VALGRIND) n = 4;
+	if(RUNNING_ON_VALGRIND) k = 2;
+#endif
+
 	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
