@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2017                                CNRS
- * Copyright (C) 2015,2016                                Université de Bordeaux
- * Copyright (C) 2015                                     Inria
+ * Copyright (C) 2015-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -52,7 +50,7 @@ void callback(void * /*buffers*/[], void * /*cl_arg*/)
 	unsigned val;
 	val = STARPU_ATOMIC_ADD(&nb, 1);
 	FPRINTF(stdout,"callback in (%u)\n", val); fflush(stdout);
-	usleep(SLEEP_FAST);
+	starpu_usleep(SLEEP_FAST);
 	val = STARPU_ATOMIC_ADD(&nb, -1);
 	FPRINTF(stdout,"callback out (%u)\n", val); fflush(stdout);
 }
@@ -62,7 +60,7 @@ void callback_slow(void * /*buffers*/[], void * /*cl_arg*/)
 	unsigned val;
 	val = STARPU_ATOMIC_ADD(&nb_slow, 1);
 	FPRINTF(stdout,"callback_slow in (%u)\n", val); fflush(stdout);
-	usleep(SLEEP_SLOW);
+	starpu_usleep(SLEEP_SLOW);
 	val = STARPU_ATOMIC_ADD(&nb_slow, -1);
 	FPRINTF(stdout,"callback_slow out (%u)\n", val); fflush(stdout);
 }
