@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2010-2013,2015                           CNRS
- * Copyright (C) 2010,2013-2014,2016                      Université de Bordeaux
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,20 +21,6 @@
  * Test using a task with NULL codelet as a synchronization task through task dependencies
  */
 
-void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
-{
-}
-
-static struct starpu_codelet dummy_codelet =
-{
-	.cpu_funcs = {dummy_func},
-	.cuda_funcs = {dummy_func},
-	.opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.model = NULL,
-	.nbuffers = 0
-};
-
 int main(int argc, char **argv)
 {
 	int ret;
@@ -47,25 +31,24 @@ int main(int argc, char **argv)
 
 	/* {A,B,C} -> D -> {E,F}, D is empty */
 	struct starpu_task *taskA = starpu_task_create();
-	taskA->cl = &dummy_codelet;
+	taskA->cl = &starpu_codelet_nop;
 
 	struct starpu_task *taskB = starpu_task_create();
-	taskB->cl = &dummy_codelet;
+	taskB->cl = &starpu_codelet_nop;
 
 	struct starpu_task *taskC = starpu_task_create();
-	taskC->cl = &dummy_codelet;
+	taskC->cl = &starpu_codelet_nop;
 
 	struct starpu_task *taskD = starpu_task_create();
 	taskD->cl = NULL;
 
 	struct starpu_task *taskE = starpu_task_create();
-	taskE->cl = &dummy_codelet;
+	taskE->cl = &starpu_codelet_nop;
 
 	struct starpu_task *taskF = starpu_task_create();
-	taskF->cl = &dummy_codelet;
+	taskF->cl = &starpu_codelet_nop;
 
-	struct starpu_task *tasksABC[3] = {taskA, taskB, taskC};
-	starpu_task_declare_deps_array(taskD, 3, tasksABC);
+	starpu_task_declare_deps(taskD, 3, taskA, taskB, taskC);
 	starpu_task_declare_deps_array(taskE, 1, &taskD);
 	starpu_task_declare_deps_array(taskF, 1, &taskD);
 
