@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +16,7 @@
 
 #include <starpu.h>
 #include "../helper.h"
-
-extern void cuda_host_increment(void *descr[], void *_args);
+#include "../variable/increment.h"
 
 void cpu_increment(void *descr[], void *arg)
 {
@@ -26,17 +25,8 @@ void cpu_increment(void *descr[], void *arg)
 	(*var) += 2;
 }
 
-static struct starpu_codelet my_codelet =
-{
-	.cpu_funcs = {cpu_increment},
-	.cpu_funcs_name = {"cpu_increment"},
-#ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_host_increment},
-	.cuda_flags = {STARPU_CUDA_ASYNC},
-#endif
-	.modes = { STARPU_RW },
-	.nbuffers = 1
-};
+/* Also test that the application can provide its own main function */
+#undef main
 
 int main(void)
 {
@@ -44,21 +34,34 @@ int main(void)
 	int x = 12;
 	int y = 12;
 	int ret, ret1, ret2;
+	struct starpu_conf conf;
 
-	ret = starpu_init(NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+#ifndef STARPU_SIMGRID
+	conf.ncuda = -1;
+#endif
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	starpu_variable_data_register(&data_handles[0], STARPU_MAIN_RAM, (uintptr_t)&x, sizeof(x));
 	starpu_variable_data_register(&data_handles[1], STARPU_MAIN_RAM, (uintptr_t)&y, sizeof(y));
 
-	ret1 = starpu_task_insert(&my_codelet,
+	// We change the cpu function to have a different computation
+	increment_cl.cpu_funcs[0] = cpu_increment;
+
+	ret1 = starpu_task_insert(&increment_cl,
 				  STARPU_EXECUTE_WHERE, STARPU_CPU,
 				  STARPU_RW, data_handles[0],
 				  0);
 	if (ret1 != -ENODEV) STARPU_CHECK_RETURN_VALUE(ret1, "starpu_task_insert");
 
-	ret2 = starpu_task_insert(&my_codelet,
+	ret2 = starpu_task_insert(&increment_cl,
 				  STARPU_EXECUTE_WHERE, STARPU_CUDA,
 				  STARPU_RW, data_handles[1],
 				  0);
