@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -41,22 +41,6 @@ static void callback(void *arg)
 		task->regenerate = 0;
 }
 
-void dummy_func(void *descr[], void *arg)
-{
-	(void)descr;
-	(void)arg;
-}
-
-static struct starpu_codelet dummy_codelet = 
-{
-	.cpu_funcs = {dummy_func},
-	.cuda_funcs = {dummy_func},
-	.opencl_funcs = {dummy_func},
-	.cpu_funcs_name = {"dummy_func"},
-	.model = NULL,
-	.nbuffers = 0
-};
-
 static void parse_args(int argc, char **argv)
 {
 	int c;
@@ -93,7 +77,7 @@ int main(int argc, char **argv)
 		starpu_task_init(&task[i]);
 		cnt[i] = 0;
 
-		task[i].cl = &dummy_codelet;
+		task[i].cl = &starpu_codelet_nop;
 		task[i].regenerate = 1;
 		task[i].detach = 1;
 
@@ -104,7 +88,7 @@ int main(int argc, char **argv)
 	FPRINTF(stderr, "#tasks : %d x %u tasks\n", K, ntasks);
 
 	start = starpu_timing_now();
-	
+
 	for (i = 0; i < K; i++)
 	{
 		ret = starpu_task_submit(&task[i]);
