@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -85,8 +85,7 @@ const char * get_sched_name(int sched_value)
 	return sched_name;
 }
 
-int
-main (void)
+int main(void)
 {
 	const int nb_cpus = starpu_cpu_worker_get_count();
 
