@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2018-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2018-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +25,7 @@
 #if !defined(STARPU_OPENMP)
 int main(void)
 {
-   return STARPU_TEST_SKIPPED;
+	return STARPU_TEST_SKIPPED;
 }
 #else
 __attribute__((constructor))
@@ -58,28 +58,27 @@ void taskloop_callback_wrapper(void *buffers[], void *_args)
 	taskloop_callback(args->begin_i, args->end_i);
 }
 
-int
-main (void)
+int main(void)
 {
 	struct starpu_omp_task_region_attr attr;
 	memset(&attr, 0, sizeof(attr));
 #ifdef STARPU_SIMGRID
-	attr.cl.model         = &starpu_perfmodel_nop;
+	attr.cl.model	      = &starpu_perfmodel_nop;
 #endif
-	attr.cl.flags         = STARPU_CODELET_SIMGRID_EXECUTE;
+	attr.cl.flags	      = STARPU_CODELET_SIMGRID_EXECUTE;
 	attr.cl.cpu_funcs[0]  = taskloop_callback_wrapper;
-	attr.cl_arg           = &attr;
-	attr.cl.where         = STARPU_CPU;
-	attr.if_clause        = 1;
+	attr.cl_arg	      = &attr;
+	attr.cl.where	      = STARPU_CPU;
+	attr.if_clause	      = 1;
 	attr.final_clause     = 0;
 	attr.untied_clause    = 1;
 	attr.mergeable_clause = 0;
 	attr.nogroup_clause   = 0;
-	attr.is_loop          = 0;
-	attr.collapse         = 0;
-	attr.num_tasks        = 5;
+	attr.is_loop	      = 0;
+	attr.collapse	      = 0;
+	attr.num_tasks	      = 5;
 	attr.nb_iterations    = 400;
-	attr.grainsize        = 130;
+	attr.grainsize	      = 130;
 
 	starpu_omp_taskloop_inline_begin(&attr);
 	starpu_omp_taskloop_inline_end(&attr);
