@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2014-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -101,8 +101,7 @@ void parallel_region_f(void *buffers[], void *args)
 	starpu_omp_task_region(&attr);
 }
 
-int
-main (void)
+int main(void)
 {
 	struct starpu_omp_parallel_region_attr attr;
 	memset(&attr, 0, sizeof(attr));
