@@ -189,12 +189,14 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 	starpu_resume();
-	starpu_shutdown();
 	HASH_ITER(hh, models, model, tmp)
 	{
-		free(model->name);
 		HASH_DEL(models, model);
+		starpu_perfmodel_unload_model(&model->model);
+		free(model->name);
+		free(model);
 	}
+	starpu_shutdown();
 	return 0;
 }
 
